/*  NOISE_FILL

  Routine for examining a Magellan EDR file set to look for data areas
  that were zero'd by MHR, and fill them with random data.

  Functional Description:

  Checks SAB header for each burst of data to determine amount of missing
  data.  If above a certain threshold, replace the zeros with random data.

  Inputs:

	The file names contained in the array in the header TAPE_BUFF.H
	The ORBIT_HEADER (FILE_02) and the SAR_DATA (FILE_12) files will
	be accessed for analysis.

 Outputs:

	The routine will replace the data specified in the SAR_DATA file 
	as missing RCDs with the random data values.

  Summary:

	This routine is adapted from Scott Schaffer's VAX FORTRAN program
	to fill the zero EDR data records with psuedo-noise data so SAR
	data will not create undue anomalies when BAQ decoded and processed.
	The program checks the contents of the SAB headers in the SAR_DATA
	file for each burst to determine whether noise filling is needed.
	The number of SAB headers available in the SAR_DATA file is stored
	in the ORBIT_HEADER file.

	The ORBIT_HEADER file (FILE_02) starts with a label and header
	length at the beginning.  The label is 20 bytes, and the header
	length can vary.  Following this file header is the orbit data.
	the count of number of SAB headers is stored as a 
	The SAR_DATA file starts with a label and header length at the
	beginning.  The label length is 20 bytes, and the header length
	is usually 278 bytes, so there is an offset of 298 bytes to the
	start of the first burst of data.  The burst data consists of
	370 bytes of header, followed by a variable amount of SAR data,
	as copied from RCDs from ODRs.  Each burst header contains the
	information necessary to determine if any data in that burst
	has been zero-filled and need to be replaced by psuedo-noise.

	The SAB Quaternary Header, 156 bytes offset from start of burst
	data, contains the required information for checking the SAR data:

	Item	Offset	Offset	Description
		(SOB)	(SABQ)

	FILL	162	6	111 bytes of fill status, one for each
				RCD in this burst.  0=RCD was zero-filled.
	IOFF	284	128	2 bytes counting the number of bytes
				offset from the start of burst data
				group to the first RCD in this burst
	NRCD	288	132	Number of valid RCDs in this burst
	MRCD	289	133	Number of missing RCDs in this burst

	The SAB Header, 316 bytes offset from start of burst data,
	contains information that describes the size of the SAR data:

	NRGL	361	45	Range Gate Length value; multiply by 2
				and subtract from 256 gives number of
				range samples per range pulse.
	NOP	362	46	Number of Pulses; shift SAB[46] up 3 bits
				OR with 3 LSB of SAB[47], and subract from
				2048 gives number of pulses in burst.
*/

#include <stdio.h>
#include <fcntl.h>
#include <errno.h>
extern int errno;

#define o_SABQ 156
#define o_SAB 316
#define o_BDAT 370

#include "tape_buff.h"

unsigned char *nrbuff;
char c_buf[64];
unsigned char brand[16] =
{
	128, 130, 136, 138, 160, 162, 168, 170, 
	0, 2, 8, 10, 32, 34, 40, 42
};

noise_fill(seed)
	int seed;
{
	int sfd, sar_loc_fd, i, j, ncnt, itot, inr, ioff, last, nrcd, mrcd;
	int nrgl, burst, bdat_size, total_bursts, nop, hist[128], fill[111];
	long ipos, rand(), srand(), sar_loc, *lptr;
	FILE *fp, *fopen();
	float plim = { 50.0 };

	/* Open Orbit Header file and find number of bursts in this orbit  */

	/* point to 2nd file name  */
	nrbuff = &(file_names[1] [0]);

	printf("NOISE_FILL openning file %s (ORBIT_HEADER)...\n", nrbuff);
	sfd = open((char *) nrbuff, O_RDONLY);
	if(sfd <= 0)
	{
    	perror("NOISE_FILL error openning Orbit Header FILE");
    	return(-1);
	}

	if((i = read( sfd, buffer, 32500)) != 32500)
	{
    	perror("NOISE_FILL error reading Orbit Header file:");
    	return(-1);
	}
	close(sfd);

	/* Don't need this any more  */
	for(i = 0; i < 20; i++)
	{
		/*  Get file ID  */
		c_buf[i] = buffer[i+54];
	}

	c_buf[20] = 0;

	if(strcmp(c_buf, "ORBIT_HEADER_RECORD.") != 0)
	{
   		printf("NOISE_FILL file ID check failed; expected ORBIT_HEADER_RECORD.\n");
    	printf("	ID read from file %s:  %s\n", nrbuff, c_buf);
    	return(-1);
	}

	for(i = 0; i < 8; i++)
	{
		c_buf[i] = buffer[i+12];
	}

	/*  get header length  */
	c_buf[8] = 0;
	sscanf(c_buf, "%d", &j);
	/* Offset to number of SAB headers  */
	j+=20+161;

	for(i = 0; i < 4; i++)
	{
		/*  get number of SABs  */
		c_buf[i] = buffer[i+j];
	}

	c_buf[4] = 0;
	sscanf(c_buf, "%d", &total_bursts);

	for(i = 0; i < 5; i++)
	{
		/*  get orbit number  */
		c_buf[i] = buffer[i+74];
	}

	c_buf[5] = 0;
	sscanf(c_buf, "%d", &j);
	printf("Total bursts for orbit %d: %d\n", j, total_bursts);

	/* Open SAR Data file  */

	/* point to 12th file name  */
	nrbuff = &(file_names[11][0]);
	printf("NOISE_FILL openning file %s (SAR_DATA)...\n", nrbuff);
	sfd = open((char *) nrbuff, O_RDWR);
	if(sfd <= 0)
	{
    	perror("NOISE_FILL error openning SAR DATA FILE");
    	return(-1);
	}

	/* read some header stuff  */

	if((i = read(sfd, buffer, 128)) != 128)
	{
    	perror("NOISE_FILL error reading file identifier:");
    	return(-1);
	}

	for(i = 0; i < 10; i++)
	{
		/*  get file ID  */
		c_buf[i] = buffer[i+54];
	}

	c_buf[10] = 0;
	if(strcmp(c_buf, "SAR_BURST.") != 0)
	{
   		printf("NOISE_FILL file ID check failed; expected SAR_BURST.\n" );
    	printf("	ID read from file %s:  %s\n", nrbuff, c_buf);
    	return( -1 );
	}

	for(i = 0; i < 8; i++)
	{
		/*  get header length  */
		c_buf[i] = buffer[i+12];
	}
	c_buf[8] = 0;
	/* read length of SAR_DATA file header  */
	sscanf(c_buf, "%d", &ipos);
	/* offset to start of first burst data group  */
	ipos += 20;

	for(i = 0; i < 5; i++)
	{
		/*  get orbit number  */
		c_buf[i] = buffer[i+64];
	}

	c_buf[5] = 0;
	sscanf(c_buf, "%d", &i);
	if(i != j)
	{
    	printf("NOISE_FILL error checking Orbit number consistency\n");
    	printf("Orbit from Orbit Header File: %d\n", j );
    	printf("Orbit from SAR Data file: %d\n", i );
    	return( -1 );
	}

	printf("Orbit Number: %s, First Burst at: %d(%oo)\n", c_buf, ipos, ipos );

	/* Open Log file and start recording stuff  */

	strcpy(c_buf, edr_dir);					/*  get the EDR directory  */
	strcat(c_buf, "/noise_fill.log");		/*  complete the file name  */
	if((fp = fopen(c_buf, "w")) == NULL)
	{
    	printf("Cannot open Log file\n");
    	return( -1 );
	}

	fprintf(fp, "\nNoise fill report for orbit %s (%d total bursts)\n\n",
		edr_dir, total_bursts);

	/* Open the SAR location file */

	/* get the EDR directory  */
	strcpy(c_buf, edr_dir);
	strcat( c_buf, "/sar_location" );

	printf("NOISE_FILL openning SAR location file: %s\n", c_buf);
	sar_loc_fd = open(c_buf, O_RDWR|O_CREAT, 0644);
	if(sar_loc_fd <= 0)
	{
    	perror("NOISE_FILL cannot open SAR Location File");
    	return(-1);
	}

	/* generate some random numbers  */

	nrbuff = buffer + 131072;	/*  point into the big buffer  */
	srand(seed);	/*  initialize with seed  */
	for(i = 0; i < 10000; i++)
	{
		nrbuff[i] = brand[(rand(seed) & 0xf)];
	}

	/*
	printf("NRBUFF:\n");
	for(i = 0; i < 32; i++)
	{
		printf("%2x  ", nrbuff[i]);
	}
	printf("\n");
	*/

	nop = 0;
	nrgl = 0;
	burst = 0;
	inr = 0;
	itot = 0;
	ncnt = 0;

	for(i = 0; i < 128; i++)
	{
		hist[i] = 0;
	}

	while(burst < total_bursts)
	{
    	if((burst % 500) == 0)
		{
			printf("NOISE_FILL at burst %d %d(%oo)\n", burst, ipos, ipos);
		}
    	burst += 1;

		/*  get burst info from burst header  */

		/*  seek to start of burst header  */
    	lseek(sfd, ipos, 0);

    	if((i = read(sfd, buffer, o_BDAT)) != o_BDAT)
		{
			perror("NOISE_FILL error reading SAB header:");
			printf("NOISE_FILL at burst %d, requested %d bytes, got %d\n",
				burst, o_BDAT, i );
			return(-1);
    	}

		/* save SAR location  */

    	sar_loc = ipos + o_BDAT;
    	if((i = write(sar_loc_fd, &sar_loc, 4)) !=4)
		{
			perror("NOISE_FILL error writing SAR location:");
			printf("NOISE_FILL at burst %d, requested 4 bytes, wrote %d\n",
				burst, i );
			return(-1);
    	}

		/* Process if SAB SYNC is valid  */

		/*  initialize SYNC match count  */
    	i = 0;
    	if(buffer[o_SAB]==0x03 ) i++;
    	if(buffer[o_SAB+1]==0x91 ) i++;
    	if(buffer[o_SAB+2]==0x5e ) i++;
    	if(buffer[o_SAB+3]==0xd3 ) i++;

    	if(i > 2)
		{
			/* match at least 3 bytes  */
			nrcd = (int) buffer[o_SABQ+132]; /*  Valid RCDs  */
			mrcd = (int) buffer[o_SABQ+133];		/*  Missing RCDs  */
			nrgl = 2*(256 - (int)buffer[o_SAB+45]);	/*  range gate length  */
			i = (int) buffer[o_SAB+46];
			j = (int) buffer[o_SAB+47];
			nop = 2048 - ( (i<<3) | ((j>>5)&7) );	/*  Number of pulses  */
			bdat_size = nop * nrgl / 2;
			/*
			printf("Burst %d, NRCD=%d, MRCD=%d, NRGL=%d, NOP=%d, %x, %x, %x\n",
				burst, nrcd, mrcd, nrgl, nop, 
			buffer[o_SAB+45], buffer[o_SAB+46], buffer[o_SAB+47] );
			*/

    	}
		else
		{
			/* SAB header SYNC code mismatch */

			printf("NOISE_FILL SAB SYNC code mismatch at burst %d\n", burst);
			printf("	SAB SYNC expected 03 91 5e d3\n");
			printf("	SAB SYNC received %x %x %x %x\n",
			buffer[o_SAB], buffer[o_SAB+1], buffer[o_SAB+2], buffer[o_SAB+3]);
			return(-1);
    	}

		/*  histogram percantage of missing RCDs  */

    	if(mrcd + nrcd > 0)
		{
			itot += 1;
			i = (int) (100.*mrcd / (mrcd+nrcd));
			hist[i] += 1;
    	}

		/* check to see if missing data is within patchable bounds  */

    	if(mrcd > 0 && 100. * mrcd / (mrcd + nrcd) <= plim)
		{
			for(i = 0; i < 111; i++)
			{
				fill[i] = (int) buffer[i+o_SABQ+6];
			}

			ioff = ((int) buffer[o_SABQ+129]) | (((int) buffer[o_SABQ+128]) << 8);
			/*  count number of bursts filled  */
			ncnt += 1;
			fprintf(fp,
				"Burst %4d, POS=%9d, NRCD=%3d, MRCD=%2d, RGL=%3d, NOP=%4d, OFF=%3d\n",
				burst, sar_loc, nrcd, mrcd, nrgl, nop, ioff );

			/*
			printf("Octal position=%o\n", sar_loc );
			*/

			if((i = read(sfd, buffer, bdat_size)) != bdat_size)
			{
	    		perror("NOISE_FILL error reading burst data");
	    		printf("at burst %d, IPOS=%d, %d bytes attempted, %d returned\n",
					burst, ipos, bdat_size, i );
	    		return(-1);
			}

			/* loop through the RCDs for this burst (skip RCD 1)  */

			for(j = 1; j < nrcd + mrcd; j++)
			{
	    		if(fill[j] == 0)
				{
					/*  if RCD was zero filled  */

					/*  Start of RCD  */
					i = ioff + (j-1)*823-108;
					/* last data byte in RCD  */
					last = i + 823;
					if(i < 0)
					{
						i = 0;
					}

					if(last > bdat_size)
					{
						/*  stay in burst  */
						last = bdat_size;
					}

					/*
					printf("Burst=%d, RCD=%d, first=%d, last=%d\n",
						burst, j, i, last);
					*/

					while(i < last)
					{
		    			buffer[i++] = nrbuff[inr++];
		    			inr = inr % 10000;
					}
	    		} /*  end checking this RCD  */
			} /*  end looping through RCDs  */

			lseek(sfd, sar_loc, 0);

			if((i = write(sfd, buffer, bdat_size)) != bdat_size)
			{
	    		perror("NOISE_FILL error rewriting burst:");
	    		printf("NOISE_FILL at burst %d, IPOS=%d, %d\n", burst, ipos, i);
	    		return( -1 );
			}
    	} /*  end checking and filling this burst  */

		/*  increment file pointer  */
    	ipos += o_BDAT + bdat_size;
	}

	close(sfd);

	fprintf(fp,
		"\nMRCD histogram for %d bursts processed from %d total bursts scanned\n\n",
		ncnt, itot);

	fprintf(fp, "%%MRCD\tNumber of bursts\n");

	for(i = 0; i < 50; i++)
	{
		fprintf(fp, "%4d,\t%d\n", i, hist[i] );
	}

	fclose(fp);

	/* fill up remaining SAR locations to 8000 entries with -1  */

	i = 8000 - burst;
	lptr = (long *) buffer;
	for(j = 0; j < i; j++)
	{
		lptr[j] = -1;
	}

	write(sar_loc_fd, lptr, i*4);
	close(sar_loc_fd);

	return(1);
}
