/*  Program for copying files from a MGN SAR EDR tape set to disk files.

  Functional Description:

  Each MGN SAR EDR consists of one or two reels of tapes containing data 
  files in the ANSI level II format.  The organization of the tape is:

	First Record:	Volume Header (80 bytes)
	Second Record:	Data File Header 1 (80 bytes)
	Third Record:	Data File Header 2 (80 bytes)
	Tape Mark (End of file)
	Fourth Record:	First data record
	Fifth Record:	Second data record
				.
				.
				.
	N+4th Record:	Nth data record
	Tape Mark (End of file)
	N+5th Record:	Data File Trailer 1 (80 bytes)
	N+6th Record:	Data File Trailer 2 (End of first data file)
	Tape Mark (End of file)
	N+7th Record:	Data File Header 1 (Start of second data file)
	N+8th Record:	Data File Header 2 (80 bytes)
				.
				.
				.
	Tape Mark (Last data record on volume)
	End of Volume Record (80 bytes)
	Tape Mark
	Tape Mark (Logical EOT)

  Records 2 through N+4+2 constitute one complete data file.  The MGN EDRs 
  will consist of 13 files, some of which are sometimes null, stored on one
  whole reel (sometimes part of a second reel) of 2400 ft CCTs, The data 
  records of each file will be fixed at 32500 bytes.
  This program will copy each file on each tape(s) to designated disk files
  residing in directories provided by the inputs.  The first and last two
  files will go into the EDR directory, while the remaining files will go 
  into the BIDR directory.  (The two directories may be the same.)  A count
  of the number of records read by each file will be kept in part of the
  input buffer and later read by the host.

 Inputs:

	TAPE1:  The name of the tape drive containing the first reel of tape
	TAPE2:  The name of the tape drive containing the second reel of tape
	EDR_DIR:  The directory used to store the EDR header, data, and 
			trailer files
	EDR_ANC:  The directory used to store the EDR ancillary files

 Outputs:

	The program will output either a success if all went well
	or an error code.
*/

#include <sys/ioctl.h>
#include <stdio.h>
#include <errno.h>
extern int errno;

#include "tape_buff.h"

main( argc, argv )
int argc;
char *argv[];
{
unsigned char *fn, tf_id_save[32], tv_id_save[32], junk[80];
int same_drive, in, out, switch_vol, tf_opened, tv_opened, i;
int *rec_count, status, level_h, vol;

/*  printf("This is the UNIX version of the Magellan EDR copy program.\n");  */

rec_count = recs;
for( i=0; i<20; i++ ) rec_count[i] = 0;

if( argc<4 ){		/*  verify inputs  */
    printf("Usage:  copy_edr edr_dir edr_ancillary tape1 [tape2]\n");
    exit(-1);
}

/*  copy inputs to local variables  */

strcpy( edr_dir, argv[1] );
strcpy( edr_anc, argv[2] );
strcpy( tape1, argv[3] );
if( argc>4 ) strcpy( tape2, argv[4] );	/*  second tape drive was given  */
else strcpy( tape2, tape1 );
same_drive = ( ( strcmp( tape1, tape2 )==0 ) && argc<5 );

/*
printf("COPY_EDR received tape names: %s %s\n", tape1, tape2 );
printf("COPY_EDR using directories: %s %s\n", edr_dir, edr_anc );
*/

tv_opened = 0;			/*  opened tape volume flag  */
tf_opened = 0;			/*  opened tape file flag  */
switch_vol = 0;			/*  switching volume flag  */
level_h = 0;			/*  header level  */
out = 0;			/*  initialize disk file descriptor  */
fn = &file_names[0][0];		/*  point to first disk file name  */
rlen = 32500;			/*  logical record length  */
blen = 32500;			/*  physical record length  */
tape = tape1;			/*  point to first tape drive  */
vol = 0;			/*  tape volume number  */

for( i=1; i<=13; ){

/******/
/*  Open tape volume if necessary  */
/******/

    if( tv_opened <= 0 ){

	in = -1;
	while( in==-1 ){
	    if( switch_vol && same_drive ){
		printf("COPY_EDR: Please load next volume on %s\n", tape );
		printf("COPY_EDR: Type carriage return when ready:  \n");
		junk[0] = getchar();
		if( junk[0] == 'z' ) exit( -1 );
	    }

/*	printf("COPY_EDR opening tape drive...%s\n", tape );  */
	    if( (in=open(tape,0))==-1 ){
		perror("COPY_EDR open error:");
		printf("COPY_EDR error opening TAPE drive: %d\n", errno);
	    }
	}
	if( tape == tape1 ) tape = tape2;	/*  swapping tape drives  */
	else tape = tape1;

	tv_opened = open_tape_volume( in, 0 );
	if( tv_opened != 1 )
	    printf("COPY_EDR error opening tape volume: %d\n", tv_opened );
	if( switch_vol && ( strcmp( tv_id, tv_id_save ) != 0 ) ){
	    printf("COPY_EDR error, tape volume label mismatch:\n");
	    printf("COPY_EDR expected %s, received %s\n",
		tv_id_save, tv_id );
/*	    exit( -1 );  */
	}
	strcpy( tv_id_save, tv_id );
/*	printf("COPY_EDR ID: %s\n", tv_id );  */
	vol ++;
	if( vol==1 ){		/*  extract and create directory  */
	    set_dir( tv_id_save, edr_dir, edr_anc, fn, 13, 1 );
	    mkdir( edr_dir, 0777 );
	    if( strcmp( edr_dir, edr_anc )!=0 ) mkdir( edr_anc, 0777 );
	}
    }

/******/
/*  Open the file on tape  */
/******/

    if( tf_opened <=0 ){	/*  open tape file  */
	open_tape_file( in, &level_h, 0 );
	if( level_h>0 ){		/*  HDRs found  */
	    if( switch_vol==0 ){		/*  not switching volume  */
		strcpy( tf_id_save, tf_id );
	    } else if( out>0 ){	/*  check tape file name across volume  */
		if( strcmp( tf_id, tf_id_save ) != 0 ){
		    printf("COPY_EDR error file continuation across volume\n");
		    printf("COPY_EDR expected %s, received %s\n",
			tf_id_save, tf_id );
		    exit( -1 );
		}
		switch_vol = 0;	/*  reset switched volume flag  */
	    }
	    tf_opened = 1;
	} else if( level_h==0 ){	/*  encountered EOV  */
	    if( (status=close_tape_volume( in, 0 ))!=1 ){
		printf("COPY_EDR error unloading tape: %d\n", status );
		exit( -1 );
	    }
	    close( in );
	    tv_opened = 0;
	    switch_vol = 1;
	} else {
	    printf("COPY_EDR error opening tape file %d, code=%d\n",i,level_h);
	    exit( -1 );
	}
    }

/******/
/*  open disk file as necessary  */
/******/

    if( out<=0 && tf_opened==1 ){
	if( ( out=creat( fn, 0666 ) ) == -1 ){
	    printf("COPY_EDR error creating file %d, name=%s\n", i, fn );
	    exit( errno );
	}
/*	else {
	    printf("COPY_EDR opened disk file %d:  %s (ok)\n", i, fn );
	}
*/
    }			/*  end opening disk file  */

/******/
/*  Copy data from tape file into disk file  */
/******/

    if( out>0 && tf_opened==1 && tv_opened==1 ){
	status = copy_records( in, out, buffer, blen, -1, &i_stat, &o_stat );
        if( status<=0 || i_stat!=1 || o_stat!=1 ){
            printf("COPY_EDR error copying file %d i=%d, o=%d, status=%d\n",
                i, i_stat, o_stat, status );
            if( i_stat<0 ){     /*  input error  */
                printf("COPY_EDR error reading from input file \n" );
                exit( -1 );
            }
            if( o_stat<0 ){     /*  output error  */
                printf("COPY_EDR error writing file %d i_stat=%d, status=%d\n",
                i, i_stat, status );
                exit( -1 );
            }
        }
	*rec_count += status;		/*  increment record count  */
    }

/******/
/*  Close tape file and check trailer level  */
/******/

    if( tf_opened ){
	close_tape_file( in, &status, 0 );
	if( status==level_h ){	/*  tape file ended, close disk file  */
	    close( out );
	    out = 0;			/*  clear disk file descriptor  */
	    rec_count++;		/*  next record count  */
	    i += 1;			/*  increment file count  */
	    fn += 64;			/*  point to next file name  */

	} else if( status==0 ){	/*  EOV on tape, rewind, unload, prompt  */
	    status = close_tape_volume( in, 0 );
	    close( in );		/*  close tape fd  */
	    tv_opened = 0;		/*  flag to open new tape volume  */
	    switch_vol = 1;		/*  switch volume flag  */

	} else {			/*  other errors  */
	    printf("COPY_EDR error closing tape file\n");
	    exit( -1 );
	}
	tf_opened = 0;
    }
}
close_tape_volume( in, 0 );
close( in );		/*  close tape unit FD  */

/*  write REC_ROUNT file  */

junk[0] = 0;
strcpy( junk, edr_anc );
strcat( junk, "/REC_COUNT" );
out=creat( junk, 0666 );
write( out, recs, 88 );
close( out );

/*  extract orbit number and build final output message  */

i = strlen( edr_anc );
strcpy(junk, "COPY_EDR orbit: " );
strncat( junk, edr_anc+i-10, 10 );

/*  Do the psuedo-noise fill in the missing RCDs  */

if( noise_fill( 4321 ) != 1 ){
    printf("Error in NOISE_FILL of SAR_DATA file\n");
    exit( -1 );
}

/*  Output the orbit number message  */

printf( "%s\n", junk );

exit( 1 );
}
