c***********************************************************************
c	Jet Propulsion Laboratory
c	Section:	334, Radar Science and Engineering
c	Project:	Magellan
c	Task:		Correlator, Alliant Fortran Version
c-----------------------------------------------------------------------
c	Module Name:	az_comp.f
c
c		Perform the Azimuth compression
c		for the Magellan SAR Data.
c
c	Input:	1). A transposed range compressed data.
c		2). A associated Processing Parameters file.
c		    1.npulses :# of pulses (echoes)			  
c		    2.nsamples:# of samples.
c		    3.prf     :pulse repetition frequency in hz.
c		    4.fd_m    :mid-range doppler freqency in hz.
c		    5.fr_m    :mid-range doppler rate in hz per sec.
c
c	Output: A range compressed data in common buffer with complex form.
c
c	Function:    
c		The azimuth processing include are
c
c		1) Azimuth Compreesion:
c		   The deramp-FFT method is chosen because of its computational
c		   efficiency.
c
c		2).Impulse Response Shaping:
c		   Using a Kaiser weight in time domain to reduce the 
c		   integrated side lobe ratio (ISLR).
c
c		3).Perform a circular shift for each azimuth line:
c	          A circular shift is performed for each azimuth line to place
c		  the Mid-pulse of the burst at the beginning of the FFT buff 
c		  in time domain.
c
c	*** Algorithms designed and original coded by: Mike Jin ***
c----------------------------------------------------------------------------
c       11/29/93 Lawrence Chen : change fft subroutine
c	7/31/92 Anhua Chu Reset kw for concurrent operation
c	3/10/92	Anhua Chu Initial code.
c****************************************************************************
	subroutine az_comp(buff_az,buff_shift)
c       9/17/93 David Swantek Change FFTs from Alliant concurrent to
c               Eugene Chu's FORTRAN CFFT routine for Alliant to Sun Port.
	
	implicit none
	include '../../../inc/keyvar.inc'
	include '../../../inc/pp.inc'
	include '../../../inc/constant.inc'
	include '../../../inc/corr_buffer.inc'

c	PASSING IN PARAMETERS

	complex buff_az(*)	!Input buffer,range compressed data
	complex buff_shift(*)	!Output buffer,azimuth compressed data

c	LOCAL VARIABLES

	complex kw(2048)
	complex buff_opt(2048*512)
	real	r_prf,h_prf,h_npulses,h_fr_m
	integer i,j,k,index_az_rg,azrg_add_hnp,ar_sub_hnp
	real 	taz,d_r_cell,phi,phi1,phi2
	integer nstage,iflag

d	type *,' Enter Azimuth Compression'

c ds	call cfft1d(nfft_az,buff_az,0)		! initial cfft

        call init_fft(nfft_az)    		! Sun port ds

c	do i=1,2048
c	   kw(i)=cmplx(0.0,0.0)
c	enddo
c ds	call csscal(nfft_az,0.0,kw,1)	!reset to 0.

	do i=1,nfft_az				! Sun port ds
	   kw(i)=cmplx(0.0,0.0)
	enddo

	r_prf        = 1. / prf
	h_prf        = prf / 2.
	h_npulses    = npulses / 2
	h_fr_m       = fr_m / 2.
C-----------------------------------------------------------------------
C	GENERATE KAISER WEIGHTS ACCORDING TO AZ LINES
C-----------------------------------------------------------------------
	call kaiser ( kw, npulses,0.8 )		!shape factor=.8

C-----------------------------------------------------------------------
C	AZIMUTH COMPRESSION DO-LOOP
C-----------------------------------------------------------------------
cvd$ cncall
	do i = 1 ,nfft_rg

	   index_az_rg = nfft_az*(i-1)
	   azrg_add_hnp= index_az_rg + h_npulses
	   ar_sub_hnp= nfft_az*i- h_npulses

cvd$ nodepchk
cvd$ nosync
	   do j=1,npulses

c	      PERFORM DERAMP COMPENSATION   

	      taz = ( j - h_npulses - 1 ) * r_prf 
	      phi = -6.283185*((fd_m-h_prf)*taz+h_fr_m*taz*taz)

	      buff_opt (index_az_rg+j)  
     *	      = buff_az(index_az_rg+j) * cmplx(cos(phi),sin(phi)) 

cccc	      taz = ( j - npulses/2 - 1 ) / prf 
cccc	      phi = -2.*pi*(( fd_m - prf/2 )*taz+fr_m/2.*taz**2)
cccc
cccc	      buff_opt (nfft_az*(i-1)+j)  
c     *	      = buff_az(nfft_az*(i-1)+j) * cmplx(cos(phi),sin(phi)) 

	   enddo 

C	   PLACE THE MID-ECHO OF THE BURST AT THE BEGINNING OF FFT BUFFER
c	   & PERFORM KAISER WEIGHT 

cvd$ nodepchk
cvd$ nosync
	   do j=1,h_npulses

	      buff_shift(index_az_rg+j)  
     *	     = buff_opt(azrg_add_hnp + j) * kw(j)		

	      buff_shift( ar_sub_hnp + j)
     *	     = buff_opt(index_az_rg+j) * kw(j+h_npulses)	

ccc	      buff_shift(nfft_az*(i-1)+j)  
c     *	     = buff_opt(nfft_az*(i-1)+npulses/2 + j)* kw(j) 
ccc	      buff_shift( nfft_az*i - npulses/2 + j)
c     *	     = buff_opt(nfft_az*(i-1)+j) * kw(j+npulses/2)	
	
	   end do

c	type *,' AZ:',buff_shift(ar_sub_hnp+10),buff_shift(ar_sub_hnp+20)

C	   PADDING ZERO

	   do j =(h_npulses+1),(nfft_az-h_npulses)
	      buff_shift(index_az_rg+j)=cmplx(0.0,0.0)		
	   end do

ccc	   do j =(npulses/2+1),(nfft_az-npulses/2)
ccc	      buff_shift(nfft_az*(i-1)+j)=cmplx(0.0,0.0)		
ccc	   end do

C	   TRANSFORM TO FREQUENCY DOMAIN

c ds	   call cfft1d(nfft_az,buff_shift(index_az_rg+1),-1) !forward cfft
c	   call cfft1d(nfft_az,buff_shift(nfft_az*(i-1)+1),-1) !forward cfft

	   call cfft(buff_shift(index_az_rg+1),1,nfft_az,1) !+1 forward FFT

c L. Chen
	do k=1,nfft_az
           buff_shift(index_az_rg+k)=buff_shift(index_az_rg+k)
c 12.1     *                            *real(nfft_az)
	end do
	end do

	return
	end
c********************  end of az_comp.f ****************************
