c**********  CFFT  **********************************************************
c  Performs In-Place, Decimation-In-Frequency, complex Fast-Fourier Transform
c
c  Format:  CALL CFFT( A, AINC, N, IFLG)
c  Arguments:
c	A:      complex array holding input and output data
c	AINC:   increment between complex elements
c	N:      number of complex pairs
c	IFLG:   direction flag:
c		 1 = forward
c		 -1 = inverse
c
c  Summary:  Implements the fast algorithm developed by W. J. Tukey and
c  W. J. Cooley to perform the discrete Fourier transform summations:
c
c  Forward:  X(k) = SUM{ x(n) * exp[ -j2PI*nk/N ] } / N for n = 0...N
c  or
c  Inverse:  x(n) = SUM{ X(k) * exp[  j2PI*nk/N ] } for k = 0...N
c
c  By convention, the results of the forward transform is scaled down by N.
c
c  This routine was derived from the signal flow graphs presented by Rabiner
c  and Gold in "Theory and Application of Digital Signal Processing".
c  There are three nested loops that operate (from outside in) on the stages,
c  groups within a stage, and number of butterflies within a group.  The number
c  of stages are fixed by N.  The number of groups increase by a factor of 2
c  with each successive stage, while the number of butterfly pairs within each
c  group decrease by a factor of 2.  This is a DIF implementation with
c  bit-reversed shuffling at the end of the butterfly operations.
c
c  Author:  Eugene Y. C. Chu, Jet Propulsion Laboratory
c
c  Major Revisions:
c
c  12-AUG-1983 -  First completed version simulates ADSP FFT modules
c  21-SEP-1983 -  Copied and modified ADSP simulation code to perform native
c		  mode operations on double precision numbers.
c  22-SEP-1983 -  New version performs operations in single precision
c  06-MAR-1991 -  Completely renovated old routine for more efficiency and
c		  flexibility:
c		  - Added AINC to specify equal length strides between elements
c		    of input array A.
c		  - Added FFT.INC to include definitions for the complex roots
c		    of 1 (twiddle factors) in the TWIDDLE array and the bit-
c		    reversed address array REV.  These must be pregenerated and
c		    stored into the common block defined in FFT.INC.  The first
c		    half of the TWIDDLE array must contain the normal twiddle
c		    factors while the second half must contain its complex
c		    conjugate.  The MAX_FFT in FFT.INC must be set to the
c		    maximum length that the arrays were generated for.
c		  - Added starting point and stride for accessing TWIDDLE and
c		    REV to use one set of these arrays in calculating any
c		    length of FFT size, up to the maximum size, MAX_FFT.
c		    The starting point of TWIDDLE is at its first element for
c		    a forward FFT, and at N/2 for an inverse FFT.
c		  - Streamlined code to minimize calculations within the loops,
c		    the loops, eliminated intermediate indices that were not
c		    not absolutely necessary.
c		  - Moved calculation of the last stage out of the main loop.
c--
      subroutine cfft( a, ainc, n, iflg)

	implicit none	   !  take no prisoners

      integer ainc, n, iflg
      complex  a(0:n*ainc-1)

c  get the FFT common block

	include '../../../inc/fft.inc'
	complex temp
	integer last, pairs, p2, its, itw, tinc, hi, lo, hl, j, num

c  Initialize some pointers and increments

	tinc = max_fft / n	!  Twiddle index increment
	num = n * ainc		!  Total number of elements in array
	pairs = num		!  Number of pairs per group
	last = ainc * 2		!  samples in the last stage

c  Determine Twiddle Factors to use

	if( iflg .eq. 1 )then
	    its = 0		!  point to straight Twiddles scaled down by 2

	    do j = 0, num-1, ainc	!  scale data down for forward FFT
		a( j ) = a( j ) / n
	    end do
	else
	    its = max_fft/2	!  point to conjugate of straight Twiddles
	end if

c  >:< >:< >:< >:< >:< >:< >:< >:< >:< >:< >:< >:< >:< >:< >:< >:< >:< >:< >:<
c  >:<        Do the Butterfly on all but the last stage of the FFT        >:<
c  >:< >:< >:< >:< >:< >:< >:< >:< >:< >:< >:< >:< >:< >:< >:< >:< >:< >:< >:<

	do while( pairs .gt. last )	!  groups of pairs

	    p2 = pairs / 2		!  actual number of pairs
	    do j = 0, num-1, pairs

		itw = its		!  initialize Twiddle Index
		hl = j + p2 - 1		!  last High index
		do hi = j, hl, ainc		!  Index to upper and
		    lo = hi + p2		!  lower parts of pairs

		    temp = a(hi) - a(lo)	!  save difference
		    a(hi) = a(hi) + a(lo)	!  High result
		    a(lo) = temp * twiddle(itw)	!  Low result
		    itw = itw + tinc		!  next Twiddle index

		end do

	    end do

	    tinc = tinc * 2		!  Next Twiddle Increment
	    pairs = p2			!  new pairs count

	end do

c  >:< >:< >:< >:< >:< >:< >:< >:< >:< >:< >:< >:< >:< >:< >:< >:< >:< >:< >:<
c  >:<            Do the Butterfly on the last stage of the FFT            >:<
c  >:< >:< >:< >:< >:< >:< >:< >:< >:< >:< >:< >:< >:< >:< >:< >:< >:< >:< >:<

	do hi = 0, num-1, last
	    lo = hi + ainc
	    temp = a( hi ) - a( lo )
	    a( hi ) = a( hi ) + a( lo )
	    a( lo ) = temp
	end do

c  Do the bit-reverse shuffling

	itw = 0			!  Starting index of Bit Reversed address table
	tinc = max_fft / n	!  Table index increment
	do j = 0, n-1
	    if( j .gt. rev( itw ) )then		!  only do this once per pair
		hi = j * ainc			!  adjust for data increment
		lo = rev( itw ) * ainc
		temp = a( hi )			!  swap the pair
		a( hi ) = a( lo )
		a( lo ) = temp
	    end if
	   itw = itw + tinc			!  bump table index
	end do

	return
      end
