c***********************************************************************
c	Jet Propulsion Laboratory
c	Section:	334 Radar Science and Engineering
c	Project:	Magellan
c	Task:		Correlator, Alliant Fortran Version
c-----------------------------------------------------------------------
c	Module Name: radio_comp.f
c
c		Perform the radiometric compensation to remove gain
c		variation and present pixel intensity in defined unit
c		for the Magellan SAR Image Data.
c
c	Input:	1). A range-azimuth compressed data.
c		2). A associated Processing Parameters file.
c		    1.nfft_az:# of az fftn length.
c		    2.nfft_rg:# of ragne fft length.			  
c		    2.nsamples     :# of samples.
c		    3.prf    :pulse repetition frequency in hz.
c		    4.fd_m   :mid-range doppler freqency in hz.
c		    5.fr_m   :mid-range doppler rate in hz per sec.
c		    6.dr     :range cell.
c		    7.mu_r(3,3):9 points for radio compensation coefficients.
c		    8.i_m    :incidence anlge (mrp)
c
c	Output: A radiometric calibrated range-doppler image data.
c
c	Function:    
c		The radiocompensation processing will be performed on the 
c		range-azimuth domain.  Each pixels of the compressed response
c		shall be multiplied with an absolute compensation factors and 
c		a relative compensation factor.
c
c		The nine points of the radiometric compensation coefficients 
c		are read from pp file.
c
c		The backscatters are defined relative to the Muhleman model
c		which has the backscatter varying as a funtion of incidence 
c		angle.
c		
c		I). Remove relative gain variation introduced by:
c		    1.Distance between sensor and target.
c		    2.Gain variation within antenna beam.
c		    3.Backscatter strngth variation due to incidence angle 
c		      change.
c		II).Compensation for absolute gain change:
c		    1.Tranmitter power change.
c		    2.Receiver gain change.
c		    3.PRF and burst interval change.
c		    4.Radar pulse width.
c		    5.Sensor and processor gain.
c
c	*** Algorithms designed and original coded by: Mike Jin ***
c----------------------------------------------------------------------------
c
c	9/15/92	Anhua Chu DCS nfft_az size compensation factor.
c	8/18/92	Anhua Chu Using r_cmp_coe.f to calculate mu_r(3,3)
c	8/17/92	Anhua Chu Added removing ant_gain coefficient
c	8/13/92	Anhua Chu Revused for range weight
c	3/30/92	Anhua Chu Initial code.
c****************************************************************************
	subroutine radio_comp(buff_in,buff_out)
	
	implicit none
	include '../../../inc/keyvar.inc'
	include '../../../inc/pp.inc'
	include '../../../inc/constant.inc'
	include '../../../inc/corr_buffer.inc'

c	PASSING IN PARAMETER
	complex buff_in(*)		!Input buffer for azimuth-range image

c	PASSING OUT PARAMETER
	complex buff_out(*)		!Output buffer for radio compensation

c	LOCAL VARIABLES
	real 	r_antenna(400)	!Input antenna range weights
	real 	b_offset,c_rg(3,3),Nc,th_max,s2,s3
	real 	fd,gain,d_r,r_m_bin,delta_fd,scale_real,q_range
	real 	scale,s_factor,r_ant_wt,e_coef_1,e_coef_2,e_coef_3
	real 	r_nfft_az,r_dcs_nfft_az
	integer	ia,j,k,i,rg_count,h_nfft_az,h_nfft_rg,ant_index
	integer	az_min,az_max,rg_num
	integer test_burst_no
	common /test_burst_no/scale,test_burst_no

d	type *,'  ' 
c       ------------
c	GENERATE RANGE ANTENNA PATTERN WEIGHT
c       ------------
	call ant_weight ( r_antenna)
	
c	----------
c	BAQ Gain of DCS for Compensation
c	----------
	th_max = max(sar_threshold(1),sar_threshold(2),sar_threshold(3),
     *           sar_threshold(4),sar_threshold(5),sar_threshold(6),
     *           sar_threshold(7),sar_threshold(8),sar_threshold(9),
     *           sar_threshold(10),sar_threshold(11),sar_threshold(12),
     *           sar_threshold(13),sar_threshold(14),sar_threshold(15),
     *           sar_threshold(16),sar_threshold(17),sar_threshold(18),
     *           sar_threshold(19),sar_threshold(20),sar_threshold(21),
     *           sar_threshold(22),sar_threshold(23),sar_threshold(24))

	s3 = int(31.*log(th_max+1.)/log(127.)) + 1.
	s2 = exp(-(s3*log(127.)/31.))

d	type *,' baq s2 s3 thmax:',s2,s3,th_max

c	----------
c	PROCESSING GAIN SCALE FACTOR
C	----------
c	r_nfft_az = float(nfft_az)

	r_dcs_nfft_az = float(dcs_nfft_az)
	s_factor = scale*s2/r_dcs_nfft_az

c	s_factor = scale/s2
d	type *,'R_COMP S_factor scale az_ratio:',s_factor,scale,
d    *	      r_nfft_az/r_dcs_nfft_az

c	----------
c	Boresight Offset
c	----------
	b_offset = (R_bip - R_m)/Dr

c	----------
c	EACH RANGE WEIGHT SAMPLE applies on
c	"TWO" SLANT RANGE SAMPLES
C	----------
	q_range = 2 

c	----------
c	CALCULATE RADIOMETRIC COMPENSATION COEFFICIENTS
c	---------
c	Antenna Gain be taken away from RADIOMETRIC COMP FACTOR,
c	Normalized by Antenna Gain

	do i = 1, 3
	   do j = 1, 3 

	   c_rg(i,j)=rcomp_factor(i,j)* sqrt(rcomp_ant_gain(i,2))

	   enddo
	enddo
 
	call r_cmp_coe(r_rg,fd_rg,c_rg,mu_r) 
	
d	type *,' '
d	type *,' rcomp_ant_gain(i,2)',rcomp_ant_gain(1,2),
d    *           rcomp_ant_gain(2,2),rcomp_ant_gain(3,2)
d	type *,' 9 rcom_rcom G :',rcomp_factor 
d	type *,' '
d	type *,' r_rg fd_rg:',r_rg,fd_rg
d	type *,'R_COMP 9 mu_r:',mu_r
d	type *,' '
c	----------
c	DETERMINE INDEX
c	----------
c	r_m_bin = (Nsamples - Nc + 1)/2	!DCS range center,nc=60 

	delta_fd=prf/nfft_az
	az_min=nint((prf/2-pbw/2)/delta_fd) - 4
	az_max=nint((prf/2+pbw/2)/delta_fd) + 4

	h_nfft_az = nfft_az / 2
	h_nfft_rg = nfft_rg / 2		!s/w corr rg center

c       -------------	
c	Main do-loop
c	------------

cvd$ nodepchk
cvd$ nosync

	do i=1,nfft_rg

c	GENERATE RADIO. COMPENSATION GAIN COEFFICIENTS,
c	SELECT RANGE WEIGHT LINE INDEX

	   d_r = (i - h_nfft_rg - b_offset) * dr 
	   
	   e_coef_1=mu_r(3,1)*d_r*d_r + mu_r(2,1)*d_r + mu_r(1,1)
	   e_coef_2=mu_r(3,2)*d_r*d_r + mu_r(2,2)*d_r + mu_r(1,2)
	   e_coef_3=mu_r(3,3)*d_r*d_r + mu_r(2,3)*d_r + mu_r(1,3)

	   ant_index=nint((i-h_nfft_rg-b_offset-0.001)/q_range+200)
	   if (ant_index .gt. 400 ) ant_index =400 
	   if (ant_index .lt. 1   ) ant_index = 1
	   r_ant_wt=r_antenna(ant_index)  
	
cvd$ nodepchk
cvd$ nosync

	   do j=az_min,az_max

C	      ESTIMATE RADIOMETRIC COMPENSATION GAIN 
	      fd = ( j - h_nfft_az ) * delta_fd
	      gain=(e_coef_3*fd*fd+e_coef_2*fd+e_coef_1)*s_factor

	      buff_out(j+nfft_az*(i-1))=buff_in(j+nfft_az*(i-1))
     *	                                * gain * r_ant_wt 

	   end do

	enddo	! end of #nsamples
d	type *,'Radio Gain ant 2nd_c',gain,r_ant_wt,gain/s_factor
        return 
	end
c********************  end of radio_comp.f ****************************
