
C$Procedure      CONICS ( Determine state from conic elements )

      SUBROUTINE CONICS ( ELTS, ET, STATE )

C$ Abstract
C
C     Determine the state (position, velocity) of an orbiting body
C     from a set of elliptic, hyperbolic, or parabolic orbital
C     elements.
C
C$ Required_Reading
C
C     None.
C
C$ Keywords
C
C     CONIC
C     EPHEMERIS
C
C$ Declarations

      DOUBLE PRECISION      ELTS    ( 8 )
      DOUBLE PRECISION      ET
      DOUBLE PRECISION      STATE   ( 6 )

C$ Brief_I/O
C
C     VARIABLE  I/O  DESCRIPTION
C     --------  ---  --------------------------------------------------
C     ELTS       I   Conic elements.
C     ET         I   Input time.
C     STATE      O   State of orbiting body at T.
C
C$ Detailed_Input
C
C     ELTS       are conic elements describing the orbit of a body
C                around a primary. The elements are, in order:
C
C                      RP      Perifocal distance.
C                      ECC     Eccentricity.
C                      INC     Inclination.
C                      LNODE   Longitude of the ascending node.
C                      ARGP    Argument of periapsis.
C                      M0      Mean anomaly at epoch.
C                      T0      Epoch.
C                      MU      Gravitational parameter.
C
C                Units are kg, rad, rad/sec. The epoch is given
C                in ephemeris seconds past J2000. The same elements
C                are used to describe all three types (elliptic,
C                hyperbolic, and parabolic) of conic orbit.
C
C     ET         is the time at which the state of the orbiting body
C                is to be determined, in ephemeris seconds J2000.
C
C$ Detailed_Output
C
C     STATE      is the state (position and velocity) of the body at
C                time ET. Components are x, y, z, dx/dt, dy/dt, dz/dt.
C
C$ Parameters
C
C      None.
C
C$ Exceptions
C
C     None.
C
C$ Files
C
C     None.
C
C$ Particulars
C
C     None.
C
C$ Examples
C
C     Let VINIT contain the initial state of a spacecraft relative to
C     the center of a planet at epoch ET, and let GM be the gravitation
C     parameter of the planet. The call
C
C        CALL OSCELT ( VINIT, ET, GM, ELTS )
C
C     produces a set of osculating elements describing the nominal
C     orbit that the spacecraft would follow in the absence of all
C     other bodies in the solar system.
C
C     Now let STATE contain the state of the same spacecraft at some
C     other epoch, LATER. The difference between this state and the
C     state predicted by the nominal orbit at the same epoch can be
C     computed as follows.
C
C        CALL CONICS ( ELTS, LATER, NOMINAL )
C        CALL VSUBG  ( NOMINAL, STATE, 6, DIFF )
C
C        WRITE (*,*) 'Perturbation in x, dx/dt = ', DIFF(1), DIFF(4)
C        WRITE (*,*) '                y, dy/dt = ', DIFF(2), DIFF(5)
C        WRITE (*,*) '                z, dz/dt = ', DIFF(3), DIFF(6)
C
C$ Restrictions
C
C     None.
C
C$ Literature_References
C
C     [1] Roger Bate, Fundamentals of Astrodynamics, Dover, 1971.
C
C$ Author_and_Institution
C
C     I.M. Underwood  (JPL)
C
C$ Version
C
C-    SPICELIB Version 1.0.0, 31-JAN-1990 (IMU)
C
C-&


C$ Revisions
C
C-    Beta Version 1.0.1, 27-JAN-1989 (IMU)
C
C        Examples section completed.
C
C-&



C
C     SPICELIB functions
C
      LOGICAL               RETURN
      DOUBLE PRECISION      PI
      DOUBLE PRECISION      TWOPI

C
C     Local variables
C
      DOUBLE PRECISION      RP
      DOUBLE PRECISION      ECC
      DOUBLE PRECISION      INC
      DOUBLE PRECISION      LNODE
      DOUBLE PRECISION      ARGP
      DOUBLE PRECISION      M0
      DOUBLE PRECISION      T0
      DOUBLE PRECISION      MU

      DOUBLE PRECISION      T
      DOUBLE PRECISION      P
      DOUBLE PRECISION      A
      DOUBLE PRECISION      M

      DOUBLE PRECISION      D
      DOUBLE PRECISION      E
      DOUBLE PRECISION      F
      DOUBLE PRECISION      NU

      DOUBLE PRECISION      COSI
      DOUBLE PRECISION      SINI
      DOUBLE PRECISION      COSN
      DOUBLE PRECISION      SINN
      DOUBLE PRECISION      COSW
      DOUBLE PRECISION      SINW
      DOUBLE PRECISION      BASISP     ( 3 )
      DOUBLE PRECISION      BASISQ     ( 3 )
      DOUBLE PRECISION      R
      DOUBLE PRECISION      V



C
C      There is a particularly simple expression for the position
C      and velocity of an orbiting object, expressed in perifocal
C      coordinates (where the basis vectors P and Q point toward
C      perifocus and true anomaly 90 degrees, respectively):
C
C            R = r cos(nu) P  +  r sin(nu) Q
C            -             -               -
C
C            V = v [  -sin(nu) P  +  (e + cos(nu)) Q ]
C            -                 -                   -
C
C      where
C
C                     p                         1/2
C            r = --------------- ,    v = (mu/p)
C                 1 + e cos(nu)
C
C      Unfortunately, this expresses the position and velocity in
C      terms of the true anomaly (nu) instead of the eccentric anomaly
C      (D, E, or F). The relationships between the anomalies for the
C      various conic sections are shown below.
C
C                        e + cos(nu)
C            cos(E)  = ---------------         (ellipse)
C                        1 + e cos(nu)
C
C
C                        e + cos(nu)
C            cosh(F) = ---------------         (hyperbola)
C                        1 + e cos(nu)
C
C
C            D       =  tan(nu/2)              (parabola)
C
C      So, if we can get either anomaly as a function of time, we are
C      home free.
C
C      The time of flight and eccentric anomaly are related as follows.
C
C                      3 1/2
C            (t-T)(mu/a )         =  E - e sin(E)      (ellipse)
C
C                         3 1/2
C            (t-T)(mu/(-a) )      =  e sinh(F) - F     (hyperbola)
C
C                                         3
C                        3  1/2          D
C            (t-T)(mu/(2q ))      =  D + -             (parabola)
C                                        3
C
C      Newton's method can be used to find the values of E, F and D
C      at the current time. The equivalent value for nu can then be
C      used to determine the position and velocity.
C



C
C     Standard SPICE error handling.
C
      IF ( RETURN () ) THEN
         RETURN
      ELSE
         CALL CHKIN ( 'CONICS' )
      END IF


C
C     Unpack the element vector. Use time of flight instead of time.
C
      RP     = ELTS(1)
      ECC    = ELTS(2)
      INC    = ELTS(3)
      LNODE  = ELTS(4)
      ARGP   = ELTS(5)
      M0     = ELTS(6)
      T0     = ELTS(7)
      MU     = ELTS(8)

      T = ET - T0

C
C     One of the nice things about the semi-latus rectum is that
C     the same formula works for all conics.
C
      P = RP * ( 1.D0 + ECC )

C
C     Ellipses first, since they are the most common.
C
      IF ( ECC .LT. 1.D0 ) THEN

         A = RP / ( 1.D0 - ECC )
         M = MOD ( M0 + T * SQRT(MU/A) / A, TWOPI() )

         CALL ELLTOF ( M, ECC, E )

         NU = ACOS ( ( ECC - COS(E) ) / ( ECC*COS(E) - 1.D0 ) )

         IF ( E .LT. 0.D0  .OR.  E .GT. PI() ) THEN
            NU = -NU
         END IF

C
C     Hyperbolas next.
C
      ELSE IF ( ECC .GT. 1 ) THEN

         A = RP / ( 1.D0 - ECC )
         M = M0 + T * SQRT(MU/ABS(A)) / ABS(A)

         CALL HYPTOF ( M, ECC, F )

         NU = ACOS ( ( COSH(F) - ECC ) / ( 1.D0 - ECC*COSH(F) ) )

         IF ( F .LT. 0.D0  .OR.  F .GT. PI() ) THEN
            NU = -NU
         END IF

C
C     Finally, parabolas.
C
      ELSE

         M = M0 + T * SQRT(MU/(2.D0*RP)) / RP

         CALL PARTOF ( M, D )

         NU = 2.D0 * ATAN ( D )

      END IF

C
C     Construct the basis vectors.
C
      COSI = DCOS ( INC   )
      SINI = DSIN ( INC   )
      COSN = DCOS ( LNODE )
      SINN = DSIN ( LNODE )
      COSW = DCOS ( ARGP  )
      SINW = DSIN ( ARGP  )

      BASISP(1) =  COSN * COSW  -  SINN * COSI * SINW
      BASISP(2) =  SINN * COSW  +  COSN * COSI * SINW
      BASISP(3) =  SINI * SINW

      BASISQ(1) = -COSN * SINW  -  SINN * COSI * COSW
      BASISQ(2) = -SINN * SINW  +  COSN * COSI * COSW
      BASISQ(3) =  SINI * COSW

C
C     And compute the state.
C
      R = P / ( 1.D0 + ECC*COS(NU) )

      CALL VLCOM ( R *COS(NU), BASISP,
     .             R *SIN(NU), BASISQ,  STATE(1) )


      V = SQRT ( MU/P )

      CALL VLCOM ( -V *        SIN(NU),  BASISP,
     .              V * (ECC + COS(NU)), BASISQ, STATE(4) )


      CALL CHKOUT ( 'CONICS' )
      RETURN
      END
