C$Procedure      ERRCH  ( Insert String into Error Message Text )

      SUBROUTINE ERRCH ( MARKER, STRING )

C$ Abstract
C
C      Substitute a character string for the first occurrence of
C      a marker found in the current long error message.
C
C$ Required_Reading
C
C      ERROR
C
C$ Keywords
C
C      ERROR, CONVERSION
C
C$ Declarations

      CHARACTER*(*)         MARKER
      CHARACTER*(*)         STRING

C$ Brief_I/O
C
C      VARIABLE  I/O  DESCRIPTION
C      --------  ---  --------------------------------------------------
C      MARKER     I   A substring of the error message to be replaced.
C      STRING     I   The character string to substitute for MARKER.
C
C$ Detailed_Input
C
C
C      MARKER     is a character string which marks a position in
C                 the long error message where a character string
C                 is to be substituted.  Leading and trailing blanks
C                 in MARKER are not significant.
C
C                 Case IS significant;  'XX' is considered to be
C                 a different marker from 'xx'.
C
C      STRING     is a character string which will be substituted for
C                 the first occurrence of MARKER in the long error
C                 message.  This occurrence of the substring indicated
C                 by MARKER will be removed, and replaced by STRING.
C                 Leading and trailing blanks in STRING are not
C                 significant.
C
C
C$ Detailed_Output
C
C      None.
C
C$ Parameters
C
C     None.
C
C$ Exceptions
C
C      This routine does not detect any errors.
C
C      However, this routine is part of the SPICELIB error
C      handling mechanism.
C
C$ Files
C
C      None.
C
C$ Particulars
C
C      Please read the "required reading" first!
C
C      The effect of this routine is to update the current long
C      error message.  If no marker is found, (e.g., in the
C      case that the long error message is blank), the routine
C      has no effect.  If multiple instances of the marker
C      designated by MARKER are found, only the first one is
C      replaced.
C
C      If the character string resulting from the substitution
C      exceeds the maximum length of the long error message, the
C      characters on the right are lost.  No error is signalled.
C
C      This routine has no effect if changes to the long message
C      are not allowed.
C
C$ Examples
C
C
C      1.   In this example, the marker is:   #
C
C
C           The current long error message is:
C
C           'Invalid operation value.  The value was #'.
C
C
C           After the call,
C
C
C                CALL ERRCH ( '#',  '  five   ' )
C
C           The long error message becomes:
C
C           'Invalid operation value.  The value was five'.
C
C
C
C
C      2.   In this example, the marker is:   XX
C
C
C           The current long error message is:
C
C           'Left endpoint exceeded right endpoint.  The left'//
C           'endpoint was:  XX.  The right endpoint was:  XX.'
C
C
C           After the call,
C
C                CALL ERRCH ( 'XX',  'five'  )
C
C           The long error message becomes:
C
C           'Left endpoint exceeded right endpoint.  The left'//
C           'endpoint was:  five.  The right endpoint was:  XX.'
C
C
C$ Restrictions
C
C      The caller must ensure that the message length, after sub-
C      stitution is performed, doesn't exceed 320 characters.
C
C$ Literature_References
C
C      None.
C
C$ Author_and_Institution
C
C      N.J. Bachman    (JPL)
C
C$ Version
C
C-      SPICELIB Version 1.0.0, 31-JAN-1990  (NJB)
C
C-&


C
C     SPICELIB functions
C
      INTEGER               LASTNB
      INTEGER               FRSTNB
      LOGICAL               ALLOWD
C
C     Local Variables:
C
      INTEGER               LONGLN
      PARAMETER           ( LONGLN = 320 )

      CHARACTER*(LONGLN)    LNGMSG
      CHARACTER*(LONGLN)    TMPMSG



      INTEGER               STRPOS


C
C     Executable Code:
C


C
C     Changes to the long error message have to be allowed, or we
C     do nothing.
C

      IF ( .NOT. ALLOWD() ) THEN

         RETURN

      END IF


C
C     MARKER has to have some non-blank characters, or we do nothing.
C     The same goes for STRING.
C

      IF (   (  LASTNB (MARKER)  .EQ.  0  )   .OR.
     .       (  LASTNB (STRING)  .EQ.  0  )
     .   )
     .THEN

         RETURN

      END IF

C
C     Get a copy of the current long error message.
C

      CALL GETLMS ( LNGMSG )

C
C     Locate the leftmost occurrence of MARKER, if there is one
C     (ignoring leading and trailing blanks):
C

      STRPOS = INDEX (  LNGMSG,
     .                  MARKER ( FRSTNB(MARKER):LASTNB(MARKER) )    )

      IF  ( STRPOS .EQ. 0 ) THEN
         RETURN
      ELSE

C
C        We put together TMPMSG, a copy of LNGMSG with MARKER
C        replaced by the character representation of INTEGR:
C

         IF ( STRPOS .GT. 1 ) THEN

            IF (  ( STRPOS + LASTNB(MARKER) - FRSTNB(MARKER) ) .LT.
     .              LASTNB ( LNGMSG )

     .         ) THEN

C
C              There's more of the long message after the marker...
C

               TMPMSG =

     .         LNGMSG ( :STRPOS - 1 )                                 //
     .         STRING ( FRSTNB(STRING) : LASTNB(STRING) )             //
     .         LNGMSG ( STRPOS + LASTNB(MARKER) - FRSTNB(MARKER) + 1: )

            ELSE

               TMPMSG =  LNGMSG ( :STRPOS - 1     )                   //
     .                   STRING ( FRSTNB(STRING) : LASTNB(STRING) )

            END IF


         ELSE

C
C           We're starting with STRING...
C


            IF (  ( LASTNB(MARKER) - FRSTNB(MARKER) ) .LT.
     .              LASTNB(LNGMSG)

     .         ) THEN


C
C              There's more of the long message after the marker...
C

               TMPMSG = STRING ( FRSTNB(STRING) : LASTNB(STRING) )    //


     .                  LNGMSG ( STRPOS + LASTNB(MARKER)
     .                                  - FRSTNB(MARKER)  + 1 :     )

            ELSE

C
C              The marker's the whole string:
C

               TMPMSG = STRING ( FRSTNB(STRING) : LASTNB(STRING) )

            END IF

         END IF

C
C        Update long message:
C

         CALL PUTLMS ( TMPMSG )

      END IF

      END
