

C$Procedure      ET2UTC ( Ephemeris Time to UTC )

      SUBROUTINE ET2UTC ( ET, FORMAT, PREC, UTCSTR )

C$ Abstract
C
C      Convert an input time from ephemeris seconds past J2000
C      to Calendar, Day-of-Year, or Julian Date format, UTC.
C
C$ Required_Reading
C
C      TIME
C
C$ Keywords
C
C      TIME
C
C$ Declarations

      DOUBLE PRECISION ET
      CHARACTER*1      FORMAT
      INTEGER          PREC
      CHARACTER*(*)    UTCSTR

C$ Brief_I/O
C
C      VARIABLE  I/O  DESCRIPTION
C      --------  ---  --------------------------------------------------
C      ET         I   Input epoch, ephemeris seconds past J2000.
C      FORMAT     I   Format of output epoch.
C      PREC       I   Digits of precision in fractional seconds or days.
C      UTCSTR     O   Output time string, UTC.
C
C$ Detailed_Input
C
C      ET          is the input epoch, ephemeris seconds past J2000.
C
C      FORMAT      is the format of the output time string. It may be
C                  any of the following:
C
C                        'C'      Calendar format, UTC.
C                        'D'      Day-of-Year format, UTC.
C                        'J'      Julian Date format, UTC.
C
C      PREC        is the number of digits of precision to which
C                  fractional seconds (for Calendar and Day-of-Year
C                  formats) or days (for Julian Date format) are to
C                  be computed. If PREC is zero, no decimal point is
C                  appended to the output string. PREC may not be
C                  less than zero.
C
C$ Detailed_Output
C
C      UTCSTR      is the output time string equivalent to the input
C                  epoch, in the specified format, as shown below.
C
C                        'C'      '1986 APR 12 16:31:09.814'
C                        'D'      '1986-102 // 16:31:12.814'
C                        'J'      'JD 2446533.18834276'
C
C                  If an error occurs, UTCSTR is not changed.
C
C$ Parameters
C
C     None.
C
C$ Exceptions
C
C      If the format for the output string is not recognized, the
C      error SPICE(INVALIDTIMEFORMAT) is signalled.
C
C$ Files
C
C      None.
C
C$ Particulars
C
C      The value of Delta ET corresponding to the input epoch is
C      determined, and subtracted from the input epoch to give UTC
C      seconds past J2000, and the result converted to the specified
C      format.
C
C$ Examples
C
C      Let the value of ET be -527644192.5403653 ephemeris seconds
C      past J2000. Assuming that the nominal values in the kernel pool
C      have not been altered, the following calls
C
C            CALL ET2UTC ( ET, 'C', 0, UTCSTR )
C            CALL ET2UTC ( ET, 'C', 3, UTCSTR )
C            CALL ET2UTC ( ET, 'D', 5, UTCSTR )
C            CALL ET2UTC ( ET, 'J', 7, UTCSTR )
C
C      produce the following output strings
C
C            '1983 APR 13 12:09:14'
C            '1983 APR 13 12:09:14.274'
C            '1983-103 // 12:09:14.27400'
C            'JD 2445438.0064152'
C
C      respectively.
C
C$ Restrictions
C
C      The conversion between ET and UTC depends on the values in the
C      kernel pool. The kernel pool should be loaded prior to calling
C      this subroutine.
C
C$ Literature_References
C
C      Jesperson and Fitz-Randolph, From Sundials to Atomic Clocks,
C      Dover Publications, New York, 1977.
C
C$ Author_and_Institution
C
C      W.M. Owen       (JPL)
C      I.M. Underwood  (JPL)
C
C$ Version
C
C-      SPICELIB Version 1.0.0, 31-JAN-1990 (WMO) (IMU)
C
C-&



C
C     SPICELIB functions
C
      LOGICAL               RETURN
      DOUBLE PRECISION      J2000
      DOUBLE PRECISION      SPD
      LOGICAL               FAILED

C
C     Local variables
C
      DOUBLE PRECISION UTCSEC
      DOUBLE PRECISION DELTA

      CHARACTER*1      FMT
      CHARACTER*80     STR
      CHARACTER*80     FRACT

      DOUBLE PRECISION DP2000
      DOUBLE PRECISION SECOND
      INTEGER          DAY
      INTEGER          MONTH
      INTEGER          YEAR
      INTEGER          HOUR
      INTEGER          MINUTE
      INTEGER          DOY

      INTEGER          I
      DOUBLE PRECISION D

      CHARACTER*3      MTHNAM   ( 12 )
      INTEGER          DAYS     ( 12 )

C
C     Save everything between calls
C
      SAVE

C
C     Initial values
C
      DATA         MTHNAM       / 'JAN', 'FEB', 'MAR', 'APR',
     .                            'MAY', 'JUN', 'JUL', 'AUG',
     .                            'SEP', 'OCT', 'NOV', 'DEC'  /

      DATA         DAYS         /    0,    31,    59,    90,
     .                             120,   151,   181,   212,
     .                             243,   273,   304,   334   /



C
C     Standard SPICE error handling.
C
      IF ( RETURN () ) THEN
         RETURN
      ELSE
         CALL CHKIN ( 'ET2UTC' )
      END IF

C
C     Convert FORMAT to uppercase for ease of comparison. Make sure it's
C     one of the recognized formats.
C
      CALL UCASE ( FORMAT, FMT )

      IF ( INDEX ( 'CDJ', FMT ) .EQ. 0 ) THEN

         CALL SETMSG ( 'ET2UTC: Format specification for output time '//
     .                 'string not recognized.  Valid specifications '//
     .                 'are:  ''C'', ''J'', or ''D''.  The supplied  '//
     .                 'value was  '// FORMAT
     .               )


         CALL SIGERR ( 'SPICE(INVALIDTIMEFORMAT)' )
         CALL CHKOUT ( 'ET2UTC' )
         RETURN

      END IF

C
C     Convert to UTC seconds past J2000.
C
      CALL DELTET ( ET, 'ET', DELTA )

      IF ( .NOT. FAILED () ) THEN
         UTCSEC = ET - DELTA
      ELSE
         CALL CHKOUT ( 'ET2UTC' )
         RETURN
      END IF



C
C     If the output is Julian Date, we're ready to go. Remember that
C     the day part of Julian Date already has seven digits built in.
C
      IF ( FMT .EQ. 'J' ) THEN


         CALL DPSTRF ( J2000()+(UTCSEC/SPD()), PREC+7, 'F', STR )
         CALL PREFIX ( 'JD', 0,                             STR )


C
C     It's one of the calendar formats.
C
      ELSE

C
C        Convert to days past 2000. Round up seconds here.
C
         D = 0.5D0 / SPD()

         DO 50001
     .      I = 1, PREC
            D = 0.1D0 * D
50001    CONTINUE

         DP2000 = UTCSEC / SPD() + D + 0.5D0

C
C        Get the year, month, and day. This somewhat convoluted code
C        is the inverse of the Muller-Wimberly formula.
C        Make sure that the integer DAY
C        corresponds to the preceding midnight even for times before
C        the J2000 origin.  Also note that 1461 = 4 years (4*365 + 1)
C        and 146097 = 4 Gregorian centuries (400*365 + 97).
C        The constant 730426 is the number of days from March 1, 1 BC
C        (in the proleptic Gregorian calendar; everything is Gregorian)
C        to J2000; it moves the time origin to then, which
C        makes the problem more tractable.  In particular, it
C        enables the elegant (5*DAY-3)/153 expression, which gives
C        months of 31,30,31,30,31, 31,30,31,30,31, 31,30,... days.
C        (This series is in fact the lengths of the first 11 months if
C        one starts with March.)  Finally, MONTH is changed to start
C        at January; months 11 and 12 correspond to Jan or Feb of the
C        following year.
C
         IF ( DP2000 .GE. 0.D0  .OR.  DP2000 .EQ. DINT ( DP2000) ) THEN
            DAY = DP2000
         ELSE
            DAY = DP2000 - 1.D0
         END IF

         SECOND   = SPD()*(DP2000-DBLE(DAY))
         I        = DAY + 730426
         YEAR     = (4*I-1)/146097
         DAY      = ((4*I-1)-(146097*YEAR)) / 4
         I        = (4*DAY+3) / 1461
         DAY      = (((4*DAY+3)-(1461*I))+4) / 4
         MONTH    = (5*DAY-3) / 153
         DAY      = (((5*DAY-3)-(153*MONTH))+5) / 5
         YEAR     = 100*YEAR + I

         IF ( MONTH .LT. 10 ) THEN
            MONTH = MONTH + 3

         ELSE
            MONTH = MONTH - 9
            YEAR  = YEAR + 1
         END IF

C
C        Obtain hours, minutes, and seconds. (This is somewhat
C        more straightforward.)
C
         HOUR   = SECOND / 3600.D0
         SECOND = SECOND - 3600.D0*DBLE(HOUR)
         MINUTE = SECOND / 60.D0
         SECOND = SECOND - 60.D0*DBLE(MINUTE)

C
C        Time to make a decision about the first 11 characters of
C        the output string. If the output format is Calendar, convert
C        YEAR, MONTH, and DAY to character format.
C
         IF ( FMT .EQ. 'C' ) THEN

            STR = ' '
            WRITE (STR(1:4),FMT='(I4)') YEAR

            STR(6:8) = MTHNAM(MONTH)

            WRITE (STR(10:11),FMT='(I2.2)') DAY

C
C        Otherwise, convert MONTH and DAY to Day-of-Year.
C
         ELSE

C
C           In leap years, add an extra day after February.
C
            IF ( MONTH .GT. 2 ) THEN
               IF (      ( MOD (YEAR, 400) .EQ. 0 )
     .              .OR. ( MOD (YEAR,   4) .EQ. 0    .AND.
     .                     MOD (YEAR, 100) .NE. 0          ) ) THEN
                 DAY = DAY + 1
               END IF
            END IF

            DOY = DAYS(MONTH) + DAY

            STR = '    -    //'

            WRITE (STR(1:4),FMT='(I4)')   YEAR
            WRITE (STR(6:8),FMT='(I3.3)') DOY

         END IF

C
C        Convert HOUR, MINUTE, integral part of seconds to character
C        format. Leading zeros required.
C
         WRITE (STR(13:14),FMT='(I2.2)') HOUR
         STR(15:15) = ':'

         WRITE (STR(16:17),FMT='(I2.2)') MINUTE
         STR(18:18) = ':'

         I = SECOND
         WRITE (STR(19:20),FMT='(I2.2)') I

C
C        Convert the fractional part of the second to the character
C        format. Be careful with negative values of UTCSEC.
C
         IF ( PREC .GT. 0 ) THEN

            D = DMOD ( UTCSEC, 1.D0 )

            IF ( D .GE. 0.D0 ) THEN
               CALL DPSTRF ( D, PREC, 'F', FRACT )

            ELSE
               CALL DPSTRF ( 1.D0+D, PREC+1, 'F', FRACT )
            END IF

            I = INDEX ( FRACT, '.' )
            STR(21: ) = FRACT(I:I+PREC)

         END IF

      END IF

C
C     Assign to UTCSTR. If the output string is too short, it will
C     be truncated on the right.
C
      UTCSTR = STR


      CALL CHKOUT ( 'ET2UTC' )
      RETURN
      END
