C$Procedure      FNDLUN ( Find a free logical unit )

      SUBROUTINE FNDLUN ( UNIT )

C$ Abstract
C
C     Return the number of a free logical unit, if one is available.
C
C$ Required_Reading
C
C     None.
C
C$ Keywords
C
C     FILES
C
C$ Declarations

      INTEGER          UNIT

      INTEGER          MINLUN
      PARAMETER      ( MINLUN =  1 )

      INTEGER          MAXLUN
      PARAMETER      ( MAXLUN = 63 )

C$ Brief_I/O
C
C     VARIABLE  I/O  DESCRIPTION
C     --------  ---  --------------------------------------------------
C     UNIT       O   The number of a free logical unit.
C     MINLUN     P   Minimum logical unit number.
C     MAXLUN     P   Maximum logical unit number.
C
C$ Detailed_Input
C
C     None.
C
C$ Detailed_Output
C
C     UNIT        is the number of a free logical unit (also called
C                 an "external unit"). If no free units are available,
C                 the value of UNIT is 0.
C
C$ Parameters
C
C     MINLUN      is the minimum logical unit number. The Fortran
C                 standard states that unit numbers must be zero or
C                 positive.  However, the value 0 is reserved as a
C                 status code for this routine, so MINLUN must be
C                 assigned a value greater than 0.
C
C     MAXLUN      is the maximum logical unit number allowed by the
C                 VAX Fortran compiler. This may differ for other
C                 machines.
C
C     Listed below are the values for several machines:
C
C     Machine                       MINLUN     MAXLUN
C     -----------------------       -------    -------
C     VAX                                 1         99
C
C     IBM PC / Microsoft 5.0              1      32767
C
C     Sun / FORTRAN 1.2                   1         63
C
C$ Exceptions
C
C         Error free.
C
C
C     1)  If no logical units are available, UNIT is set equal
C         to 0.
C
C     2)  This routine performs a Fortran INQUIRE operation.  If
C         the INQUIRE fails, UNIT is set equal to the negative
C         of the INQUIRE iostat ( UNIT will thus have a negative
C         value).
C
C$ Files
C
C     None.
C
C$ Particulars
C
C     FNDLUN returns the number of the first (unreserved) unit not
C     currently connected to a file.  It thus frees the user from
C     having to maintain an accounting of which units are open, which
C     are closed, and which are available.
C
C     This routine is related to the routines GETLUN, RESLUN, and
C     FRELUN.  Together, these routines support coordinated usage of
C     Fortran logical units.  FNDLUN (Find a free logical unit) and
C     GETLUN (Get a free logical unit) both have the function of
C     returning a logical unit number that is not reserved or already
C     in use.  The principal difference between the functionality of
C     these routines is that GETLUN both returns a status code and
C     signals an error if a free unit is not found, while FNDLUN
C     merely returns a status code.
C
C     RESLUN is used to reserve logical unit numbers, so that they will
C     not be returned by GETLUN or FNDLUN; FRELUN frees logical units
C     previously reserved via calls to RESLUN.
C
C     Logical units 5-7 are reserved by default. Other units may be
C     reserved by calling RESLUN. Once reserved, units (except 5-7) may
C     be unreserved by calling FRELUN.
C
C     To reserve logical unit numbers for special use, refer to
C     RESLUN. To make reserved units available to FNDLUN and GETLUN,
C     refer to FRELUN.
C
C     A unit returned by FNDLUN does NOT automatically become a
C     reserved unit.  If the user wishes to reserve a unit found by
C     FNDLUN, the call to FNDLUN must be followed by a call to RESLUN.
C
C     Note that although 0 is a valid logical unit number on some
C     systems, a value of 0 returned by FNDLUN indicates that no free
C     logical unit was available, rather than that logical unit 0 is
C     available.  Similarly, negative values returned by FNDLUN are
C     status codes, not logical unit numbers.
C
C$ Examples
C
C     The following code fragment illustrates the use of FNDLUN.
C
C        CALL FNDLUN ( UNIT )
C
C        IF ( UNIT .LT. 0 ) THEN
C           RETURN
C        END IF
C
C$ Restrictions
C
C     This routine never returns logical unit numbers that are less
C     than or equal to 0.
C
C$ Author_and_Institution
C
C     N.J. Bachman    (JPL)
C     H.A. Neilan     (JPL)
C
C$ Literature_References
C
C     1. "Programming in VAX FORTRAN", Digital Equipment Corporation,
C        September 1984, Section 11.1.1, page 11-2.
C
C     2. "Microsoft FORTRAN Reference", Microsoft Corporation
C        1989, Section 3.2.2, page 61.
C
C     3. "Sun FORTRAN Programmer's Guide", Sun Microsystems,
C        Revision A of 6 May 1988, Section 7.2, page 73.
C
C$ Version
C
C-     SPICELIB Version 1.0.1, 20-MAR-1990 (HAN)
C
C         Parameters section was updated to include the values
C         of MINLUN and MAXLUN for several machines. Sources of
C         these values are listed in the Literature References
C         section.
C
C-     SPICELIB Version 1.0.0, 31-JAN-1990  (NJB)
C
C-&

C$ Revisions
C
C-     Beta Version 1.1.0, 09-MAR-1989  (HAN)
C
C         Declaration of the variable RETURN was removed from the code.
C         The variable was declared, but not used.
C
C-&





C
C     Local variables.
C
      LOGICAL          RESVD   ( MINLUN : MAXLUN )
      INTEGER          LAST
      LOGICAL          OPENED
      INTEGER          I
      INTEGER          IOSTAT


C
C     Save everything between calls.
C
      SAVE

C
C     Initial values
C
      DATA             LAST      / MINLUN /
      DATA             RESVD(5)  / .TRUE. /
      DATA             RESVD(6)  / .TRUE. /
      DATA             RESVD(7)  / .TRUE. /



C
C     Begin with the unit following the last one returned.
C     Cycle through the available units. Skip reserved units,
C     INQUIRE about others.
C
      DO 50001
     .   I = LAST+1, MAXLUN

         IF ( RESVD(I) ) THEN
            OPENED = .TRUE.
         ELSE
            INQUIRE ( UNIT=I, OPENED=OPENED, IOSTAT=IOSTAT )
         END IF

         IF ( IOSTAT .GT. 0 ) THEN
            UNIT = -IOSTAT
            RETURN
         END IF

         IF ( .NOT. OPENED ) THEN
            UNIT = I
            LAST = UNIT
            RETURN
         END IF

50001 CONTINUE

C
C     If we've come this far, we need to search the first part of
C     the list again, up to the last unit returned. Once again,
C     skip reserved units, INQUIRE about others.
C
      DO 50002
     .   I = MINLUN, LAST

         IF ( RESVD(I) ) THEN
            OPENED = .TRUE.
         ELSE
            INQUIRE ( UNIT=I, OPENED=OPENED, IOSTAT=IOSTAT )
         END IF

         IF ( IOSTAT .GT. 0 ) THEN
            UNIT = -IOSTAT
            RETURN
         END IF

         IF ( .NOT. OPENED ) THEN
            UNIT = I
            LAST = UNIT
            RETURN
         END IF

50002 CONTINUE

C
C     If we've come this far, there are no free units to be had.
C     C'est la vie. Assign 0 to the unit number.
C

      UNIT = 0

      RETURN




C$Procedure RESLUN ( Reserve a logical unit )

      ENTRY RESLUN ( UNIT )

C$ Abstract
C
C     Reserve a logical unit number. Reserved units are never returned
C     by FNDLUN or GETLUN.
C
C$ Required_Reading
C
C     None.
C
C$ Keywords
C
C     FILES
C
C$ Declarations
C
C     INTEGER               UNIT
C
C$ Brief_I/O
C
C     VARIABLE  I/O  DESCRIPTION
C     --------  ---  --------------------------------------------------
C     UNIT       I   Number of the logical unit to be reserved.
C
C$ Detailed_Input
C
C     UNIT      is the number of the logical unit to be reserved.
C               Once reserved, the unit number will not be returned
C               by the routines FNDLUN or GETLUN, even if it is not
C               connected to a file.
C
C               Units 5-7 are reserved by default.
C
C$ Detailed_Output
C
C     None.
C
C$ Parameters
C
C     None.
C
C$ Exceptions
C
C     Error free.
C
C$ Files
C
C     None.
C
C$ Particulars
C
C     FNDLUN maintains an array of logical flags, one for each positive
C     unit number offered by the system. RESLUN sets the value of the
C     flag for UNIT to TRUE.
C
C     Once reserved, units (except 5-7) may be unreserved by calling
C     FRELUN.
C
C$ Examples
C
C     The following code fragment illustrates the use of RESLUN.
C
C           C
C           C     Units 17-23 are used by non-NAIF file readers.
C           C     Reserve these, so that they will not be returned
C           C     by FNDLUN or GETLUN.
C           C
C                 DO I = 17, 23
C                    CALL RESLUN ( I )
C                 END DO
C
C$ Restrictions
C
C     None.
C
C$ Author_and_Institution
C
C     N.J. Bachman    (JPL)
C     C.A. Curzon     (JPL)
C     H.A. Neilan     (JPL)
C     I.M. Underwood  (JPL)
C
C$ Literature_References
C
C     None.
C
C$ Version
C
C-     SPICELIB Version 1.0.0, 31-JAN-1990 (CAC) (IMU)
C
C-&

C$ Revisions
C
C-     Beta Version 1.1.0, 27-FEB-1989 (HAN) (NJB)
C
C         This routine is now an entry point of FNDLUN rather than
C         GETLUN.  The code of this entry point itself has not changed
C         however. References to the routine FNDLUN were added to the
C         header.  The restrictions section was updated to read "none."
C         This module was declared "error free", which means
C         that it will never participate in error handling.
C
C-&


C
C     If UNIT is in the proper range, set the corresponding flag
C     to TRUE.
C
      IF ( UNIT .GE. MINLUN .AND. UNIT .LE. MAXLUN ) THEN
         RESVD(UNIT) = .TRUE.
      END IF

      RETURN




C$Procedure FRELUN ( Free a reserved logical unit )

      ENTRY FRELUN ( UNIT )

C$ Abstract
C
C     Unreserve a logical unit number reserved by RESLUN.
C
C$ Required_Reading
C
C     None.
C
C$ Keywords
C
C     FILES
C
C$ Declarations
C
C     INTEGER               UNIT
C
C$ Brief_I/O
C
C     VARIABLE  I/O  DESCRIPTION
C     --------  ---  --------------------------------------------------
C     UNIT       I   Number of the logical unit to be unreserved.
C
C$ Detailed_Input
C
C     UNIT      is the number of the logical unit to be unreserved.
C               Once unreserved, the unit number may be returned by
C               the routines GETLUN or FNDLUN whenever not connected to
C               a file.
C
C               Units 5-7 are reserved by default, and may not be
C               unreserved.
C
C$ Detailed_Output
C
C     None.
C
C$ Parameters
C
C     None.
C
C$ Exceptions
C
C     Error free.
C
C$ Files
C
C     None.
C
C$ Particulars
C
C     FNDLUN maintains an array of logical flags, one for each unit
C     offered by the system. FRELUN sets the value of the flag for
C     UNIT to FALSE.
C
C$ Examples
C
C     The following code fragment illustrates the use of FRELUN.
C
C           C
C           C     Free the units used by the non-NAIF file readers,
C           C     so that they may be returned by FNDLUN or GETLUN.
C           C
C                 DO I = 17, 23
C                    CALL FRELUN ( I )
C                 END DO
C
C$ Restrictions
C
C     None.
C
C$ Author_and_Institution
C
C     N.J. Bachman    (JPL)
C     C.A. Curzon     (JPL)
C     H.A. Neilan     (JPL)
C     I.M. Underwood  (JPL)
C
C$ Literature_References
C
C     None.
C
C$ Version
C
C-     SPICELIB Version 1.0.0, 31-JAN-1990 (CAC) (IMU)
C
C-&

C$ Revisions
C
C-     Beta Version 1.1.0, 27-FEB-1989 (HAN) (NJB)
C
C         This routine is now an entry point of FNDLUN rather than
C         GETLUN.  The code of this entry point itself has not changed
C         however. References to the routine FNDLUN were added to the
C         header.  The restrictions section was updated to read "none."
C         This module was declared "error free", which means
C         that it will never participate in error handling.
C
C-&


C
C     If UNIT is in the proper range, set the corresponding flag
C     to FALSE.
C
      IF (( UNIT .GE. MINLUN .AND. UNIT .LE. MAXLUN ) .AND.
     .    ( UNIT .LT. 5 .OR. UNIT .GT. 7 )) THEN

         RESVD(UNIT) = .FALSE.

      END IF

      RETURN

      END
