C$Procedure      INSSUB ( Insert a substring )

      SUBROUTINE INSSUB ( IN, SUB, LOC, OUT )

C$ Abstract
C
C      Insert a substring into a character string at a specified
C      location.
C
C$ Required_Reading
C
C     None.
C
C$ Keywords
C
C      ASSIGNMENT,  CHARACTER,  STRING
C
C$ Declarations

      CHARACTER*(*)    IN
      CHARACTER*(*)    SUB
      INTEGER          LOC
      CHARACTER*(*)    OUT

C$ Brief_I/O
C
C      VARIABLE  I/O  DESCRIPTION
C      --------  ---  --------------------------------------------------
C      IN         I   Input string.
C      SUB        I   Substring to be inserted.
C      LOC        I   Position at which substring is to be inserted.
C      OUT        O   Output string.
C
C$ Detailed_Input
C
C      IN          is an input character string, into which a substring
C                  is to be inserted.
C
C      SUB         is the substring to be inserted. Leading and trailing
C                  blanks are significant.
C
C      LOC         is the position in the input string at which the
C                  substring is to be inserted. To append to the
C                  string, set LOC equal to LEN(IN) + 1.
C
C$ Detailed_Output
C
C      OUT         is the output string. This is equivalent to the
C                  string that would be created by the concatenation
C
C                        OUT = IN(1:LOC-1) // SUB // IN(LOC: )
C
C                  If the output string is too long, it is truncated
C                  on the right.
C
C                  OUT may overwrite IN.
C
C$ Parameters
C
C     None.
C
C$ Particulars
C
C      Shift the end of the input string, beginning with LOC, to the
C      right, leaving space for the substring. Then insert the substring
C      into the vacated space in the middle of the string. This has
C      the same effect as the concatenation
C
C            OUT = IN(1:LOC-1) // SUB // IN(LOC: )
C
C      Because this operation is not standard for strings of length (*),
C      this routine does not use concatenation.
C
C$ Examples
C
C      The following examples illustrate the use of INSSUB.
C
C      IN                SUB      LOC    OUT
C      ----------------- -------  ---    ------------------------
C      'ABCDEFGHIJ'      ' YXZ '    3    'AB XYZ CDEFGHIJ'
C      'The rabbit'      'best '    5    'The best rabbit'
C      ' other woman'    'The'      1    'The other woman'
C      'An Apple a day'  ' keeps'  15    'An Apple a day keeps'
C      'Apple a day'     'An '      0     An error is signalled.
C      'Apple a day'     'An '     -3     An error is signalled.
C      'An Apple a day'  ' keeps'  16     An error is signalled.
C
C$ Restrictions
C
C      None.
C
C$ Exceptions
C
C     If LOC is not in the interval [1, LEN(IN)+1], the error
C     SPICE(INVALIDINDEX) is signalled.
C
C$ Files
C
C      None.
C
C$ Author_and_Institution
C
C      H.A. Neilan     (JPL)
C      I.M. Underwood  (JPL)
C
C$ Literature_References
C
C      None.
C
C$ Version
C
C-     SPICELIB Version 1.0.0, 31-JAN-1990 (IMU)
C
C-&



C$ Revisions
C
C-     Beta Version 2.0.0, 4-JAN-1989 (HAN)
C
C         If the location at which the substring is to be inserted is
C         not in the interval [1, LEN(IN)+1], an error is signalled.
C         Locations not within that interval refer to non-existent
C         characters positions. (To append to the string, set the
C         location equal to LEN(IN)+1.)
C
C-&




C
C     Spicelib functions
C
      LOGICAL          RETURN


C
C   Local Variables
C
      INTEGER          OUTLEN
      INTEGER          BEGIN
      INTEGER          SHIFT



C
C     Standard SPICE error handling.
C
      IF ( RETURN () ) THEN
         RETURN
      ELSE
         CALL CHKIN ( 'INSSUB' )
      END IF


C
C     We have to be careful not to exceed the length of the output
C     string when doing substring assignments.
C
      OUTLEN = LEN ( OUT )

C
C     If the insertion occurs after the end of the output string,
C     just return the original string.
C
      IF ( LOC .GT. OUTLEN ) THEN

         OUT = IN
         CALL CHKOUT ( 'INSSUB' )
         RETURN

C
C     If insertion occurs before the beginning of the string
C     or after LEN(IN) + 1, signal an error.
C
      ELSE IF ( ( LOC .LT. 1 ) .OR. ( LOC .GT. LEN ( IN ) + 1 ) ) THEN

         CALL SETMSG ( 'Location was *.'     )
         CALL ERRINT ( '*', LOC              )
         CALL SIGERR ( 'SPICE(INVALIDINDEX)' )
         CALL CHKOUT ( 'INSSUB' )
         RETURN

      ELSE

         BEGIN = LOC

      END IF

C
C     The size of the substring determines the size of the shift.
C
      SHIFT = LEN ( SUB )


C
C     The first part of the input string is copied without change
C     to the output string. (Watch for overrun.)
C
      IF ( BEGIN .GT. 1 ) THEN
         OUT(1 : MIN(BEGIN-1,OUTLEN)) = IN
      END IF

C
C     The part following the new substring is shifted into place.
C     And the new word is dropped into the middle.
C     (Again, watch for overrun.)
C
      IF ( BEGIN .LE. OUTLEN ) THEN

         CALL SHIFTC ( IN(BEGIN: ), 'R', SHIFT, ' ', OUT(BEGIN: ) )
         OUT(BEGIN : MIN(BEGIN+SHIFT-1,OUTLEN)) = SUB

      END IF


      CALL CHKOUT ( 'INSSUB' )
      RETURN
      END
