C$Procedure      NEXTWD ( Next word in a character string )

      SUBROUTINE NEXTWD ( STRING, NEXT, REST )

C$ Abstract
C
C      Return the next word in a given character string, and
C      left justify the rest of the string.
C
C$ Required_Reading
C
C     None.
C
C$ Keywords
C
C      CHARACTER,  PARSING,  WORD
C
C$ Declarations

      CHARACTER*(*)     STRING
      CHARACTER*(*)     NEXT
      CHARACTER*(*)     REST

C$ Brief_I/O
C
C      VARIABLE  I/O  DESCRIPTION
C      --------  ---  --------------------------------------------------
C      STRING     I   Input character string.
C      NEXT       O   The next word in the string.
C      REST       O   The remaining part of STRING, left-justified.
C
C$ Detailed_Input
C
C      STRING      is the input character string. This may be a list
C                  of items, a sentence, or anything else.
C
C$ Detailed_Output
C
C      NEXT        is the next word in STRING. A word is any sequence
C                  of consecutive non-blank characters. NEXT is always
C                  returned left-justified.
C
C                  If STRING is blank, NEXT is blank.
C
C                  NEXT may NOT overwrite STRING.
C
C     REST         is the remaining part of STRING, left-justified after
C                  the removal of NEXT.
C
C                  REST may overwrite STRING.
C
C$ Parameters
C
C     None.
C
C$ Exceptions
C
C      Error free.
C
C$ Files
C
C      None.
C
C$ Particulars
C
C      NEXTWD is used primarily for parsing input commands consisting
C      of one or more words, where a word is defined to be any sequence
C      of consecutive non-blank characters. Successive calls to NEXTWD,
C      each using the previous value of REST as the input string, allow
C      the calling routine to neatly parse and process one word at a
C      time.
C
C      NEXTWD cuts the input string into two pieces, and returns them
C      separately. The first piece is the first word in the string.
C      (Leading blanks are ignored. The next word runs from the first
C      non-blank character in the string up to the first blank that
C      follows it.) The second piece is whatever is left after the
C      first word is removed. The second piece is left justified,
C      to simplify later calls to NEXTWD.
C
C      If NEXT and REST are not large enough to hold the output
C      strings, they are truncated on the right.
C
C$ Examples
C
C      Let STRING be the following string:
C
C            '  Now is the time,  for all good men    to come.'
C
C      Then successive aplications of NEXTWD yield the following:
C
C            NEXT         REST
C            -----------  ----------------------------
C            'Now'        'is the time,  for all good men    to come.'
C            'is'         'the time,  for all good men    to come.'
C            'the'        'time,  for all good men    to come.'
C            'time,'      'for all good men    to come.'
C            'for'        'all good men    to come.'
C            'all'        'good men    to come.'
C            'good'       'men    to come.'
C            'men'        'to come.'
C            'to          'come.'
C            'come.'      ' '
C            ' '          ' '
C
C$ Restrictions
C
C      None.
C
C$ Literature_References
C
C      None.
C
C$ Author_and_Institution
C
C      I.M. Underwood  (JPL)
C
C$ Version
C
C-     SPICELIB Version 1.0.0, 31-JAN-1990 (IMU)
C
C-&



C
C     Local variables
C
      INTEGER          BEGIN
      INTEGER          END
      INTEGER          I



C
C     The trivial case.
C
      IF ( STRING .EQ. ' ' ) THEN

         NEXT = ' '
         REST = ' '

C
C     The non-trivial case.
C
      ELSE

C
C        Skip leading blanks.
C
         BEGIN = 1
50001    IF       ( STRING(BEGIN:BEGIN) .EQ. ' ' )
     .   THEN
            BEGIN = BEGIN + 1
            GO TO 50001
         END IF

C
C        Step through the input string until the next blank is
C        encountered, or until the end of the string is found.
C
         END = LEN ( STRING )

         I = BEGIN
50002    IF       ( I .LT. END .AND. STRING(I:I) .NE. ' ' )
     .   THEN
            I = I + 1
            GO TO 50002
         END IF

C
C        Most likely, we found a blank before the end of the string.
C        We just break the string at the location of the delimiter.
C
         IF ( I .LT. END ) THEN
            NEXT = STRING(BEGIN:I-1)
            REST = STRING(I+1  :   )

C
C       Otherwise, we just return the entire string (left-justified)
C       in NEXT, and nothing in REST.
C
         ELSE
            NEXT = STRING(BEGIN: )
            REST = ' '
         END IF

C
C        If there is anything in REST, we need to left-justify it.
C
         IF ( REST .NE. ' ' ) THEN
            CALL LJUST ( REST, REST )
         END IF

      END IF

      RETURN
      END
