
C$Procedure      OSCELT ( Determine conic elements from state )

      SUBROUTINE OSCELT ( STATE, ET, MU, ELTS )

C$ Abstract
C
C     Determine the set of osculating conic orbital elements that
C     corresponds to the state (position, velocity) of a body at
C     some epoch.
C
C$ Required_Reading
C
C     None.
C
C$ Keywords
C
C     CONIC
C     EPHEMERIS
C
C$ Declarations

      DOUBLE PRECISION      STATE  ( 6 )
      DOUBLE PRECISION      ET
      DOUBLE PRECISION      MU
      DOUBLE PRECISION      ELTS   ( 8 )

C$ Brief_I/O
C
C     VARIABLE  I/O  DESCRIPTION
C     --------  ---  --------------------------------------------------
C     STATE      I   State of body at epoch of elements.
C     ET         I   Epoch of elements.
C     MU         I   Gravitational parameter (GM) of primary body.
C     ELTS       O   Equivalent conic elements.
C
C$ Detailed_Input
C
C      STATE      is the state (position and velocity) of the body
C                 at some epoch. Components are x, y, z, dx/dt, dy/dt,
C                 dz/dt. Units are km and km/sec.
C
C      ET         is the epoch of the input state, in ephemeris seconds
C                 past J2000.
C
C                                                       3    2
C      MU         is the gravitational parameter (GM, km /sec ) of
C                 the primary body.
C
C$ Detailed_Output
C
C     ELTS        are equivalent conic elements describing the orbit
C                 of the body around its primary. The elements are,
C                 in order:
C
C                       RP      Perifocal distance.
C                       ECC     Eccentricity.
C                       INC     Inclination.
C                       LNODE   Longitude of the ascending node.
C                       ARGP    Argument of periapsis.
C                       M0      Mean anomaly at epoch.
C                       T0      Epoch.
C                       MU      Gravitational parameter.
C
C                 The epoch of the elements is the epoch of the input
C                 state. Units are km, rad, rad/sec. The same elements
C                 are used to describe all three types (elliptic,
C                 hyperbolic, and parabolic) of conic orbit.
C
C$ Parameters
C
C      None.
C
C$ Exceptions
C
C     None.
C
C$ Files
C
C     None.
C
C$ Particulars
C
C     None.
C
C$ Examples
C
C     Let VINIT contain the initial state of a spacecraft relative to
C     the center of a planet at epoch ET, and let GM be the gravitation
C     parameter of the planet. The call
C
C        CALL OSCELT ( VINIT, ET, GM, ELTS )
C
C     produces a set of osculating elements describing the nominal
C     orbit that the spacecraft would follow in the absence of all
C     other bodies in the solar system.
C
C     Now let STATE contain the state of the same spacecraft at some
C     other epoch, LATER. The difference between this state and the
C     state predicted by the nominal orbit at the same epoch can be
C     computed as follows.
C
C        CALL CONICS ( ELTS, LATER, NOMINAL )
C        CALL VSUBG  ( NOMINAL, STATE, 6, DIFF )
C
C        WRITE (*,*) 'Perturbation in x, dx/dt = ', DIFF(1), DIFF(4)
C        WRITE (*,*) '                y, dy/dt = ', DIFF(2), DIFF(5)
C        WRITE (*,*) '                z, dz/dt = ', DIFF(3), DIFF(6)
C
C$ Restrictions
C
C     None.
C
C$ Literature_References
C
C     [1] Roger Bate, Fundamentals of Astrodynamics, Dover, 1971.
C
C$ Author_and_Institution
C
C     I.M. Underwood  (JPL)
C
C$ Version
C
C-    SPICELIB Version 1.0.0, 31-JAN-1990 (IMU)
C
C-&


C$ Revisions
C
C-    Beta Version 1.0.1, 27-JAN-1989 (IMU)
C
C        Examples section completed.
C
C-&



C
C     SPICELIB functions
C
      LOGICAL               RETURN
      DOUBLE PRECISION      DACOSH
      DOUBLE PRECISION      EXACT
      DOUBLE PRECISION      PI
      DOUBLE PRECISION      TWOPI
      DOUBLE PRECISION      VNORM
      DOUBLE PRECISION      VDOT
      DOUBLE PRECISION      VSEP

C
C     Local parameters
C
      DOUBLE PRECISION      CLOSE
      PARAMETER           ( CLOSE = 1.D-10 )

C
C     Local variables
C
      DOUBLE PRECISION      R      ( 3 )
      DOUBLE PRECISION      V      ( 3 )
      DOUBLE PRECISION      H      ( 3 )
      DOUBLE PRECISION      N      ( 3 )
      DOUBLE PRECISION      E      ( 3 )

      DOUBLE PRECISION      P
      DOUBLE PRECISION      NU
      DOUBLE PRECISION      EA
      DOUBLE PRECISION      SENSE

      DOUBLE PRECISION      RP
      DOUBLE PRECISION      ECC
      DOUBLE PRECISION      INC
      DOUBLE PRECISION      LNODE
      DOUBLE PRECISION      ARGP
      DOUBLE PRECISION      M0




C
C     Standard SPICE error handling.
C
      IF ( RETURN () ) THEN
         RETURN
      ELSE
         CALL CHKIN ( 'OSCELT' )
      END IF


C
C     In order to convert a position and velocity to an equivalent
C     set of (osculating) orbital elements, we need to determine three
C     principal vectors associated with the orbit:
C
C        h         The angular momentum vector. This is perpendicular
C        -         to the plane of the orbit.
C
C                        h = r X v
C                        -   -   -
C
C        n         The node vector. This is perpendicular to the
C        -         normals of both the reference and orbital planes;
C                  it lies in the intersecton of these planes,
C                  pointing toward the ascending node.
C
C                             ^
C                        n  = k X h  = ( -h , h , 0 )
C                        -        -        x   y
C
C        e         The eccentricity vector. This lies in the plane
C        -         of the orbit, and points toward periapse. The
C                  magnitude of this vector is the eccentricity.
C
C                                      2
C                        e = (1/mu)( (v  - mu/r) r  -  <r,v> v )
C                        -                       -      - -  -
C
      CALL VEQU ( STATE(1), R )
      CALL VEQU ( STATE(4), V )

      CALL VCRSS ( R, V, H )

      CALL VPACK ( -H(2), H(1), 0.D0, N )

      CALL VLCOM ( VNORM(V)**2 - MU/VNORM(R), R, -VDOT ( R, V ), V, E )
      CALL VSCL  ( 1.D0/MU, E, E )


C
C     We begin by determining the size and shape of the orbit.
C
C     The eccentricity of the orbit is the magnitude of the
C     eccentricity vector. If the eccentricity is "close" to one,
C     go ahead and make this a parabola.
C
C     The perifocal distance depends on the eccentricity and the
C     semi-latus rectum, which in turn orbit depends only on the
C     specific angular momentum of the orbiting object.
C
      ECC = EXACT ( VNORM ( E ), 1.D0, CLOSE )

      P   = VDOT  ( H,H ) / MU
      RP  = P / (1.D0 + ECC)

C
C     Next, the orientation of the orbit.
C                                                       ^
C     The inclination of the orbit is the angle between k (which is
C     perpendicular to the equator) and h (which is perpendicular to
C     the orbit.                        -
C
C     If close to zero or pi, make it exact. In either case, the node
C     vector becomes undefined.
C
      INC  = ACOS ( H(3) / VNORM(H) )

      IF ( ABS ( INC - 0.D0 ) .LT. CLOSE ) THEN
         INC = 0.D0
         CALL VPACK ( 1.D0, 0.D0, 0.D0, N )

      ELSE IF ( ABS ( INC - PI() ) .LT. CLOSE ) THEN
         INC = PI()
         CALL VPACK ( 1.D0, 0.D0, 0.D0, N )
      END IF

C
C                                                              ^
C     The longitude of the ascending node is the angle between i
C     (the x-axis) and the node vector, n.
C                                       -
C
      LNODE = ATAN2 ( N(2), N(1) )

      IF ( LNODE .LT. 0.D0 ) THEN
         LNODE = LNODE + TWOPI()
      END IF

C
C     The argument of periapsis is the angle between the node and
C     eccentricity vectors, n and e. This is not defined for circular
C     orbits.               -     -
C
C     If periapsis is below the reference plane, the argument of
C     periapsis must be less than 180 degrees.
C
      IF ( ECC .EQ. 0.D0 ) THEN
         ARGP = 0.D0
      ELSE
         ARGP = VSEP ( N, E )

         IF ( E(3) .LT. 0.D0 ) THEN
            ARGP = TWOPI() - ARGP
         END IF
      END IF

C
C     And finally, the position of the object within the orbit.
C     The true anomaly, nu, is the angle between the eccentricity
C     and radius vectors, e and r. (For circular orbits, substitute
C     n for e.)           -     -
C     -     -
C
C     The sense of this angle is negative when the object is between
C     apoapsis and periapsis (<r,v> < 0).
C                              - -
C
      IF ( ECC .EQ. 0.D0 ) THEN
         NU    = VSEP ( N, R )
      ELSE
         NU = VSEP ( E, R )
      END IF

      IF ( VDOT ( R, V ) .LT. 0 ) THEN
         SENSE = -1.D0
      ELSE
         SENSE =  1.D0
      END IF

C
C     Unfortunately, the other element routines need the mean
C     anomaly, M. The true and mean anomalies are related through
C     the eccentric anomalies D (parabolas), E (ellipses), and
C     F (hyperbolas), as shown below.
C
C                      e + cos(nu)
C           cos(E)  = ---------------         (ellipse)
C                      1 + e cos(nu)
C
C           M       = E - e sin(E)
C
C
C                       e + cos(nu)
C           cosh(F) = ---------------         (hyperbola)
C                       1 + e cos(nu)
C
C           M       = e sinh(F) - F
C
C
C           D       = tan(nu/2)               (parabola)
C
C                           3
C           M       =  D + D / 3
C
C     For elliptic orbits, the mean anomaly should be in [0,2*pi].
C
      IF ( ECC .LT. 1.D0 ) THEN
         EA = ACOS ( (ECC + COS(NU)) / (1.D0 + ECC*COS(NU)) )
         M0 = ( EA - ECC*SIN(EA) ) * SENSE

         IF ( M0 .LT. 0.D0 ) THEN
            M0 = M0 + TWOPI()
         END IF

      ELSE IF ( ECC .GT. 1.D0 ) THEN
         EA = DACOSH ( (ECC + COS(NU)) / (1.D0 + ECC*COS(NU)) )
         M0 = ( ECC*SINH(EA) - EA ) * SENSE

      ELSE
         EA = TAN(NU/2.D0)
         M0 = ( EA + EA**3/3.D0 ) * SENSE
      END IF

C
C     Return the elements as a vector, suitable for input to CONICS.
C
      ELTS(1) = RP
      ELTS(2) = ECC
      ELTS(3) = INC
      ELTS(4) = LNODE
      ELTS(5) = ARGP
      ELTS(6) = M0
      ELTS(7) = ET
      ELTS(8) = MU


      CALL CHKOUT ( 'OSCELT' )
      RETURN
      END
