
C$Procedure      POOL ( Maintain a pool of kernel variables )

      SUBROUTINE POOL ( KERNEL, UNIT, NAME, N, VALUES, FOUND )

C$ Abstract
C
C     Maintain a pool of variables read from SPICE
C     ASCII kernel files.
C
C$ Required_Reading
C
C     KERNEL
C
C$ Keywords
C
C     CONSTANTS,  FILES
C
C$ Declarations

      CHARACTER*(*)         KERNEL
      INTEGER               UNIT
      CHARACTER*(*)         NAME
      INTEGER               N
      DOUBLE PRECISION      VALUES    ( * )
      LOGICAL               FOUND

      INTEGER               MAXVAR
      PARAMETER           ( MAXVAR =  500 )

      INTEGER               MAXVAL
      PARAMETER           ( MAXVAL = 2000 )

      INTEGER               MAXLEN
      PARAMETER           ( MAXLEN =   32 )

      CHARACTER*(*)         BEGDAT
      PARAMETER           ( BEGDAT = '\begindata' )

      CHARACTER*(*)         BEGTXT
      PARAMETER           ( BEGTXT = '\begintext' )

C$ Brief_I/O
C
C     Variable  I/O  Entry
C     --------  ---  --------------------------------------------------
C     KERNEL     I   LDPOOL
C     UNIT       I   WRPOOL
C     NAME       I   RTPOOL, EXPOOL
C     N          O   RTPOOL
C     VALUES     O   RTPOOL
C     FOUND      O   RTPOOL, EXPOOL
C     MAXVAR     P   (All)
C     MAXLEN     P   (All)
C     MAXVAL     P   (All)
C     BEGDAT     P   WRPOOL
C     BEGTXT     P   WRPOOL
C
C$ Detailed_Input
C
C     See the ENTRY points for a discussion of their arguments.
C
C$ Detailed_Output
C
C     See the ENTRY points for a discussion of their arguments.
C
C$ Parameters
C
C     MAXVAR      is the maximum number of variables that the
C                 kernel pool may contain at any one time.
C
C     MAXLEN      is the maximum length of the variable names
C                 that can be stored in the kernel pool.
C
C     MAXVAL      is the maximum number of distinct values that
C                 may belong to the variables in the kernel pool.
C                 Each variable must have at least one value, and
C                 may have any number, so long as the total number
C                 does not exceed MAXVAL. MAXVAL must be at least
C                 as large as MAXVAR.
C
C
C     BEGDAT      is the character string within the kernel file which
C                 indicates the beginning of a section of data. Not all
C                 FORTRAN 77 compilers accept this as valid text. This
C                 character string may have to be changed depending on
C                 the compiler that is used. For example, the string
C                 should be changed to "\\begindata" for UNIX FORTRAN 77
C                 compiler.
C
C     BEGTXT      is the character string within the kernel file which
C                 indicates the beginning of a section of text. Not all
C                 FORTRAN 77 compilers accept this as valid text. This
C                 character string may have to be changed depending on
C                 the compiler that is used. For example, the string
C                 should be changed to "\\begintext" for UNIX FORTRAN 77
C                 compiler.
C
C$ Files
C
C     See the ENTRY points for a discussion of their arguments.
C
C$ Exceptions
C
C     1) If POOL is called directly, the error SPICE(BOGUSENTRY) is
C        signalled.
C
C$ Particulars
C
C     POOL should never be called directly, but should instead be
C     accessed only through its entry points.
C
C     The purpose of this routine is to maintain a pool of variables
C     read from ASCII kernel files. The following entry points may be
C     used to access the pool.
C
C           CLPOOL         Clears the pool.
C
C           LDPOOL         Loads the variables in a kernel file into
C                          the pool.
C
C           RTPOOL         Returns the value of a variable from
C                          the pool.
C
C           EXPOOL         Confirms the existence of a variable in
C                          the pool.
C
C           WRPOOL         Writes the contents of the pool to an
C                          ASCII kernel file.
C
C     Nominally, the kernel pool contains up to 500 separate variables
C     and up to 2000 separate values. The names of the individual
C     variables may contain up to 32 characters. All of these figures
C     may be increased or decreased as necessary.
C
C$ Examples
C
C     The following code fragment demonstrates how the data from
C     several kernel files can be loaded into a kernel pool. After the
C     pool is loaded, the values in the pool are written to a kernel
C     file.
C
C     C
C     C     Store in an array the names of the kernel files whose
C     C     values will be loaded into the kernel pool.
C     C
C           KERNEL (1) = 'AXES.KER'
C           KERNEL (2) = 'GM.KER'
C           KERNEL (3) = 'LEAP_SECONDS.KER'
C
C     C
C     C     Clear the kernel pool.
C     C
C           CALL CLPOOL
C
C     C
C     C     Load the variables from the three kernel files into the
C     C     the kernel pool.
C     C
C           DO I = 1, 3
C             CALL LDPOOL ( KERNEL (I) )
C           END DO
C
C     C
C     C     We can examine the values associated with any variable
C     C     in the kernel pool using RTPOOL.
C     C
C           CALL RTPOOL ( VARIABLE, NUMVAL, VALUES, FOUND )
C
C     C
C     C     Get a free logical unit and open the file 'NEWKERNEL.KER'.
C     C
C           CALL GETLUN ( UNIT )
C
C           OPEN ( FILE            = 'NEWKERNEL.KER',
C          .       UNIT            = UNIT,
C          .       STATUS          = 'NEW',
C          .       IOSTAT          = IOSTAT,
C          .       CARRIAGECONTROL = 'LIST'  )
C
C     C
C     C     Write the values in the kernel pool to the file.
C     C
C           CALL WRPOOL ( UNIT )
C
C
C$ Restrictions
C
C     None.
C
C$ Literature_References
C
C     None.
C
C$ Author_and_Institution
C
C     N.J. Bachman    (JPL)
C     H.A. Neilan     (JPL)
C     I.M. Underwood  (JPL)
C
C$ Version
C
C-    SPICELIB Version 1.0.0, 31-JAN-1990 (IMU)
C
C-&


C$ Revisions
C
C-    Beta Version 2.0.0, 23-OCT-1989 (HAN)
C
C        Added declaration of FAILED. FAILED is checked in the
C        DO-loops in LDPOOL and WRPOOL to prevent infinite looping.
C
C-    Beta Version 1.2.0, 9-MAR-1989 (HAN)
C
C        Parameters BEGDAT and BEGTXT have been moved into the
C        Declarations section.
C
C-    Beta Version 1.1.0, 16-FEB-1989 (IMU) (NJB)
C
C        Parameters MAXVAR, MAXVAL, MAXLEN moved into Declarations.
C        (Actually, MAXLEN was implicitly 32 characters, and has only
C        now been made an explicit---and changeable---limit.)
C
C        Declaration of unused function FAILED removed.
C-&


C
C     SPICELIB functions
C
      LOGICAL               RETURN
      LOGICAL               FAILED
      INTEGER               SYDIMD

C
C     Local variables
C
      INTEGER               LBCELL
      PARAMETER           ( LBCELL = -5 )

      CHARACTER*(MAXLEN)    TABSYM   ( LBCELL:MAXVAR )
      SAVE                  TABSYM

      INTEGER               TABPTR   ( LBCELL:MAXVAR )
      SAVE                  TABPTR

      DOUBLE PRECISION      TABVAL   ( LBCELL:MAXVAL )
      SAVE                  TABVAL

      CHARACTER*(MAXLEN)    VARNAM
      LOGICAL               EOF
      LOGICAL               MORE
      INTEGER               IOSTAT
      INTEGER               I



C
C     Standard SPICE error handling.
C
      IF ( RETURN () ) THEN
         RETURN
      ELSE
         CALL CHKIN ( 'POOL' )
      END IF

C
C     This routine should never be called. If this routine is called,
C     an error is signalled.
C
      CALL SETMSG ( 'POOL: You have called an entry which performs '  //
     .              'performs no run-time function. This may '        //
     .              'indicate a bug. Please check the documentation ' //
     .              'for the subroutine POOL.' )

      CALL SIGERR ( 'SPICE(BOGUSENTRY)' )

      CALL CHKOUT ( 'POOL' )
      RETURN





C$Procedure CLPOOL ( Clear the pool of kernel variables )

      ENTRY CLPOOL

C$ Abstract
C
C     Remove all variables from the kernel pool.
C
C$ Required_Reading
C
C     KERNEL
C
C$ Keywords
C
C     CONSTANTS
C     FILES
C
C$ Declarations
C
C     None.
C
C$ Brief_I/O
C
C     None.
C
C$ Detailed_Input
C
C     None.
C
C$ Detailed_Output
C
C     None.
C
C$ Parameters
C
C     None.
C
C$ Exceptions
C
C     None.
C
C$ Files
C
C     None.
C
C$ Particulars
C
C     CLPOOL must be called prior to calling any of the other entry
C     points. CLPOOL clears the pool of kernel variables maintained by
C     the subroutine POOL. All the variables in the pool are deleted.
C
C$ Examples
C
C
C     The following code fragment demonstrates how the data from
C     several kernel files can be loaded into a kernel pool. After the
C     pool is loaded, the values in the pool are written to a kernel
C     file.
C
C
C     C
C     C     Store in an array the names of the kernel files whose
C     C     values will be loaded into the kernel pool.
C     C
C           KERNEL (1) = 'AXES.KER'
C           KERNEL (2) = 'GM.KER'
C           KERNEL (3) = 'LEAP_SECONDS.KER'
C
C     C
C     C     Clear the kernel pool.
C     C
C           CALL CLPOOL
C
C     C
C     C     Load the variables from the three kernel files into the
C     C     the kernel pool.
C     C
C           DO I = 1, 3
C             CALL LDPOOL ( KERNEL (I) )
C           END DO
C
C     C
C     C     We can examine the values associated with any variable
C     C     in the kernel pool using RTPOOL.
C     C
C           CALL RTPOOL ( VARIABLE, NUMVAL, VALUES, FOUND )
C
C     C
C     C     Get a free logical unit and open the file 'NEWKERNEL.KER'.
C     C
C           CALL GETLUN ( UNIT )
C
C           OPEN ( FILE            = 'NEWKERNEL.KER',
C          .       UNIT            = UNIT,
C          .       STATUS          = 'NEW',
C          .       IOSTAT          = IOSTAT,
C          .       CARRIAGECONTROL = 'LIST'  )
C
C
C     C
C     C     Write the values in the kernel pool to the file.
C     C
C           CALL WRPOOL ( UNIT )
C
C
C$ Restrictions
C
C     None.
C
C$ Literature_References
C
C     None.
C
C$ Author_and_Institution
C
C     I.M. Underwood  (JPL)
C
C$ Version
C
C-    SPICELIB Version 1.0.0, 31-JAN-1990 (IMU)
C
C-&


C$ Revisions
C
C-    Beta Version 1.1.0, 8-JAN-1989 (IMU)
C
C        Calls to EMPTYx were replaced with calls to SSIZEx, consistent
C        with the removal of the EMPTYx routines from the library.
C
C-&




C
C     Standard SPICE error handling.
C
      IF ( RETURN() ) THEN
         RETURN
      ELSE
         CALL CHKIN ( 'CLPOOL' )
      END IF


C
C     Just initialize the component tables.
C
      CALL SSIZEC ( MAXVAR, TABSYM )
      CALL SSIZEI ( MAXVAR, TABPTR )
      CALL SSIZED ( MAXVAL, TABVAL )


      CALL CHKOUT ( 'CLPOOL' )
      RETURN



C$Procedure LDPOOL ( Load variables from a kernel file into the pool )

      ENTRY LDPOOL ( KERNEL )

C$ Abstract
C
C     Load the variables contained in a NAIF ASCII kernel file into the
C     kernel pool.
C
C$ Required_Reading
C
C     KERNEL
C
C$ Keywords
C
C     CONSTANTS
C     FILES
C
C$ Declarations
C
C     CHARACTER*(*)         KERNEL
C
C$ Brief_I/O
C
C     VARIABLE  I/O  DESCRIPTION
C     --------  ---  --------------------------------------------------
C     KERNEL     I   Name of the kernel file.
C
C$ Detailed_Input
C
C     KERNEL     is the name of the kernel file whose variables will be
C                loaded into the pool.
C
C$ Detailed_Output
C
C     None.
C
C$ Parameters
C
C     None.
C
C$ Exceptions
C
C     None.
C
C$ Files
C
C     The NAIF ASCII kernel file KERNEL is opened by RDKNEW.
C
C$ Particulars
C
C     None.
C
C$ Examples
C
C
C     The following code fragment demonstrates how the data from
C     several kernel files can be loaded into a kernel pool. After the
C     pool is loaded, the values in the pool are written to a kernel
C     file.
C
C
C     C
C     C     Store in an array the names of the kernel files whose
C     C     values will be loaded into the kernel pool.
C     C
C           KERNEL (1) = 'AXES.KER'
C           KERNEL (2) = 'GM.KER'
C           KERNEL (3) = 'LEAP_SECONDS.KER'
C
C     C
C     C     Clear the kernel pool.
C     C
C           CALL CLPOOL
C
C     C
C     C     Load the variables from the three kernel files into the
C     C     the kernel pool.
C     C
C           DO I = 1, 3
C             CALL LDPOOL ( KERNEL (I) )
C           END DO
C
C     C
C     C     We can examine the values associated with any variable
C     C     in the kernel pool using RTPOOL.
C     C
C           CALL RTPOOL ( VARIABLE, NUMVAL, VALUES, FOUND )
C
C     C
C     C     Get a free logical unit and open the file 'NEWKERNEL.KER'.
C     C
C           CALL GETLUN ( UNIT )
C
C           OPEN ( FILE            = 'NEWKERNEL.KER',
C          .       UNIT            = UNIT,
C          .       STATUS          = 'NEW',
C          .       IOSTAT          = IOSTAT,
C          .       CARRIAGECONTROL = 'LIST'  )
C
C
C     C
C     C     Write the values in the kernel pool to the file.
C     C
C           CALL WRPOOL ( UNIT )
C
C
C$ Restrictions
C
C     CLPOOL must be called prior to calling LDPOOL.
C
C$ Literature_References
C
C     None.
C
C$ Author_and_Institution
C
C     R.E. Thurman    (JPL)
C     I.M. Underwood  (JPL)
C
C$ Version
C
C-    SPICELIB Version 1.0.0, 31-JAN-1990 (IMU)
C
C-&


C$ Revisions
C
C-    Beta Version 2.0.0, 18-OCT-1989 (RET)
C
C       A FAILED test was inserted into the control of the DO-loop which
C       reads in each kernel variable.
C
C       Previously, if the error action 'RETURN' had been set by a
C       calling program, and the call to RDKNEW by LDPOOL failed,
C       then execution would continue through LDPOOL, with SPICELIB
C       routines returning upon entry. This meant that the routine
C       RDKVAR never got a chance to set the EOF flag, which was the
C       only control of the DO-loop. An infinite loop resulted in such
C       cases.  The FAILED test resolves that situation.
C
C-&



C
C     Standard SPICE error handling.
C
      IF ( RETURN() ) THEN
         RETURN
      ELSE
         CALL CHKIN ( 'LDPOOL' )
      END IF


C
C     Open the kernel file and read the first variable.
C
      CALL RDKNEW ( KERNEL )
      CALL RDKVAR ( TABSYM, TABPTR, TABVAL, VARNAM, EOF )

C
C     Read the variables in the file, one at a time.
C
50001 IF       ( .NOT. EOF .AND. .NOT. FAILED() )
     .THEN
         CALL RDKVAR ( TABSYM, TABPTR, TABVAL, VARNAM, EOF )
         GO TO 50001
      END IF


      CALL CHKOUT ( 'LDPOOL' )
      RETURN




C$Procedure RTPOOL ( Return the value of a pooled kernel variable )

      ENTRY RTPOOL ( NAME, N, VALUES, FOUND )

C$ Abstract
C
C     Return the value of a kernel variable from the kernel pool.
C
C$ Required_Reading
C
C     KERNEL
C
C$ Keywords
C
C     CONSTANTS
C     FILES
C
C$ Declarations
C
C     CHARACTER*(*)         NAME
C     INTEGER               N
C     DOUBLE PRECISION      VALUES   ( * )
C     LOGICAL               FOUND
C
C$ Brief_I/O
C
C     VARIABLE  I/O  DESCRIPTION
C     --------  ---  --------------------------------------------------
C     NAME       I   Name of the variable whose value is to be returned.
C     N          O   Number of values associated with NAME.
C     VALUES     O   Values associated with NAME.
C     FOUND      O   True if variable is in pool.
C
C$ Detailed_Input
C
C     NAME       is the name of the variable whose values are to be
C                returned. If the variable is not in the pool, FOUND
C                will be FALSE.
C
C$ Detailed_Output
C
C     N          is the number of values associated with NAME.
C                If NAME is not in the pool, no value is given to
C                N.
C
C     VALUES     is the array of values associated with NAME.
C                If NAME is not in the pool, no values are given to
C                the elements of VALUES.
C
C     FOUND      is TRUE if the variable is in the pool, FALSE if it
C                is not.
C
C$ Parameters
C
C     None.
C
C$ Exceptions
C
C     None.
C
C$ Files
C
C     None.
C
C$ Particulars
C
C     None.
C
C$ Examples
C
C
C     The following code fragment demonstrates how the data from
C     several kernel files can be loaded into a kernel pool. After the
C     pool is loaded, the values in the pool are written to a kernel
C     file.
C
C
C     C
C     C     Store in an array the names of the kernel files whose
C     C     values will be loaded into the kernel pool.
C     C
C           KERNEL (1) = 'AXES.KER'
C           KERNEL (2) = 'GM.KER'
C           KERNEL (3) = 'LEAP_SECONDS.KER'
C
C     C
C     C     Clear the kernel pool.
C     C
C           CALL CLPOOL
C
C     C
C     C     Load the variables from the three kernel files into the
C     C     the kernel pool.
C     C
C           DO I = 1, 3
C             CALL LDPOOL ( KERNEL (I) )
C           END DO
C
C     C
C     C     We can examine the values associated with any variable
C     C     in the kernel pool using RTPOOL.
C     C
C           CALL RTPOOL ( VARIABLE, NUMVAL, VALUES, FOUND )
C
C     C
C     C     Get a free logical unit and open the file 'NEWKERNEL.KER'.
C     C
C           CALL GETLUN ( UNIT )
C
C           OPEN ( FILE            = 'NEWKERNEL.KER',
C          .       UNIT            = UNIT,
C          .       STATUS          = 'NEW',
C          .       IOSTAT          = IOSTAT,
C          .       CARRIAGECONTROL = 'LIST'  )
C
C
C     C
C     C     Write the values in the kernel pool to the file.
C     C
C           CALL WRPOOL ( UNIT )
C
C$ Restrictions
C
C     CLPOOL and LDPOOL must be called prior to calling RTPOOL.
C
C$ Literature_References
C
C     None.
C
C$ Author_and_Institution
C
C     I.M. Underwood  (JPL)
C
C$ Version
C
C-    SPICELIB Version 1.0.0, 31-JAN-1990 (IMU)
C
C-&



C
C     Standard SPICE error handling.
C
      IF ( RETURN() ) THEN
         RETURN
      ELSE
         CALL CHKIN ( 'RTPOOL' )
      END IF

C
C     Either it's there or it's not.
C
      CALL SYGETD ( NAME, TABSYM, TABPTR, TABVAL, N, VALUES, FOUND )


      CALL CHKOUT ( 'RTPOOL' )
      RETURN




C$Procedure EXPOOL ( Confirm the existence of a pooled kernel variable )

      ENTRY EXPOOL ( NAME, FOUND )

C$ Abstract
C
C     Confirm the existence of a kernel variable in the kernel pool.
C
C$ Required_Reading
C
C     KERNEL
C
C$ Keywords
C
C     CONSTANTS
C     FILES
C
C$ Declarations
C
C     CHARACTER*(*)         NAME
C     LOGICAL               FOUND
C
C$ Brief_I/O
C
C     VARIABLE  I/O  DESCRIPTION
C     --------  ---  --------------------------------------------------
C     NAME       I   Name of the variable whose value is to be returned.
C     FOUND      O   True when the variable is in the pool.
C
C$ Detailed_Input
C
C     NAME       is the name of the variable whose values are to be
C                returned.
C
C$ Detailed_Output
C
C     FOUND      is true whenever the specified variable is included
C                in the pool.
C
C$ Parameters
C
C     None.
C
C$ Exceptions
C
C     None.
C
C$ Files
C
C     None.
C
C$ Particulars
C
C     None.
C
C$ Examples
C
C     See BODFND.
C
C$ Restrictions
C
C     CLPOOL and LDPOOL must be called prior to calling EXPOOL.
C
C$ Literature_References
C
C     None.
C
C$ Author_and_Institution
C
C     I.M. Underwood  (JPL)
C
C$ Version
C
C-    SPICELIB Version 1.0.0, 31-JAN-1990 (IMU)
C
C-&



C
C     Standard SPICE error handling.
C
      IF ( RETURN() ) THEN
         RETURN
      ELSE
         CALL CHKIN ( 'EXPOOL' )
      END IF

C
C     Either it's there or it's not.
C
      FOUND = ( SYDIMD ( NAME, TABSYM, TABPTR, TABVAL ) .GT. 0 )

      CALL CHKOUT ( 'EXPOOL' )
      RETURN



C$Procedure WRPOOL ( Write the values in pool to a specified unit )

      ENTRY WRPOOL ( UNIT )

C$ Abstract
C
C     Write the values in the pool to the specified unit.
C
C$ Required_Reading
C
C     KERNEL
C
C$ Keywords
C
C     CONSTANTS
C     FILES
C
C$ Declarations
C
C     INTEGER        UNIT
C
C$ Brief_I/O
C
C     VARIABLE  I/O  DESCRIPTION
C     --------  ---  --------------------------------------------------
C     UNIT       I   Logical unit to which the values in the pool will
C                    be written.
C
C$ Detailed_Input
C
C     UNIT       is the logical unit to which the values in the pool
C                will be written.
C
C$ Detailed_Output
C
C     None.
C
C$ Parameters
C
C     None.
C
C$ Files
C
C     If the values are to be written to an output kernel file, the
C     file should be opened with a logical unit determined by the
C     calling program.
C
C$ Exceptions
C
C     None.
C
C$ Particulars
C
C     None.
C
C$ Examples
C
C
C     The following code fragment demonstrates how the data from
C     several kernel files can be loaded into a kernel pool. After the
C     pool is loaded, the values in the pool are written to a kernel
C     file.
C
C
C     C
C     C     Store in an array the names of the kernel files whose
C     C     values will be loaded into the kernel pool.
C     C
C           KERNEL (1) = 'AXES.KER'
C           KERNEL (2) = 'GM.KER'
C           KERNEL (3) = 'LEAP_SECONDS.KER'
C
C     C
C     C     Clear the kernel pool.
C     C
C           CALL CLPOOL
C
C     C
C     C     Load the variables from the three kernel files into the
C     C     the kernel pool.
C     C
C           DO I = 1, 3
C             CALL LDPOOL ( KERNEL (I) )
C           END DO
C
C     C
C     C     We can examine the values associated with any variable
C     C     in the kernel pool using RTPOOL.
C     C
C           CALL RTPOOL ( VARIABLE, NUMVAL, VALUES, FOUND )
C
C     C
C     C     Get a free logical unit and open the file 'NEWKERNEL.KER'.
C     C
C           CALL GETLUN ( UNIT )
C
C           OPEN ( FILE            = 'NEWKERNEL.KER',
C          .       UNIT            = UNIT,
C          .       STATUS          = 'NEW',
C          .       IOSTAT          = IOSTAT,
C          .       CARRIAGECONTROL = 'LIST'  )
C
C
C     C
C     C     Write the values in the kernel pool to the file.
C     C
C           CALL WRPOOL ( UNIT )
C
C
C$ Restrictions
C
C     CLPOOL and LDPOOL must be called prior to calling WRPOOL.
C
C$ Literature_References
C
C     None.
C
C$ Author_and_Institution
C
C     H.A. Neilan     (JPL)
C     I.M. Underwood  (JPL)
C
C$ Version
C
C-     SPICELIB Version 1.0.0, 31-JAN-1990 (IMU)
C
C-&

C$ Revisions
C
C-     Beta Version 2.0.0, 23-OCT-1989 (HAN)
C
C        A FAILED test was added to the DO-loop which fetches a
C        symbol in the kernel pool and writes it to a file.
C        If the error action was set to 'RETURN' and writing the
C        variable and value to the file failed, an infinte loop
C        resulted.
C
C-&


C
C     Standard SPICE error handling.
C
      IF ( RETURN() ) THEN
         RETURN
      ELSE
         CALL CHKIN ( 'WRPOOL' )
      END IF


C
C     Indicate the beginning of a data section.
C
      WRITE (UNIT,*,IOSTAT=IOSTAT) BEGDAT


C
C     Obtain the first variable name.
C
      I = 1
      CALL SYFETD ( I, TABSYM, TABPTR, TABVAL, VARNAM, MORE )


C
C     While there are variables in the pool, write the
C     variables and their values to the specified unit.
C
50002 IF       ( MORE .AND. ( .NOT. FAILED() ) )
     .THEN

         WRITE (UNIT,*,IOSTAT=IOSTAT)
         CALL WRKVAR ( UNIT,   VARNAM, '=',
     .                 TABSYM, TABPTR, TABVAL )

         I = I + 1
         CALL SYFETD ( I, TABSYM, TABPTR, TABVAL, VARNAM, MORE )

         GO TO 50002
      END IF


C
C     Indicate the beginning of a text section. Data sections and
C     text sections must alternate, even if the text section is blank.
C
      WRITE (UNIT,*,IOSTAT=IOSTAT) BEGTXT

      CALL CHKOUT ( 'WRPOOL' )
      RETURN
      END

