
C$Procedure      RDTEXT ( Read a line from a text file )

      SUBROUTINE RDTEXT ( FILE, LINE, EOF )

C$ Abstract
C
C      Read the next line of text from a text file.
C
C$ Required_Reading
C
C     None.
C
C$ Keywords
C
C      FILES,  TEXT
C
C$ Declarations

      CHARACTER*(*)       FILE
      CHARACTER*(*)       LINE
      LOGICAL             EOF

      INTEGER             MAXOPN
      PARAMETER         ( MAXOPN = 64  )

      INTEGER             MAXLEN
      PARAMETER         ( MAXLEN = 128 )

C$ Brief_I/O
C
C      VARIABLE  I/O  DESCRIPTION
C      --------  ---  --------------------------------------------------
C      FILE       I   Input text file.
C      LINE       O   Next line from the input text file.
C      EOF        O   End-of-file indicator.
C      MAXOPN     P   Maximum number of open files.
C      MAXLEN     P   Maximum file name length.
C
C$ Detailed_Input
C
C      FILE        is the name of the text file from which the next
C                  line is to be read. If the file is not currently
C                  open, it is opened with a logical unit determined
C                  at run time, and the first line of the file is
C                  returned. Otherwise, the next line not yet read
C                  from the file is read and returned.
C
C$ Detailed_Output
C
C      LINE        is next line of text in the specified file.
C                  If the end of the file is reached, LINE is blank.
C
C      EOF         is true when the end of the file is reached, and is
C                  otherwise false.
C
C$ Parameters
C
C      MAXOPN      is the maximum number of files that can be kept
C                  open simultaneously by RDTEXT.
C
C                  VAX:
C
C                  The default number of files that can be open at one
C                  time during a user's process is determined by the
C                  value of FILLM. This number is usually 20, but it
C                  may be changed by a user with sufficient privileges.
C
C                  IBM PC / Microsoft FORTRAN 5.0:
C
C                  The default value for the maximum number of files
C                  open at one time is 20. This value may be changed
C                  by modifying the appropriate startup files as
C                  specified in the reference documentation.
C
C                  Sun / FORTRAN 1.2:
C
C                  "The maximum number of logical units that a program
C                  can have open at one time is the same as the SunOS
C                  system limit, currently 64."
C
C      MAXLEN      is the maximum length of the file names that may
C                  used to identify the files opened by RDTEXT.
C
C$ Particulars
C
C      RDTEXT reads the next line from a text file. If the file is
C      not currently open, it is opened with a logical unit determined
C      at run time, and the first line of the file is returned.
C      Otherwise, the next line not yet read from the file is returned.
C
C      If the end of the file is reached, a blank line is returned,
C      the end-of-file indicator is true, and the file is closed.
C
C      Several files may be opened and read simultaneously. Thus,
C      you may begin reading from one file before the end of another
C      file has been reached. RDTEXT maintains a separate file pointer
C      for each file.
C
C$ Examples
C
C      Let FILE.1 contain the following lines.
C
C         Mary had a little lamb
C         Everywhere that Mary went
C
C      Let FILE.2 contain the following lines.
C
C         Its fleece was white as snow.
C         The lamb was sure to go.
C
C      Then the code fragment
C
C         DO I = 1, 2
C            CALL RDTEXT ( 'FILE.1', LINE, EOF )
C            WRITE (6,*) LINE
C
C            CALL RDTEXT ( 'FILE.2', LINE, EOF )
C            WRITE (6,*) LINE
C         END DO
C
C      produces the following output
C
C         Mary had a little lamb
C         Its fleece was white as snow.
C         Everywhere that Mary went
C         The lamb was sure to go.
C
C$ Restrictions
C
C      1) The values of MAXOPN and MAXLEN should not exceed any
C         corresponding limits imposed by the operating system.
C
C      2) If the input file is a print file, the carriage control
C         character at the beginning of a given line will be considered
C         part of the line. (Text files have no carriage control
C         characters.)
C
C      3) In order to avoid access violations, the nominal version of
C         RDTEXT uses the VAX READONLY qualifier to open files. This
C         must be removed or replaced when the routine is ported.
C
C$ Exceptions
C
C      1) If too many files are open already, the error
C         SPICE(TOOMANYFILESOPEN) is signalled.
C
C      2) If the attempt to open the file fails, the error
C         SPICE(FILEOPENFAILED) is signalled.
C
C      3) If the attempt to read from the file fails, the error
C         SPICE(FILEREADFAILED) is signalled.
C
C$ Files
C
C      See input FILE.
C
C$ Author_and_Institution
C
C      N.J. Bachman    (JPL)
C      I.M. Underwood  (JPL)
C
C$ Literature_References
C
C      1. "VAX/VMS Guide to VAX/VMS System Management and Daily
C         Operations", Digital Equipment Corporation, September 1984,
C         Section 6.1.7, page 6-6.
C
C      2. "Microsoft FORTRAN Reference", Microsoft Corporation
C         1989, Section C.3, page 404.
C
C      3. "Sun FORTRAN Programmer's Guide", Sun Microsystems,
C         Revision A of 6 May 1988, Section 7.2, page 73.
C
C$ Version
C
C-     SPICELIB Version 1.0.1, 20-MAR-1990 (HAN)
C
C         Parameters section was updated to include the values
C         of MAXOPN for several machines. Sources of these values
C         are listed in the Literature References section.
C
C-     SPICELIB Version 1.0.0, 31-JAN-1990 (IMU)
C
C-&


C$ Revisions
C
C-     Beta Version 1.1.0, 17-FEB-1989 (IMU) (NJB)
C
C        The primary change was the addition of error handling.
C        At the same time, the parameters MAXOPN and MAXLEN were
C        moved into the calling sequence.  The call to IOERR was
C        replaced by a call to SETMSG.  The declaration of the unused
C        function FAILED was deleted. Finally, all internal references
C        to the entry point WRTEXT (which was dropped when the routine
C        left OPTLIB) were removed.
C
C
C-&



C
C     SPICELIB functions
C
      INTEGER             ISRCHC
      LOGICAL             RETURN

C
C     Local variables
C
      CHARACTER*(MAXLEN)  FILES   ( MAXOPN )
      INTEGER             UNITS   ( MAXOPN )
      INTEGER             UNIT

      INTEGER             N
      INTEGER             INDEX
      INTEGER             IOSTAT
      INTEGER             I


C
C     Save the names of the files, and their associated logical units.
C
      SAVE                FILES
      SAVE                UNITS
      SAVE                N



C
C     Set up the error processing.
C
      IF ( RETURN () ) THEN
         RETURN
      ELSE
         CALL CHKIN ( 'RDTEXT' )
      END IF

C
C     Is this a new file?
C
      INDEX = ISRCHC ( FILE, N, FILES )

      IF ( INDEX .EQ. 0 ) THEN

C
C        We will need a free logical unit. But only if we don't
C        have too many files open already.
C
         IF ( N .EQ. MAXOPN ) THEN
            CALL SETMSG ( 'Too many files open already.' )
            CALL SIGERR ( 'SPICE(TOOMANYFILESOPEN)'      )
            CALL CHKOUT ( 'RDTEXT' )
            RETURN
         ELSE
            CALL GETLUN ( UNIT )
         END IF

C
C        Okay, we have a unit. Open the file, and hope nothing
C        goes awry. The READONLY qualifier is nonstandard, but
C        helpful where allowed. (Standard disclaimer.)
C
         OPEN ( FILE            = FILE,
     .          UNIT            = UNIT,
     .          STATUS          = 'OLD',
     .          IOSTAT          = IOSTAT )

         IF ( IOSTAT .NE. 0 ) THEN
            CALL SETMSG ( 'Could not open #.'     )
            CALL ERRCH  ( '#', FILE               )
            CALL SIGERR ( 'SPICE(FILEOPENFAILED)' )
            CALL CHKOUT ( 'RDTEXT' )
            RETURN
         END IF

C
C        Whew! We're ready to read from this file. Save
C        the pertinent information:
C
C            - The number of files currently open.
C            - The name of this file.
C            - The logical unit connected to this file.
C            - The index of the file within the FILES array.
C
         N         = N + 1
         FILES(N)  = FILE
         UNITS(N)  = UNIT
         INDEX     = N

      END IF



C
C     This is the easy part. Read the next line from the file.
C
      READ (UNITS(INDEX),FMT='(A)',IOSTAT=IOSTAT) LINE

C
C     Well, what happened? An end-of-file condition is indicated by
C     a negative value for IOSTAT. Any other non-zero value indicates
C     some other error. In any event, close the file immediately.
C     Repack the UNITS and FILES arrays, so that subsequent
C     calls will not try to read from the file without reopening it.
C
      EOF = ( IOSTAT .LT. 0 )

      IF ( IOSTAT .NE. 0 ) THEN

         CLOSE ( UNITS(INDEX) )

         DO 50001
     .      I = INDEX+1, N
            FILES(I-1)  = FILES(I)
            UNITS(I-1)  = UNITS(I)
50001    CONTINUE

         N = N - 1

C
C        If this is just the end of the file, don't report an error.
C        (All files have to end sometime.)
C
         IF ( .NOT. EOF ) THEN
            CALL SETMSG ( 'Could not read from #.' )
            CALL ERRCH  ( '#', FILES(INDEX)        )
            CALL SIGERR ( 'SPICE(FILEREADFAILED)'  )
            CALL CHKOUT ( 'RDTEXT' )
            RETURN
         END IF

      END IF

      CALL CHKOUT ( 'RDTEXT' )
      RETURN




C$Procedure  CLTEXT ( Close a text file opened by RDTEXT)

      ENTRY  CLTEXT ( FILE )

C$ Abstract
C
C      Close a text file currently opened by RDTEXT.
C
C$ Required_Reading
C
C     None.
C
C$ Keywords
C
C      FILES,  TEXT
C
C$ Declarations
C
C      None.
C
C$ Brief_I/O
C
C      VARIABLE  I/O  DESCRIPTION
C      --------  ---  --------------------------------------------------
C      FILE       I   Text file to be closed.
C
C$ Detailed_Input
C
C      FILE        is the name of a text file which is currently
C                  opened for reading or writing by RDTEXT.
C
C$ Detailed_Output
C
C      None.
C
C$ Parameters
C
C      None.
C
C$ Exceptions
C
C     None.
C
C$ Files
C
C      The text file, FILE, was previously opened by RDTEXT.
C
C$ Particulars
C
C      CLTEXT closes one of the files currently opened for reading or
C      writing by RDTEXT. If the specified file is not open, nothing
C      happens.
C
C$ Examples
C
C      None.
C
C$ Restrictions
C
C      None.
C
C$ Author_and_Institution
C
C      I.M. Underwood  (JPL)
C
C$ Literature_References
C
C      None.
C
C$ Version
C
C-    SPICELIB Version 1.0.0, 31-JAN-1990 (IMU)
C
C-&


C$ Revisions
C
C-    Beta Version 1.1.0, 8-JAN-1989 (IMU)
C
C        References to WRTEXT removed.
C
C-&



C
C     Set up the error processing.
C
      IF ( RETURN () ) THEN
         RETURN
      ELSE
         CALL CHKIN ( 'CLTEXT' )
      END IF


C
C     Which file?
C
      INDEX = ISRCHC ( FILE, N, FILES )

      IF ( INDEX .GT. 0 ) THEN

         CLOSE ( UNITS(INDEX) )

C
C        Remember all that salient information about the file?
C        Lose it.
C
         DO 50002
     .      I = INDEX+1, N
            FILES(I-1)  = FILES(I)
            UNITS(I-1)  = UNITS(I)
50002    CONTINUE

         N = N - 1

      END IF

      CALL CHKOUT ( 'CLTEXT' )

      RETURN
      END
