C$Procedure      RESET ( Reset Error Status )

      SUBROUTINE RESET

C$ Abstract
C
C      Reset the SPICELIB error status to a value of
C      "no error."   As a result, the status routine, FAILED,
C      will return a value of .FALSE.
C
C$ Required_Reading
C
C      ERROR
C
C$ Keywords
C
C      ERROR
C
C$ Declarations
C
C      None.
C
C$ Brief_I/O
C
C      VARIABLE  I/O  DESCRIPTION
C      --------  ---  --------------------------------------------------
C      None.
C
C$ Detailed_Input
C
C      None.
C
C$ Detailed_Output
C
C      None.
C
C$ Parameters
C
C     None.
C
C$ Exceptions
C
C      This routine does not detect any errors.
C
C      However, this routine is part of the SPICELIB error
C      handling mechanism.
C
C$ Files
C
C      None.
C
C$ Particulars
C
C      Please read the "required reading" first!
C
C      The effects of this routine are:
C
C      1. The SPICELIB status is set to a value of "no error."
C
C      2. The long and short error messages are set to blank.
C
C      3. Setting of the long error message is re-enabled.
C
C      4. The frozen traceback is set equal to the current version.
C
C      Subsequent to a call to RESET, references to the status
C      indicator function, FAILED, will return a value of .FALSE.,
C      until an error is detected.
C
C      This routine should be called in cases where one wishes
C      to attempt to continue processing after detection of an
C      error, and the 'RETURN' error action is being used.  When
C      the error response action is set to 'RETURN', routines
C      that have external references, or that can
C      detect errors, return immediately upon entry when an
C      error condition exists.  This prevents a program from
C      crashing, but does not allow for a recovery attempt.
C
C      If one does wish to attempt to recover,
C      in general the procedure is to test for an error
C      condition, and if one exists, respond to the error
C      (by outputting diagnostic messages, for example).  Next,
C      a call to RESET can be made.  After resetting the
C      error status, the normal execution thread can be resumed.
C
C      It is also appropriate to call this routine when the error
C      response action is 'REPORT', if one wishes to recover
C      from errors.
C
C
C
C$ Examples
C
C      1.  In this example, we try to read a line from the file,
C          SPUD.DAT, using the toolkit routine, RDTEXT.
C          When FAILED indicates an error, we grab the short
C          error message and its explanation, using GETMSG (see),
C          log the messages using our user-defined routine,
C          USER_LOG (NOT a SPICELIB routine), reset the
C          status, and keep going.
C
C      C
C      C      We read a line from SPUD.DAT:
C      C
C
C             CALL RDTEXT ( 'SPUD.DAT', LINE, EOF )
C
C             IF ( FAILED() ) THEN
C      C
C      C         Oops! an error occurred during the read.
C      C         Recover the short error message and its
C      C         explanation, reset the error status,
C      C         log the messages, and continue...
C      C
C
C                CALL GETMSG   ( 'SHORT'    ,    SMSG )
C                CALL GETMSG   ( 'EXPLAIN'  ,    EXPL )
C
C                CALL USER_LOG (  SMSG )
C                CALL USER_LOG (  EXPL )
C
C                CALL RESET
C
C             END IF
C
C
C
C$ Restrictions
C
C      It can be dangerous to call this routine without
C      RESPONDING to the error condition first; by calling
C      RESET, you are wiping out the SPICELIB's knowledge of
C      the error.
C
C$ Literature_References
C
C      None.
C
C$ Author_and_Institution
C
C      N.J. Bachman    (JPL)
C
C$ Version
C
C-     SPICELIB Version 1.0.0, 31-JAN-1990 (NJB)
C
C-&


C
C     SPICELIB functions
C

      LOGICAL               SETERR
      LOGICAL               ACCEPT

C
C     Local Variables:
C

      LOGICAL               STAT

C
C     Executable Code:
C


C
C     This odd-looking function reference resets the error
C     status to indicate "no error":
C


      STAT = SETERR ( .FALSE. )

C
C     Wipe out the short and long error messages:
C

      CALL PUTSMS ( ' ' )

      CALL PUTLMS ( ' ' )

C
C     Allow long error message to be updated:
C

      STAT = ACCEPT ( .TRUE. )

C
C     Reset the frozen traceback:
C

      CALL FREEZE


      END
