C$Procedure      SIGERR ( Signal Error Condition )

      SUBROUTINE SIGERR ( MSG )

C$ Abstract
C
C     Inform the SPICELIB error processing mechanism that an error has
C     occurred, and specify the type of error.
C
C$ Required_Reading
C
C     ERROR
C
C$ Keywords
C
C     ERROR
C
C$ Declarations

      CHARACTER*(*)                 MSG

C$ Brief_I/O
C
C      VARIABLE  I/O  DESCRIPTION
C      --------  ---  --------------------------------------------------
C      MSG        I   A short error message.
C
C$ Detailed_Input
C
C      MSG     A ``short'' error message.
C              MSG indicates the type of error that has occurred.
C
C              The exact format that MSG must follow is
C              described in the required reading file, ERROR.REQ.
C              Only the first 25 characters of MSG will be stored;
C              additional characters will be truncated.
C
C              Generally, MSG will be stored internally by the SPICELIB
C              error handling mechanism.  The only exception
C              is the case in which the user has commanded the error
C              handling mechanism to ``ignore'' the error indicated by
C              MSG.
C
C              As a default, MSG will be output to the screen.
C              See the required reading file for a discussion of how
C              to customize SPICELIB error handling behavior, and
C              in particular, the disposition of MSG.
C
C$ Detailed_Output
C
C      None.
C
C$ Parameters
C
C     None.
C
C$ Exceptions
C
C      This routine does not detect any errors.
C
C      However, this routine is part of the interface to the
C      SPICELIB error handling mechanism.  For this reason,
C      this routine does not participate in the trace scheme,
C      even though it has external references.
C
C
C$ Files
C
C      None.
C
C$ Particulars
C
C      First of all, please read the ``required reading'' file.
C      The information below will make a lot more sense if you do.
C
C      This is the routine used by SPICELIB to signal the detection
C      of errors.
C
C      Making a call to SIGERR is the way to inform the error
C      handling mechanism that an error has occurred.
C
C      Specifically, the effects of this routine are:
C
C      1.  If responding to the error indicated by MSG has
C          not been disabled:
C
C          a. MSG will be stored internally.  As a result,
C             The SPICELIB routine, GETMSG, will be able to
C             retrieve MSG, until MSG has been ``erased''
C             by a call to RESET, or overwritten by another
C             call to SIGERR.
C
C          b. An indication of an ``error condition'' will
C             be set internally.  The SPICELIB logical
C             function, FAILED, will take the value, .TRUE.,
C             as a result, until the error condition is
C             negated by a call to RESET.
C
C          c. All of the error messages that have been selected
C             for automatic output via ERRPRT will be output.
C             The set of messages is some subset of { short message,
C             long message, explanation of short message,
C             traceback, and default message }.
C
C          d. If the error response mode is not 'RETURN',
C             Setting of the long error message is enabled.
C             You can't re-set the long error message, once
C             it has been set, without first signalling an error.
C
C          e. In 'RETURN' mode, further signalling of error
C             messages, and setting of the long message, are disabled.
C             (These capabilities can be re-enabled by calling RESET).
C
C
C      2.  If the error handling mechanism has been commanded to
C          ``ignore'' the error indicated by MSG, the call to SIGERR
C          has no effect.
C
C      If you wish to set the long error message, call
C      SETMSG BEFORE calling SIGERR.
C
C
C$ Examples
C
C
C  1.  In the following example, N is supposed to be less than
C      MAXLUN.  If it isn't, an error condition exists.
C
C      C
C      C      We will need a free logical unit.  But only if we don't
C      C      have too many files open already.
C      C
C
C             IF ( N .EQ. MAXLUN ) THEN
C
C                CALL SIGERR ( 'SPICE(TOOMANYFILESOPEN)' )
C                RETURN
C
C             END IF
C
C
C  2.  This time, we want to set the long error message, too.
C
C
C
C             IF ( N .EQ. MAXLUN ) THEN
C
C                CALL SETMSG ( 'RDTEXT:  Can't open another file; ' //
C            .                 'max number of files open at once '  //
C            .                 'for reading by RDTEXT is 20'  )
C
C                CALL SIGERR ( 'SPICE(TOOMANYFILESOPEN)' )
C                RETURN
C
C             END IF
C
C
C$ Restrictions
C
C      None.
C
C$ Literature_References
C
C      None.
C
C$ Author_and_Institution
C
C      N.J. Bachman    (JPL)
C
C$ Version
C
C-     SPICELIB Version 1.0.0, 31-JAN-1990 (NJB)
C
C-&


C
C     SPICELIB functions:
C

      LOGICAL               FAILED
      LOGICAL               SETERR
      LOGICAL               ACCEPT

C
C     Local Variables:
C

      INTEGER               ACTLEN
      PARAMETER           ( ACTLEN = 7 )

      CHARACTER*(ACTLEN)    ACTION

      LOGICAL               STAT


C
C     Executable Code:
C

C
C     We must first check whether the error indicated by
C     MSG is one we're supposed to ignore...
C
C     There are two cases in which we do not want to respond
C     to the signalled error.
C
C     1.  When the error action is 'IGNORE'.  The user has
C         commanded that all messages be ignored.
C
C     2.  When the error action is 'RETURN', and an error
C         condition already exists.  We wish to preserve the
C         error data from the FIRST error until the user/
C         user's program has reset the error status via
C         a call to RESET.
C

      CALL GETACT ( ACTION )

      IF  (    (  ACTION .NE. 'IGNORE')  .AND. .NOT.

     .         (  ACTION .EQ. 'RETURN'   .AND.  FAILED()  )

     .    ) THEN

C
C        This one's for real.  Indicate an error condition, and
C        store the short error message.
C
C        Note:  the following strange -- looking function
C        reference sets the toolkit error status.  STAT
C        doesn't have any meaning.
C

         STAT = SETERR ( .TRUE. )

         CALL PUTSMS ( MSG )


C
C        Create a frozen copy of the traceback:
C

         CALL FREEZE

C
C        Now we output the error data that are available at this time,
C        and whose output has been enabled.  The choice of data
C        is any combination of the following:
C
C           1. The short error message
C           2. The explanation of the short error message
C           3. The traceback
C           4. The long error message
C           5. The default message
C
C        Note that OUTMSG outputs only those messages which have
C        been SELECTED for output, via a call to ERRPRT, except
C        if the error action is DEFAULT.  In that case, the
C        default message selection applies.
C

         IF (  ACTION .NE. 'DEFAULT' ) THEN

            CALL OUTMSG ( 'SHORT, EXPLAIN, LONG, TRACEBACK' )

         ELSE

            CALL OUTMSG ( 'SHORT, EXPLAIN, LONG, TRACEBACK, DEFAULT' )

         END IF


         IF (  ACTION .EQ. 'RETURN' )  THEN
C
C           Don't acceptance of new long error messages or updates
C           to current long error message:
C           (STAT has no meaning).
C
            STAT  =  ACCEPT ( .FALSE. )

         ELSE

            STAT  =  ACCEPT ( .TRUE. )

         END IF



      ELSE IF ( ACTION .NE. 'IGNORE' )  THEN

         STAT  =  ACCEPT ( .FALSE. )

      END IF

C
C     We could be in ABORT or DEFAULT mode.
C

      IF (  ( ACTION .EQ. 'DEFAULT' )   .OR.
     .      ( ACTION .EQ. 'ABORT'   )        )   THEN

         STOP

      END IF

C
C     That's all, folks!
C

      END


