C$Procedure      SPKBSR ( S/P Kernel, Buffer segments for readers )

      SUBROUTINE SPKBSR ( FNAME,
     .                    HANDLE,
     .                    BODY,
     .                    ET,
     .                    DESCR,
     .                    IDENT,
     .                    FOUND   )

C$ Abstract
C
C     Load and unload files for use by the readers.  Buffer segments
C     for readers.
C
C$ Required_Reading
C
C     SPK
C
C$ Keywords
C
C     EPHEMERIS
C     FILES
C
C$ Declarations

      CHARACTER*(*)         FNAME
      INTEGER               HANDLE
      INTEGER               BODY
      DOUBLE PRECISION      ET
      DOUBLE PRECISION      DESCR    ( * )
      CHARACTER*(*)         IDENT
      LOGICAL               FOUND

      INTEGER               FTSIZE
      PARAMETER           ( FTSIZE =   20 )

      INTEGER               BTSIZE
      PARAMETER           ( BTSIZE =   20 )

      INTEGER               STSIZE
      PARAMETER           ( STSIZE =   100 )

C$ Brief_I/O
C
C     Variable  I/O  Entry points
C     --------  ---  --------------------------------------------------
C     FNAME      I   SPKLEF
C     HANDLE    I/O  SPKLEF, SPKUEF, SPKSFS
C     BODY       I   SPKSFS
C     ET         I   SPKSFS
C     DESCR      O   SPKSFS
C     IDENT      O   SPKSFS
C
C$ Detailed_Input
C
C     FNAME      is the name of an S/P-kernel ephemeris file to be
C                loaded.
C
C     HANDLE     on input is the handle of an S/P-kernel file to be
C                unloaded.
C
C     BODY       is the NAIF integer code of a solar system body.
C
C     ET         is a time, in ephemeris seconds past the epoch J2000.
C
C$ Detailed_Output
C
C     HANDLE     on output is the handle of the S/P-kernel file
C                containing a located segment.
C
C     DESCR      is the descriptor of a located segment.
C
C     IDENT      is the identifier of a located segment.
C
C     FOUND      indicates whether a requested segment was found or not.
C
C$ Parameters
C
C     FTSIZE     is the maximum number of ephemeris files that can be
C                loaded by SPKLEF at any given time for use by the
C                readers.
C
C     BTSIZE     is the maximum number of bodies whose segments can be
C                buffered by SPKSFS.
C
C     STSIZE     Maximum number of segments that can be buffered at any
C                given time by SPKSFS.
C
C
C$ Files
C
C     S/P-kernel ephemeris files are indicated by filename before
C     loading (see SPKLEF) and handle after loading (all other places).
C
C$ Exceptions
C
C     1) If SPKBSR is called directly, the error 'SPICE(BOGUSENTRY)'
C        is signalled.
C
C     2) See entry points SPKLEF, SPKUEF, and SPKSFS for exceptions
C        specific to them.
C
C$ Particulars
C
C     SPKBSR serves as an umbrella, allowing data to be shared by its
C     entry points:
C
C        SPKLEF       Load ephemeris file.
C        SPKUEF       Unload ephemeris file.
C        SPKSFS       Select file and segment.
C
C     Before a file can be read by the S/P-kernel readers, it must be
C     loaded by SPKLEF, which among other things, performs a FORTRAN
C     OPEN of the specified file.
C
C     Several files may be loaded for use simultaneously, and a file
C     only has to be loaded once to become a potential search target
C     for any number of subsequent reads.
C
C     Once a S/P-kernel file is loaded and opened, it is assigned a file
C     handle, which is used to keep track of the file internally, and
C     which is used by the calling program to refer to the file in all
C     subsequent calls to SPK routines.
C
C     A file may be removed from the list of files for potential
C     searching by unloading it, with SPKUEF.
C
C     SPKSFS performs the search for segments within a file for the
C     S/P-kernel readers.  It searches through last-loaded files first.
C     Within a single file, it searches through last-inserted segments
C     first, thus assuming that "newest data is best".
C
C     Information on loaded files is used by SPKSFS to manage a buffer
C     of saved segment descriptors and identifiers to speed up access
C     time without having to necessarily perform file reads.
C
C
C$ Examples
C
C     Suppose that ephemeris data for the Mars Observer spacecraft
C     relative to Mars are contained in three separate files:
C     PREDICT.SPK contains complete predict ephemeris data for several
C     successive orbits, and UPDATE_1.SPK and UPDATE_2.SPK contain two
C     separate updates to selected intervals within those orbits, based
C     on altimeter fits.
C
C     In the following example, states of the spacecraft are computed
C     in two different ways:
C
C     First, the predict file and one of the update files are both
C     loaded and states are requested for regular intervals within
C     the orbits. The update file is searched through first, and if no
C     data for the requested time is available, the predict file is
C     used.
C
C     Then, the first update file is unloaded, the second update file
C     is loaded, and the same requests are made as above.
C
C     Throughout the two searches, a table is written which contains
C     the state of the spacecraft, and the file from which the data
C     came, if such data was found, and an error message otherwise.
C
C     It is assumed that the beginning and ending ephemeris times
C     (BEG_ET, END_ET) for the entire span have already been
C     initialized, along with the step-size for each measurement
C     (DELTA). The two routines WRITE_TABLE and WRITE_ERROR do not
C     exist in SPICELIB.
C
C
C           INTEGER               PRED_HNDL
C           INTEGER               UPD1_HNDL
C           INTEGER               UPD2_HNDL
C           INTEGER               HANDLE
C
C           DOUBLE PRECISION      BEG_ET
C           DOUBLE PRECISION      END_ET
C           DOUBLE PRECISION      DELTA
C           DOUBLE PRECISION      ET
C           DOUBLE PRECISION      DESCR ( 5 )
C           DOUBLE PRECISION      STATE ( 3 )
C
C           CHARACTER*40          IDENT
C           CHARACTER*25          FNAME
C
C     C
C     C     Load the predict file and the first update file.  Since
C     C     last-loaded files get searched first, we want to load the
C     C     update file second.
C     C
C           CALL SPKLEF ( 'PREDICT.SPK',  PRED_HNDL )
C           CALL SPKLEF ( 'UPDATE_1.SPK', UPD1_HNDL )
C
C     C
C     C     NAIF code for the Mars Observer spacecraft is -94.
C     C
C           BODY = -94
C
C     C
C     C     Compute states for regular intervals between BEG_ET and
C     C     END_ET.
C     C
C           ET = BEG_ET
C
C           DO WHILE ( ET .LE. END_ET )
C
C     C
C     C        Locate the applicable segment (handle and descriptor).
C     C
C              CALL SPKSFS ( BODY, ET, HANDLE, DESCR, IDENT, FOUND )
C
C              IF ( FOUND ) THEN
C     C
C     C           Evaluate the state, get the name of the file from
C     C           whence the data came, and write the results to the
C     C           table.
C     C
C                 CALL SPKPV ( HANDLE, DESCR, ET, 'J2000', STATE,
C             .                CENTER )
C
C                 CALL CKHFN ( HANDLE, FNAME )
C
C                 CALL WRITE_TABLE ( ET, STATE, FNAME )
C
C              ELSE
C
C                 CALL WRITE_ERROR ( ET )
C
C              END IF
C
C     C
C     C        The next time.
C     C
C              ET = ET + DELTA
C
C           END DO
C
C     C
C     C     Unload the first update file, load the second, and do
C     C     everything over again.  Since the original file stays
C     C     loaded, the update file once again gets searched first.
C     C
C           CALL SPKUEF (  UPD1_HNDL )
C           CALL SPKLEF ( 'UPDATE_2.SPK', UPD2_HNDL )
C
C           ET = BEG_ET
C
C           DO WHILE ( ET .LE. END_ET )
C
C     C
C     C        Locate the applicable segment.
C     C
C              CALL SPKSFS ( BODY, ET, HANDLE, DESCR, IDENT, FOUND )
C
C              IF ( FOUND ) THEN
C     C
C     C           Evaluate the state, get the name of the file from
C     C           whence the data came, and write the results to the
C     C           table.
C     C
C                 CALL SPKPV ( HANDLE, DESCR, ET, 'J2000', STATE,
C             .                CENTER )
C
C                 CALL CKHFN ( HANDLE, FNAME )
C
C                 CALL WRITE_TABLE ( ET, STATE, FNAME )
C
C              ELSE
C
C                 CALL WRITE_ERROR ( ET )
C
C              END IF
C
C     C
C     C        The next time.
C     C
C              ET = ET + DELTA
C
C           END DO
C
C$ Restrictions
C
C     None.
C
C$ Literature_References
C
C     NAIF Document 168.0, "S- and P- Kernel (SPK) Specification and
C     User's Guide"
C
C$ Author_and_Institution
C
C     R.E. Thurman    (JPL)
C
C$ Version
C
C-    SPICELIB Version 1.0.1, 22-MAR-1990 (HAN)
C
C        Literature references added to the header.
C
C-    SPICELIB Version 1.0.0, 31-JAN-1990 (RET)
C
C-&

C
C     SPICELIB functions
C
      LOGICAL               FAILED
      LOGICAL               RETURN

      INTEGER               INTMAX
      INTEGER               ISRCHI

C
C     Local variables
C

C
C     The file table contains the handle and file number of each file
C     that has been loaded for use with the SPK readers. File
C     numbers begin at one, and are incremented until they reach the
C     maximum integer representable by the compiler, at which point
C     the routine fails.
C
C     (A file number is similar to a file handle, but it is assigned
C     and used exclusively by this module. The purpose of file numbers
C     is to keep track of the order in which files are loaded and the
C     order in which they are searched.)
C
C     All names begin with FT.
C
C        HAN      Handle
C        NUM      File number
C
C     NFT is the number of files that have been loaded. NEXT is
C     incremented whenever a new file is loaded to give the file
C     number of the file. FINDEX is the index of whatever file is
C     of current interest at any given time.
C
C     New files are added at the end of the table. As files are
C     removed, succeeding files are moved forward to take up the
C     slack. This keeps the table ordered by file number.
C
      INTEGER               NFT
      INTEGER               FTHAN    ( FTSIZE )
      INTEGER               FTNUM    ( FTSIZE )
      INTEGER               NEXT
      INTEGER               FINDEX

C
C     The body table contains the beginning of the list of the stored
C     segments for each body, and the expense at which that list
C     was constructed. (The expense of a body list is the number of
C     segment descriptors examined during the construction of the list.)
C     It also contains the highest and lowest file numbers searched
C     during the construction of the list.
C
C     All names begin with BT.
C
C        BOD      Body
C        EXP      Expense
C        HFS      Highest file (number) searched
C        LFS      Lowest  file (number) searched
C        BEG      Beginning of segment list
C
C     NBT is the number of bodies for which segments are currently
C     being stored in the table. BINDEX is the index of whatever
C     body is of current interest at any given time.
C
C     New bodies are added at the end of the table. As bodies are
C     removed, the last body is moved forward to take up the slack.
C     This keeps the entries in the table contiguous.
C
      INTEGER               NBT
      INTEGER               BTBOD    ( BTSIZE )
      INTEGER               BTEXP    ( BTSIZE )
      INTEGER               BTHFS    ( BTSIZE )
      INTEGER               BTLFS    ( BTSIZE )
      INTEGER               BTBEG    ( BTSIZE )
      INTEGER               BINDEX

C
C     The segment table contains the handle, descriptor, and identifier
C     for each segment that has been found so far. The segment contains
C     data for the body on whose list it appears. The list is
C     one-way:  each node points to the next node. The final node
C     points to nothing (zero). Nodes not used by any list are on the
C     free list.
C
C     All names begin with ST.
C
C        PTR      Pointer to next element
C        HAN      Handle
C        DES      Descriptor
C        IDNT     Identifier
C
C     FREE points to the beginning of the free list.
C
C     New segments are added to the front or end of an instrument list
C     as appropriate, according to the rules spelled out under
C     entry point CKSFS.
C
      INTEGER               STPTR    (   STSIZE )
      INTEGER               STHAN    (   STSIZE )
      DOUBLE PRECISION      STDES    ( 5,STSIZE )
      CHARACTER*40          STIDNT   (   STSIZE )
      INTEGER               FREE

C
C     Other stuff
C
      INTEGER               SLEN
      PARAMETER           ( SLEN = 15 )

      INTEGER               ICD      ( 6 )
      INTEGER               TOP
      INTEGER               MINEXP
      INTEGER               CHEAP
      INTEGER               NXTSEG
      INTEGER               PREV
      INTEGER               TAIL
      INTEGER               NEW
      INTEGER               CRFLBG
      INTEGER               P
      INTEGER               I
      INTEGER               J

      DOUBLE PRECISION      DCD      ( 2 )

      CHARACTER*(SLEN)      STATUS
      CHARACTER*(SLEN)      DOING
      CHARACTER*(SLEN)      URGENT
      CHARACTER*(SLEN)      STACK    ( 2 )

      LOGICAL               FND

C
C     Saved variables
C
      SAVE                  NFT,
     .                      FTHAN,
     .                      FTNUM,
     .                      NEXT,
     .                      NBT,
     .                      BTBOD,
     .                      BTEXP,
     .                      BTHFS,
     .                      BTLFS,
     .                      BTBEG,
     .                      STPTR,
     .                      STHAN,
     .                      STDES,
     .                      STIDNT,
     .                      FREE

C
C     Initial values
C
      DATA                  NFT       / 0 /
      DATA                  NBT       / 0 /
      DATA                  NEXT      / 0 /

C
C     Nobody has any business calling SPKBSR directly.
C
      IF ( RETURN() ) THEN
         RETURN
      END IF

      CALL CHKIN  ( 'SPKBSR' )
      CALL SIGERR ( 'SPICE(BOGUSENTRY)' )
      CALL CHKOUT ( 'SPKBSR' )

      RETURN


C$Procedure SPKLEF ( S/P Kernel, Load ephemeris file )

      ENTRY SPKLEF ( FNAME, HANDLE )

C$ Abstract
C
C     Load an ephemeris file for use by the readers.  Return that
C     file's handle, to be used by other SPK routines to refer to the
C     file.
C
C$ Required_Reading
C
C     SPK
C
C$ Keywords
C
C     EPHEMERIS
C     FILES
C
C$ Declarations
C
C     CHARACTER*(*)         FNAME
C     INTEGER               HANDLE
C
C$ Brief_I/O
C
C     Variable  I/O  Description
C     --------  ---  --------------------------------------------------
C     FNAME      I   Name of the file to be loaded.
C     HANDLE     O   Loaded file's handle.
C
C$ Detailed_Input
C
C     FNAME      Character name of the file to be loaded.
C
C$ Detailed_Output
C
C     HANDLE     Integer handle assigned to the file upon loading.
C                Almost every other SPK routine will subsequently use
C                this number to refer to the file.
C
C$ Parameters
C
C     None.
C
C$ Exceptions
C
C     1) If an attempt is made to load more files than is specified
C        by the paramater FTSIZE, the error 'SPICE(SPKFILETABLEFULL)' is
C        signalled.
C
C$ Files
C
C     A file specified by FNAME, to be loaded.  The file is assigned a
C     handle by SPKLEF, which will be used by most other routines to
C     refer to it.
C
C$ Particulars
C
C     If there is room for a new file in the file table, SPKLEF creates
C     an entry for it, and OPENs the file for reading, using DAFOPR.
C
C     Also, if the body table is empty, SPKLEF initializes it, this
C     being as good a place as any.
C
C$ Examples
C
C     See the Example above, in SPKBSR.
C
C$ Restrictions
C
C     None.
C
C$ Literature_References
C
C     NAIF Document 168.0, "S- and P- Kernel (SPK) Specification and
C     User's Guide"
C
C$ Author_and_Institution
C
C     R.E. Thurman    (JPL)
C     I.M. Underwood  (JPL)
C
C$ Version
C
C-    SPICELIB Version 1.0.1, 22-MAR-1990 (HAN)
C
C        Literature references added to the header.
C
C-    SPICELIB Version 1.0.0, 31-JAN-1990 (RET)
C
C-&


C$ Revisions
C
C-    Beta Version 1.1.0, 25-JAN-1990 (IMU)
C
C        If a file that has already been loaded is loaded a second
C        (or third or fourth) time, it should be removed from the
C        file table, and anysegments from the file must be removed
C        from the segment lists, just as if the user had unloaded
C        the file before loading it again. This means that a single
C        file cannot occur more than once in the file table.
C
C-&

C
C     Standard SPICE error handling.
C
      IF ( RETURN () ) THEN
         RETURN
      ELSE
         CALL CHKIN ( 'SPKLEF' )
      END IF

C
C     Nothing works unless at least one file has been loaded, so
C     this is as good a place as any to initialize the free list
C     whenever the body table is empty.
C
C     Each node except the last should point to the adjacent node;
C     the last points to nothing (zero).
C
      IF ( NBT .EQ. 0 ) THEN
         FREE = 1

         DO 50001
     .      I = 1, STSIZE - 1
            STPTR( I ) = I + 1
50001    CONTINUE

         STPTR( STSIZE ) = 0

      END IF

C
C     To load a new file, open the file for reading. If successful,
C     get the next file number and store it, with the handle of the
C     file, in the file table.
C
C     If the file is already in the table, remove it, remove its
C     segments from the segment table (using the same algorithm
C     used by SPKUEF, below), and then store it.
C
C     Of course, all this assumes that there are unused slots in the
C     file table.
C
      IF ( NFT .EQ. FTSIZE ) THEN
         CALL SETMSG ( 'The internal file table is already full, '    //
     .                 'with # entries.' )
         CALL ERRINT ( '#', FTSIZE )
         CALL SIGERR ( 'SPICE(SPKFILETABLEFULL)' )

      ELSE

         CALL DAFOPR ( FNAME, HANDLE )

         IF ( .NOT. FAILED() ) THEN

            FINDEX = ISRCHI( HANDLE, NFT, FTHAN )

            IF ( FINDEX .GT. 0 ) THEN

               NFT = NFT - 1

               DO 50002
     .            I = FINDEX, NFT
                  FTHAN( I ) = FTHAN( I + 1 )
                  FTNUM( I ) = FTNUM( I + 1 )
50002          CONTINUE

               I = 1

50003          IF       ( I .LE. NBT )
     .         THEN

                  P    = BTBEG( I )
                  PREV = 0

50004             IF       ( P .NE. 0 )
     .            THEN

                     IF ( STHAN( P ) .EQ. HANDLE ) THEN

                        IF ( P .EQ. BTBEG( I ) ) THEN

                           BTBEG( I ) = STPTR( P )
                           STPTR( P ) = FREE
                           FREE       = P
                           P          = BTBEG( I )

                        ELSE

                           STPTR( PREV ) = STPTR( P )
                           STPTR( P )    = FREE
                           FREE          = P
                           P             = STPTR( PREV )

                        END IF

                     ELSE

                        PREV = P
                        P    = STPTR( P )

                     END IF

                     GO TO 50004
                  END IF

                  IF ( BTBEG( I ) .EQ. 0 ) THEN

                     BTBOD( I ) = BTBOD( NBT )
                     BTEXP( I ) = BTEXP( NBT )
                     BTHFS( I ) = BTHFS( NBT )
                     BTLFS( I ) = BTLFS( NBT )
                     BTBEG( I ) = BTBEG( NBT )

                     NBT = NBT - 1

                  END IF

                  I = I + 1

                  GO TO 50003
               END IF

            END IF

            NFT        = NFT  + 1
            NEXT       = NEXT + 1
            FTHAN(NFT) = HANDLE
            FTNUM(NFT) = NEXT

         END IF

      END IF

      CALL CHKOUT ( 'SPKLEF' )
      RETURN


C$Procedure SPKUEF ( S/P Kernel, Unload ephemeris file )

      ENTRY SPKUEF ( HANDLE )

C$ Abstract
C
C     Unload an ephemeris file so that it will no longer be searched by
C     the readers.
C
C$ Required_Reading
C
C     SPK
C
C$ Keywords
C
C     EPHEMERIS
C     FILES
C
C$ Declarations
C
C     INTEGER               HANDLE
C
C$ Brief_I/O
C
C     Variable  I/O  Description
C     --------  ---  --------------------------------------------------
C     HANDLE     I   Handle of file to be unloaded
C
C$ Detailed_Input
C
C     HANDLE     Integer handle assigned to the file upon loading.
C
C$ Detailed_Output
C
C     None.
C
C$ Parameters
C
C     None.
C
C$ Exceptions
C
C     None.
C
C$ Files
C
C     The file referred to by HANDLE is unloaded.
C
C$ Particulars
C
C     A file is removed from consideration by the readers by a call to
C     SPKUEF.
C
C     The file table entry corresponding to the file referenced by
C     HANDLE, is removed.  Also any segment table entry which came from
C     the specified file is also deleted.
C
C     If the file specified by HANDLE does not appear in the file table,
C     nothing happens.
C
C$ Examples
C
C     See the Example above, in SPKBSR.
C
C$ Restrictions
C
C     None.
C
C$ Literature_References
C
C     NAIF Document 168.0, "S- and P- Kernel (SPK) Specification and
C     User's Guide"
C
C$ Author_and_Institution
C
C     R.E. Thurman    (JPL)
C     I.M. Underwood  (JPL)
C
C$ Version
C
C-    SPICELIB Version 1.1.0, 2-MAY-1990 (RET)
C
C        If unloading a file causes all segments in the list for a
C        body to go away, delete that body from the body list.
C
C-    SPICELIB Version 1.0.1, 22-MAR-1990 (HAN)
C
C        Literature references added to the header.
C
C-    SPICELIB Version 1.0.0, 31-JAN-1990 (RET)
C
C-&

C$ Revisions
C
C-    SPICELIB Version 1.1.0, 2-MAY-1990 (RET)
C
C        If unloading a file causes all segments in the list for a
C        body to go away, delete that body from the body list.
C
C-    Beta Version 1.1.0, 25-JAN-1990 (IMU)
C
C        When unloading a file, close it.
C
C-&


C
C     Unloading a file is a little trickier. Not only must the
C     file be removed from the file table, but all of the stored
C     segments from the file must be removed from the segment table
C     (by returning the corresponding nodes to the front of the
C     free list.)
C
C     Don't do anything if the given handle is not in the file table.
C
      FINDEX = ISRCHI( HANDLE, NFT, FTHAN )

      IF ( FINDEX .EQ. 0 ) THEN
         RETURN
      END IF

C
C     First get rid of the entry in the file table. Close the file
C     before wiping out the handle.
C
      CALL DAFCLS ( FTHAN(FINDEX) )

      NFT = NFT - 1

      DO 50005
     .   I = FINDEX, NFT
         FTHAN( I ) = FTHAN( I + 1 )
         FTNUM( I ) = FTNUM( I + 1 )
50005 CONTINUE

C
C     Check each body list individually. Note that the first node
C     on each list, having no predecessor, must be handled specially.
C
      I = 1

50006 IF       ( I .LE. NBT )
     .THEN

         P    = BTBEG( I )
         PREV = 0

50007    IF       ( P .NE. 0 )
     .   THEN

            IF ( STHAN( P ) .EQ. HANDLE ) THEN

               IF ( P .EQ. BTBEG( I ) ) THEN

                  BTBEG( I ) = STPTR( P )
                  STPTR( P ) = FREE
                  FREE       = P
                  P          = BTBEG( I )

               ELSE

                  STPTR( PREV ) = STPTR( P )
                  STPTR( P )    = FREE
                  FREE          = P
                  P             = STPTR( PREV )

               END IF

            ELSE

               PREV = P
               P    = STPTR( P )

            END IF

            GO TO 50007
         END IF

C
C        If we happened to get rid of all of the segments for this
C        body, then the body should be deleted from the table: shift
C        all entries for the body at the end of the table into the
C        space occupied by the deleted body.
C
         IF ( BTBEG( I ) .EQ. 0 ) THEN

            BTBOD( I ) = BTBOD( NBT )
            BTEXP( I ) = BTEXP( NBT )
            BTHFS( I ) = BTHFS( NBT )
            BTLFS( I ) = BTLFS( NBT )
            BTBEG( I ) = BTBEG( NBT )

            NBT = NBT - 1

         END IF

         I = I + 1

         GO TO 50006
      END IF

      RETURN



C$Procedure SPKSFS ( S/P Kernel, Select file and segment )

      ENTRY SPKSFS ( BODY, ET, HANDLE, DESCR, IDENT, FOUND )

C$ Abstract
C
C     Search through loaded files to find the first segment applicable
C     to the body and time specified.  Buffer searched segments in the
C     process, to attempt to avoid re-reading files.
C
C$ Required_Reading
C
C     SPK
C
C$ Keywords
C
C     EPHEMERIS
C     FILES
C
C$ Declarations
C
C     INTEGER               BODY
C     DOUBLE PRECISION      ET
C     INTEGER               HANDLE
C     DOUBLE PRECISION      DESCR  ( * )
C     CHARACTER*(*)         IDENT
C     LOGICAL               FOUND
C
C$ Brief_I/O
C
C     Variable  I/O  Description
C     --------  ---  --------------------------------------------------
C     BODY       I   Body ID.
C     ET         I   Ephemeris time.
C     HANDLE     O   Handle of file containing the applicable segment.
C     DESCR      O   Descriptor of the applicable segment.
C     IDENT      O   Identifier of the applicable segment.
C     FOUND      O   Indicates whether or not a segment was found.
C
C$ Detailed_Input
C
C     BODY       is the NAIF integer code of a solar system body.
C
C     ET         is a time, in ephemeris seconds past the epoch J2000.
C
C$ Detailed_Output
C
C     HANDLE     on output is the handle of the S/P-kernel file
C                containing a located segment.
C
C     DESCR      is the descriptor of a located segment.
C
C     IDENT      is the identifier of a located segment.
C
C     FOUND      indicates whether a requested segment was found or not.
C
C$ Parameters
C
C     None.
C
C$ Exceptions
C
C     1) If an attempt is made to call SPKSFS when there aren't any
C        files loaded, the error SPICE(NOLOADEDFILES) is signalled.
C
C$ Files
C
C     All files loaded by SPKLEF are potential search targets for
C     SPKSFS.
C
C$ Particulars
C
C     To find the applicable segment, a search is made through last-
C     loaded files first.  Within a single file, segments inserted last
C     at the time of creation get checked first.
C
C     Basically, the buffering is performed as follows: Once a request
C     for a segment for a particular body is made, if there are
C     no adequate entries in the buffer already, a search is made
C     through loaded files for applicable segments.  Every segment
C     pertaining to that body in a searched file is buffered,
C     before a check of the current buffer is made.  If the search
C     doesn't turn up a segment for the specified time, the next file
C     is searched and new segments are added to the list, etc.
C
C$ Examples
C
C     See the Example above, in SPKBSR.
C
C$ Restrictions
C
C     None.
C
C$ Literature_References
C
C     NAIF Document 168.0, "S- and P- Kernel (SPK) Specification and
C     User's Guide"
C
C$ Author_and_Institution
C
C     R.E. Thurman    (JPL)
C
C$ Version
C
C-    SPICELIB Version 1.1.0, 2-MAY-1990 (RET)
C
C        New error detected.
C
C-    SPICELIB Version 1.0.1, 22-MAR-1990 (HAN)
C
C        Literature references added to the header.
C
C-    SPICELIB Version 1.0.0, 31-JAN-1990 (RET)
C
C-&



C$ Revisions
C
C-    SPICELIB Version 1.1.0, 2-MAY-1990 (RET)
C
C        If an attempt is made to call SPKSFS when there are no files
C        loaded, an error is now signalled.
C
C-&

C
C     Standard SPICE error handling.
C
      IF ( RETURN () ) THEN
         RETURN
      ELSE
         CALL CHKIN ( 'SPKSFS' )
      END IF

C
C     Buffering segments involves maintaining three tables:  the
C     file table, the body table, and the segment table.  The routine
C     is broken down into various tasks, described below, which
C     perform these manipulations.  A description of the components
C     of each table is provided in the declarations section of SPKBSR.
C
C     The information in the segment table (ST) is stored in a
C     singly-linked list. Each node in the list contains several
C     individual pieces of data, which are stored in parallel
C     arrays. One of these arrays, PTR, contains the pointers
C     for the nodes. The pointer for each node contains the array
C     index of the next node on the list. The final node on any
C     list contains index zero, indicating that it is the end
C     of the list.
C
C     A list begins with a head, in this case either BTBEG( BINDEX ),
C     or FREE. The head is the array index of the first node in the
C     list. Initially, all nodes are on the free list:
C
C                      Next   Data
C                    +------+--------+
C           FREE --> |    2 |        |
C                    |      |        | ---,
C                    +------+--------+    |
C                    |    3 |        | <--`
C                    |      |        | ---,
C                    +------+--------+    |
C                    |    4 |        | <--`
C                    |      |        | ---,
C                    +------+--------+    |
C                    |    5 |        | <--`
C                    |      |        | ---,
C                    +------+--------+    |
C                    |    6 |        | <--`
C                    |      |        | ---,
C                    +------+--------+    |
C                    |    7 |        | <--`
C                    |      |        | ---,
C                    +------+--------+    |
C                    |    8 |        | <--`
C                    |      |        | ---,
C                    +------+--------+    |
C                    |    0 |        | <--`
C                    |      |        |
C                    +------+--------+
C
C     The various BTBEG( I ) all contain zero, indicating that they
C     contain no nodes.
C
C     Later, some of the nodes are associated with the body lists.
C
C                      Next   Data
C                    +------+--------+
C       BTBEG(2) --> |    2 |        |
C                    |      |        | ---,
C                    +------+--------+    |
C                    |    8 |        | <--`
C                    |      |        | -------------,
C                    +------+--------+              |
C                    |    5 |        |              |
C       FREE ------> |      |        | ------,      |
C                    +------+--------+       |      |
C       BTBEG(1) --> |    6 |        |       |      |
C                    |      |        | ---,  |      |
C                    +------+--------+    |  |      |
C                    |    0 |        |    |  |      |
C                    |      |        | <-----`      |
C                    +------+--------+    |         |
C                    |    0 |        | <--`         |
C                    |      |        |              |
C                    +------+--------+              |
C                    |    0 |        |              |
C                    |      |        | <--,         |
C                    +------+--------+    |         |
C                    |    7 |        | ---`         |
C                    |      |        | <------------`
C                    +------+--------+
C
C     Note that every node is always on exactly one list.
C
C     The whole point of using a list is to avoid shuffling the
C     contents of the arrays back and forth every time a segment
C     is added or deleted from the list of stored segments.
C
C     The algorithms for adding and deleting nodes from lists are
C     adequately documented elsewhere, and will not be descibed
C     here. However, note that elements are added only to the front
C     or end of a list, never the middle. Note also that when a
C     body is removed from the body table, the nodes associated with
C     the body are added to the beginning of the free list in a single
C     operation, rather than a node at a time.
C
C

C
C     Assume the segment is not found, until it actually is.
C
      FOUND = .FALSE.

C
C     There must be at least ONE file loaded.
C
      IF ( NFT .EQ. 0 ) THEN
         CALL SETMSG ( 'At least one SPK file needs to be loaded '   //
     .                 'by SPKLEF before beginning a search.'       )
         CALL SIGERR ( 'SPICE(NOLOADEDFILES)'                       )
         CALL CHKOUT ( 'SPKSFS'                                     )
         RETURN
      END IF

C
C     The stack of suspended tasks is empty.
C
      TOP = 0

C
C     In the following loop, we will try to simplify things by
C     doing exactly one thing on each pass through the loop.
C     After each pass, the status of the loop (STATUS) will be
C     adjusted to reflect the next thing that needs to be done.
C     Occasionally, the current task will have to be interrupted
C     until another task can be carried out. (For example, when
C     collecting new segments, an interrupt might place a segment
C     at the front or end of the current body list; when placing
C     the segment on the list, a second interrupt might free up
C     room in the segment table in order to allow the addition
C     to proceed.) In this case, the current task will be saved and
C     restored after the more urgent task has been completed.
C
C     The loop can terminate in only one of two ways. First, if
C     an applicable segment is found in the segment table, the
C     handle, descriptor, and identifier for the segment are returned
C     immediately.  Second, if the table does not contain an applicable
C     segment, and if no files remain to be searched, the loop
C     terminates normally, and no data is returned.
C
C     The individual tasks are described below.
C
C     'NEW BODY'
C
C
C        This indicates that the specified body has no segments stored
C        for it at all. It must be added to the body table.  (This is
C        followed immediately by an OLD FILES search, in which every
C        file loaded is considered an old file.)
C
C     'NEW FILES'
C
C        This indicates that at least one new file has been added
C        since the last time the segment list for the specified
C        body was searched. Find the oldest of these new files,
C        and begin a NEW SEGMENTS search in forward order for
C        segments to add to the front of the list.
C
C     'NEW SEGMENTS'
C
C        Continue a NEW FILES search, adding segments for the specified
C        body to the front of the list.
C
C     'OLD FILES'
C
C        This indicates that although the list has been searched
C        and found to contain no applicable segment, some of the
C        older files remain to be searched. Find the newest of these
C        old files, and begin an OLD SEGMENTS search in backward order.
C
C     'OLD SEGMENTS'
C
C        Continue an OLD FILES search, adding segments for the specified
C        body to the end of the list.
C
C     'CHECK LIST'
C
C        This indicates that the list is ready to be searched,
C        either because no new files have been added, or because
C        segments from a new file or an old file have recently
C        been added.
C
C        The list is never checked until all new files have been
C        searched.
C
C        If an applicable segment is found, it is returned.
C
C     'MAKE ROOM' (Interrupt)
C
C        This indicates that one of the bodies must be removed,
C        along with its stored segments, to make room for another
C        body or segment.  The body (other than the one being searched
C        for) with the smallest expense is selected for this honor.
C
C     'ADD TO FRONT' (Interrupt)
C
C        This indicates that a segment has been found (during the
C        course of a NEW FILES search) and must be added to the front
C        of the list.
C
C     'ADD TO END' (Interrupt)
C
C        This indicates that a segment has been found (during the
C        course of an OLD FILES search) and must be added to the end
C        of the list.
C
C     'SUSPEND'
C
C        This indicates that the current task (DOING) should be
C        interrupted until a more urgent task (URGENT) can be
C        carried out. The current task is placed on a stack for
C        safekeeping.
C
C     'RESUME'
C
C        This indicates that the most recently interrupted task
C        should be resumed immediately.
C
C     '?'
C
C        This indicates that the next task is not immediately
C        apparent: if new files exist, they should be searched;
C        otherwise the list should be checked.
C


C
C     Is the body already in the body table?  This determines what the
C     first task should be.
C
      BINDEX = ISRCHI ( BODY, NBT, BTBOD )

      IF ( BINDEX .EQ. 0 ) THEN
         STATUS = 'NEW BODY'
      ELSE
         STATUS = '?'
      END IF

50008 IF       ( STATUS .NE. 'HOPELESS' )
     .THEN

C
C        If new files have been added, they have to be searched.
C        Otherwise, we can go right to the list of stored segments.
C
         IF ( STATUS .EQ. '?' ) THEN

            IF ( BTHFS( BINDEX ) .LT. FTNUM( NFT ) ) THEN
               STATUS = 'NEW FILES'
            ELSE
               STATUS = 'CHECK LIST'
            END IF

C
C        New bodies are added to the end of the body table. If the
C        table is full, one of the current occupants must be removed to
C        make room for the new one.
C
C        Setting LFS to one more than the highest current
C        file number means the OLD SEARCH that follows will
C        begin with the last-loaded file.
C
         ELSE IF ( STATUS .EQ. 'NEW BODY' ) THEN

            IF ( NBT .EQ. BTSIZE ) THEN

               DOING  = 'NEW BODY'
               URGENT = 'MAKE ROOM'
               STATUS = 'SUSPEND'

            ELSE

               NBT        = NBT + 1
               BTBOD(NBT) = BODY
               BTEXP(NBT) = 0
               BTHFS(NBT) = FTNUM(NFT)
               BTLFS(NBT) = FTNUM(NFT) + 1
               BTBEG(NBT) = 0
               BINDEX     = NBT
               STATUS     = 'OLD FILES'

            END IF

C
C        When new files exist, they should be searched in forward
C        order, beginning with the oldest new file not yet searched.
C        All new files must be searched before the list can be
C        checked, to ensure that the best (newest) segments are
C        being used.
C
C        Begin a forward search, and prepare to look for individual
C        segments from the file.
C
         ELSE IF ( STATUS .EQ. 'NEW FILES' ) THEN

            FINDEX = 1

50009       IF       ( BTHFS( BINDEX ) .GE. FTNUM( FINDEX ) )
     .      THEN
               FINDEX = FINDEX + 1
               GO TO 50009
            END IF

            BTHFS( BINDEX ) = FTNUM( FINDEX )

            CALL DAFBFS ( FTHAN( FINDEX ) )
            STATUS = 'NEW SEGMENTS'

C
C        New files are searched in forward order. Segments, when
C        found, are inserted at the front of the list. Invisible
C        segments (alpha > omega) are ignored.
C
C        Each segment examined, whether applicable or not, adds to
C        the expense of the list.
C
         ELSE IF ( STATUS .EQ. 'NEW SEGMENTS' ) THEN

            CALL DAFFNA ( FND )

            IF ( .NOT. FND ) THEN
               STATUS = '?'

            ELSE

               CALL DAFGS ( DESCR )
               CALL DAFUS ( DESCR, 2, 6, DCD, ICD )

               IF ( ICD( 1 ) .EQ. BODY      .AND.
     .              DCD( 1 ) .LE. DCD( 2 ) ) THEN

                  DOING  = 'NEW SEGMENTS'
                  URGENT = 'ADD TO FRONT'
                  STATUS = 'SUSPEND'

               END IF

               BTEXP( BINDEX ) = BTEXP( BINDEX ) + 1

            END IF

C
C        When old files must be searched (because the segments in the
C        list are inadequate), they should be searched in backward
C        order, beginning with the newest old file not yet searched.
C        Not every old file must be searched. The list should be
C        re-checked after each file.
C
C        Search from the end, and prepare to look for individual
C        segments from the file.
C
         ELSE IF ( STATUS .EQ. 'OLD FILES' ) THEN

            FINDEX = NFT

50010       IF       ( BTLFS( BINDEX ) .LE. FTNUM( FINDEX ) )
     .      THEN
               FINDEX = FINDEX - 1
               GO TO 50010
            END IF

            CALL DAFBBS ( FTHAN( FINDEX ) )
            STATUS = 'OLD SEGMENTS'

C
C        Old files are searched in backward order. Segments, when
C        found, are inserted at the end of the list. Invisible
C        segments (alpha > omega) are ignored.
C
C        Each segment examined, whether applicable or not, adds to
C        the expense of the list.
C
         ELSE IF ( STATUS .EQ. 'OLD SEGMENTS' ) THEN

            CALL DAFFPA ( FND )

            IF ( .NOT. FND ) THEN

C
C              We've been through all of the segments in this file.
C              Change the lowest file searched indicator for this body
C              to be the current file, and go check the current list.
C
               BTLFS( BINDEX ) = FTNUM( FINDEX )
               STATUS = 'CHECK LIST'

            ELSE

               CALL DAFGS ( DESCR )
               CALL DAFUS ( DESCR, 2, 6, DCD, ICD )

               IF ( ICD( 1 ) .EQ. BODY      .AND.
     .              DCD( 1 ) .LE. DCD( 2 ) ) THEN

                  DOING  = 'OLD SEGMENTS'
                  URGENT = 'ADD TO END'
                  STATUS = 'SUSPEND'

               END IF

               BTEXP( BINDEX ) = BTEXP( BINDEX ) + 1

            END IF

C
C        Okay, all the new files (and maybe an old file or two) have
C        been searched. Time to look at the list of segments stored for
C        the body to see if one applicable to the specified epoch is
C        hiding in there. The initial and final epochs of the segment
C        are in the first two components of the descriptor.
C
C        If so, return it.  If not, try another old file.  If there
C        are no more old files, give up the ghost.
C
         ELSE IF ( STATUS .EQ. 'CHECK LIST'   ) THEN

            P = BTBEG( BINDEX )

50011       IF       ( P .NE. 0 )
     .      THEN

               IF ( ET .GE. STDES( 1, P )  .AND.
     .              ET .LE. STDES( 2, P ) ) THEN

                  CALL MOVED ( STDES( 1,P ), 5, DESCR )
                  IDENT  = STIDNT( P )
                  HANDLE = STHAN ( P )
                  FOUND  = .TRUE.

                  CALL CHKOUT ( 'SPKSFS' )
                  RETURN

               END IF

               P = STPTR(P)

               GO TO 50011
            END IF

            IF ( BTLFS( BINDEX ) .GT. FTNUM( 1 ) ) THEN
               STATUS = 'OLD FILES'
            ELSE
               STATUS = 'HOPELESS'
            END IF

C
C        When adding a new body or segment to a full table, one of
C        the current bodies must be dropped. The ideal candidate
C        is the one whose list was constructed at the lowest expense.
C        The candidate should be removed from the body table, and
C        its list transferred (intact) to the head of the free list.
C
C        As files are unloaded, some bodies may lose all their segments
C        (BTBEG=0). These should be removed from the body table,
C        possibly obviating the need to change the segment table at
C        all.
C
C        There is ``room'' if another body can be added, and if the free
C        list contains at least one node.
C
C        It is possible that a single body requires more than the
C        entire segment table for its own segments. Two things might
C        happen in such a case:
C
C           1) If the list under consideration was being added to at
C              the end, then a search is continued without buffering
C              any segments.
C
C           2) If the list was being added to at the beginning, then
C              that means there was a NEW FILES search going on, and
C              so a brand new list is constructed for the body, much as
C              in a 'NEW BODY' task.
C
         ELSE IF ( STATUS .EQ. 'MAKE ROOM' ) THEN

            J = 0
            DO 50012
     .         I = 1, NBT

               IF ( BTBEG( I ) .GT. 0 ) THEN

                  J        = J + 1
                  BTBOD( J ) = BTBOD( I )
                  BTEXP( J ) = BTEXP( I )
                  BTHFS( J ) = BTHFS( I )
                  BTLFS( J ) = BTLFS( I )
                  BTBEG( J ) = BTBEG( I )

               END IF

50012       CONTINUE

            NBT = J

            IF ( NBT .EQ. BTSIZE  .OR.  FREE .EQ. 0 ) THEN
               MINEXP = INTMAX()
               CHEAP  = 0

               DO 50013
     .            I = 1, NBT

                  IF ( I          .NE. BINDEX  .AND.
     .                 BTEXP( I ) .LT. MINEXP ) THEN

                    CHEAP  = I
                    MINEXP = BTEXP( I )

                  END IF

50013          CONTINUE

               IF ( CHEAP .EQ. 0 ) THEN

C
C                 What we do if there are no delete-able segments
C                 depends on the task that was suspended before entering
C                 'MAKE ROOM'.
C
                  IF ( STACK( TOP ) .EQ. 'ADD TO END' ) THEN
                     STATUS = 'SEARCH W/O BUFF'

                  ELSE IF ( STACK( TOP ) .EQ. 'ADD TO FRONT' ) THEN
C
C                    If there is no room left in the table in the middle
C                    of an attempt to add to the front of the list, just
C                    start from scratch by effectively initiating a 'NEW
C                    BODY' task.
C
C                    Turn the current list into a free list (FREE is
C                    currently zero)
C
                     FREE            = BTBEG( BINDEX )
                     BTBEG( BINDEX ) = 0
C
C                    Re-initialize the table for this body, and initiate
C                    an 'OLD FILES' search, just as in 'NEW BODY'.
C                    Also, reset the suspended task stack to be empty.
C
                     BTEXP( BINDEX ) = 0
                     BTHFS( BINDEX ) = FTNUM( NFT )
                     BTLFS( BINDEX ) = FTNUM( NFT ) + 1
                     STATUS          = 'OLD FILES'
                     TOP             = 0

                  END IF

               ELSE

C
C                 Turn this cheapest list into a free list by pointing
C                 its tail to the free list, and pointing the FREE
C                 pointer to its beginning.
C
                  TAIL   = BTBEG( CHEAP )
                  NXTSEG = STPTR( TAIL  )

50014             IF       ( NXTSEG .NE. 0 )
     .            THEN
                     TAIL   = NXTSEG
                     NXTSEG = STPTR( NXTSEG )
                     GO TO 50014
                  END IF

                  STPTR( TAIL )  = FREE
                  FREE           = BTBEG( CHEAP )

C
C                 Fill the deleted body's space in the table with
C                 the final entry in the table.
C
                  BTBOD( CHEAP ) = BTBOD( NBT )
                  BTEXP( CHEAP ) = BTEXP( NBT )
                  BTHFS( CHEAP ) = BTHFS( NBT )
                  BTLFS( CHEAP ) = BTLFS( NBT )
                  BTBEG( CHEAP ) = BTBEG( NBT )

C
C                 If the final entry in the table happened to be the
C                 current body of interest, then we also have to change
C                 the current body index.
C
                  IF ( BINDEX .EQ. NBT ) THEN
                     BINDEX = CHEAP
                  END IF

C
C                 One less body now.
C
                  NBT    = NBT - 1
                  STATUS = 'RESUME'

               END IF

            ELSE
               STATUS = 'RESUME'

            END IF

C
C        Adding a node to the front of a list is a standard algorithm.
C        The only possible wrinkle is that the free list may be empty.
C
         ELSE IF ( STATUS .EQ. 'ADD TO FRONT' ) THEN

            IF ( FREE .EQ. 0 ) THEN
               DOING  = 'ADD TO FRONT'
               URGENT = 'MAKE ROOM'
               STATUS = 'SUSPEND'

            ELSE
               STHAN( FREE ) = FTHAN( FINDEX )
               CALL MOVED ( DESCR, 5, STDES( 1,FREE ) )
               CALL DAFGN ( STIDNT( FREE ) )

               NEW             = FREE
               FREE            = STPTR( FREE )
               STPTR( NEW )    = BTBEG( BINDEX )
               BTBEG( BINDEX ) = NEW

               STATUS        = 'RESUME'
            END IF

C
C        Similarly for adding a node to the end of a list.
C
         ELSE IF ( STATUS .EQ. 'ADD TO END' ) THEN

            IF ( FREE .EQ. 0 ) THEN
               DOING  = 'ADD TO END'
               URGENT = 'MAKE ROOM'
               STATUS = 'SUSPEND'

            ELSE
C
C              The first free node is our newest node.  Move the free
C              pointer on down the free list.
C
               NEW  = FREE
               FREE = STPTR( FREE )

               STHAN( NEW ) = FTHAN( FINDEX )
               CALL MOVED ( DESCR, 5, STDES( 1,NEW ) )
               CALL DAFGN ( STIDNT( NEW ) )

               TAIL = BTBEG( BINDEX )

               IF ( TAIL .EQ. 0 ) THEN
C
C                 This is the first node in the list for this body.
C
                  BTBEG( BINDEX ) = NEW

               ELSE
C
C                 Need to move to the tail of the list before adding the
C                 pointer to the new segment.
C
                  NXTSEG = STPTR( TAIL )

50015             IF       ( NXTSEG .NE. 0 )
     .            THEN
                     TAIL   = NXTSEG
                     NXTSEG = STPTR( NXTSEG )
                     GO TO 50015
                  END IF

                  STPTR( TAIL ) = NEW

               END IF

C
C              The newest segment has no segment following it.
C
               STPTR( NEW ) = 0

               STATUS      = 'RESUME'

            END IF

C
C        When the segment table is completely full, continue the search
C        by looking through the unchecked portion of the table, and
C        then searching old, unchecked files without buffering the
C        new segments.
C
         ELSE IF ( STATUS .EQ. 'SEARCH W/O BUFF' ) THEN
C
C           Need to find the portion of the current body's segment
C           list which comes from the current file of interest.  It
C           will be returned to the free list, since the remainder of
C           the file's segments can't be added to the list.
C
            PREV   = 0
            CRFLBG = BTBEG( BINDEX )

50016       IF       ( STHAN( CRFLBG ) .NE. FTHAN( FINDEX ) .AND.
     .                 CRFLBG          .NE. 0                     )
     .      THEN

               PREV   = CRFLBG
               CRFLBG = STPTR( CRFLBG )

               GO TO 50016
            END IF

            IF ( CRFLBG .NE. 0 ) THEN
C
C              Return this space to the free list, which is currently
C              empty, and seal off the preceding list.
C
               FREE = CRFLBG

               IF ( PREV .NE. 0 ) THEN
                  STPTR( PREV ) = 0
               END IF

C
C              Finish searching through the incomplete list for the
C              desired segment.
C
50017          IF       ( CRFLBG .NE. 0 )
     .         THEN

                  IF ( ET .GE. STDES( 1, CRFLBG )  .AND.
     .                 ET .LE. STDES( 2, CRFLBG ) ) THEN

                     CALL MOVED ( STDES( 1, CRFLBG ), 5, DESCR )
                     IDENT  = STIDNT( CRFLBG )
                     HANDLE = STHAN ( CRFLBG )
                     FOUND  = .TRUE.
                     CALL CHKOUT ( 'SPKSFS' )
                     RETURN

                  END IF

                  CRFLBG = STPTR( CRFLBG )

                  GO TO 50017
               END IF

            END IF

C
C           Search through the remaining files without buffering.
C           Recall that a search is already in progress and that a
C           segment is currently under consideration (FND = .TRUE.).
C
50018       IF       ( FINDEX .GT. 0 )
     .      THEN

50019          IF       ( FND )
     .         THEN

                  CALL DAFGS ( DESCR )
                  CALL DAFUS ( DESCR, 2, 6, DCD, ICD )

                  IF ( BODY .EQ. ICD( 1 )  .AND.
     .                 ET   .GE. DCD( 1 )  .AND. ET .LE. DCD( 2 ) ) THEN

                     CALL DAFGN ( IDENT )
                     HANDLE = FTHAN( FINDEX )
                     FOUND  = .TRUE.
                     CALL CHKOUT( 'SPKSFS' )
                     RETURN

                  END IF

                  CALL DAFFPA ( FND )

                  GO TO 50019
               END IF

C
C              Try the next oldest file.
C
               FINDEX = FINDEX -1

               IF ( FINDEX .GT. 0 ) THEN

                  CALL DAFBBS ( FTHAN( FINDEX ) )
                  CALL DAFFPA ( FND             )

               END IF

               GO TO 50018
            END IF

C
C           If you get to here, sorry.
C
            STATUS = 'HOPELESS'

C
C        When a task is suspended, the current activity is placed on
C        a stack, to be restored later. Two levels are provided, since
C        some interrupts can be interrupted by others.
C
C        Trying to resume a task that was never suspended is a boo-boo.
C
         ELSE IF ( STATUS .EQ. 'SUSPEND' ) THEN

            TOP          = TOP + 1
            STACK( TOP ) = DOING
            STATUS       = URGENT

         ELSE IF ( STATUS .EQ. 'RESUME' ) THEN

            IF ( TOP .EQ. 0 ) THEN
               STATUS = '?'
            ELSE
               STATUS = STACK( TOP )
               TOP    = TOP - 1
            END IF

         END IF

         GO TO 50008
      END IF

      CALL CHKOUT ( 'SPKSFS' )
      RETURN

      END
