C$Procedure SPKE01 ( S/P Kernel, evaluate, type 1 )

      SUBROUTINE SPKE01 ( ET, RECORD, STATE )

C$ Abstract
C
C     Evaluate a single SPK data record from a segment of type 1
C     (Difference Lines).
C
C$ Required_Reading
C
C     SPK
C     SPK01
C
C$ Keywords
C
C     EPHEMERIS
C
C$ Declarations

      DOUBLE PRECISION      ET
      DOUBLE PRECISION      RECORD   ( * )
      DOUBLE PRECISION      STATE    ( 6 )

C$ Brief_I/O
C
C     Variable  I/O  Description
C     --------  ---  --------------------------------------------------
C     ET         I   Target epoch.
C     RECORD     I   Data record.
C     STATE      O   State (position and velocity).
C
C$ Detailed_Input
C
C     ET          is a target epoch, at which a state vector is to
C                 be computed.
C
C     RECORD      is a data record which, when evaluated at epoch ET,
C                 will give the state (position and velocity) of some
C                 body, relative to some center, in some inertial
C                 reference frame.
C
C$ Detailed_Output
C
C     STATE       is the state. Units are km and km/sec.
C
C$ Parameters
C
C     None.
C
C$ Exceptions
C
C     None.
C
C$ Files
C
C     None.
C
C$ Particulars
C
C     The exact format and structure of type 1 (difference lines)
C     segments are described in the required reading file SPK01.
C
C     Difference lines (DL's) are generated by JPL navigation
C     system programs P and PV. Each data record is equivelent
C     to the (slightly rearranged) 'P' portion of a NAVIO PV file
C     data record.
C
C     SPKE01 is a specialized version of Fred Krogh's subroutine DAINT.
C     Only the calling sequence has been changed.
C
C     Because the original version was undocumented, only Fred knows
C     knows how this really works.
C
C$ Examples
C
C     None.
C
C$ Restrictions
C
C     Unknown.
C
C$ Literature_References
C
C     NAIF Document 168.0, "S- and P- Kernel (SPK) Specification and
C     User's Guide"
C
C$ Author_and_Institution
C
C     F.T. Krogh      (JPL)
C     I.M. Underwood  (JPL)
C
C$ Version
C
C-    SPICELIB Version 1.0.1, 22-MAR-1990 (HAN)
C
C        Literature references added to the header.
C
C-    SPICELIB Version 1.0.0, 31-JAN-1990 (IMU) (FTK)
C
C-&


C
C     SPICELIB functions
C
      LOGICAL               RETURN

C
C     Local variables
C
C     The names below are original to the routine. They correspond
C     roughly to the original memos written by Fred Krogh to explain
C     how all this stuff really works.
C
      DOUBLE PRECISION TL
      DOUBLE PRECISION G        ( 15,1 )
      DOUBLE PRECISION YN       (    6 )
      DOUBLE PRECISION DT       ( 15,3 )
      INTEGER          KQMAX1
      INTEGER          KQ       (    3 )

      DOUBLE PRECISION FC       (   14 )
      SAVE             FC

      DOUBLE PRECISION R        (   17 )
      SAVE             R

      DOUBLE PRECISION SUM
      DOUBLE PRECISION HI
      DOUBLE PRECISION TP
      DOUBLE PRECISION WC       (   13 )
      DOUBLE PRECISION W        (   17 )

      INTEGER          MQ2
      INTEGER          K
      INTEGER          KS
      INTEGER          KQQ
      INTEGER          I
      INTEGER          J
      INTEGER          JX

C
C     Save everything between calls.
C
      SAVE

C
C     Initial values. The value of R  is just 1/R . No one knows why.
C                                   i            i
C
C     Eventually, they should be computed the first time through,
C     to get maximum precision.
C
      DATA  FC (  1 )    / 1.D0 /
      DATA  R  (  1 )    / 1.D0 / ,
     .      R  (  2 )    / .5D0 / ,
     .      R  (  3 )    / .3333333333333333333D+0 / ,
     .      R  (  4 )    / .2500000000000000000D+0 / ,
     .      R  (  5 )    / .2000000000000000000D+0 / ,
     .      R  (  6 )    / .1666666666666666667D+0 / ,
     .      R  (  7 )    / .1428571428571428571D+0 / ,
     .      R  (  8 )    / .1250000000000000000D+0 / ,
     .      R  (  9 )    / .1111111111111111111D+0 / ,
     .      R  ( 10 )    / .1000000000000000000D+0 / ,
     .      R  ( 11 )    / .9090909090909090909D-1 / ,
     .      R  ( 12 )    / .8333333333333333333D-1 / ,
     .      R  ( 13 )    / .7692307692307692308D-1 / ,
     .      R  ( 14 )    / .7142857142857142857D-1 / ,
     .      R  ( 15 )    / .6666666666666666667D-1 / ,
     .      R  ( 16 )    / .6250000000000000000D-1 / ,
     .      R  ( 17 )    / .5882352941176470588D-1 /



C
C     Standard SPICE error handling.
C
      IF ( RETURN () ) THEN
         RETURN
      ELSE
         CALL CHKIN ( 'SPKE01' )
      END IF

C
C     Unpack the contents of the MDA array.
C
C        Name    Dimension  Description
C        ------  ---------  -------------------------------
C        TL              1  Final epoch of record
C        G              15  Stepsize function vector
C        YN          2,NTE  State at TL
C        DT         15,NTE  Modified divided difference arrays
C        KQHX1           1  Maximum integration order plus 1
C        DKQ           NTE  Integration order array
C
C     For our purposes, NTE is always 3.
C
      CALL MOVED ( RECORD( 1),  1, TL )
      CALL MOVED ( RECORD( 2), 15, G  )
      CALL MOVED ( RECORD(17),  6, YN )
      CALL MOVED ( RECORD(23), 45, DT )

      KQMAX1 = INT ( RECORD(68) )
      KQ(1)  = INT ( RECORD(69) )
      KQ(2)  = INT ( RECORD(70) )
      KQ(3)  = INT ( RECORD(71) )



C
C     From here on, it's just as Fred left it. Frankly, we're
C     afraid to change it.
C

C
C     Compute the vector W of interpolation coeficients:
C
      HI = ET - TL
  190 TP = HI
      MQ2 = KQMAX1 - 2
      KS  = KQMAX1
      DO 200 K = 1, MQ2
      FC(K+1) = TP / G(K,1)
      TP = HI + G(K,1)
  200 WC(K) = HI / G(K,1)
      DO 210 K = 1, KS
  210 W(K) = R(K)
      JX =  0
  215 JX = JX+1
  220 K = KS
      DO 230 J = 1, JX
      W(K) = FC(J+1)*W(K-1) - WC(J)*W(K)
  230 K = K + 1
      KS = KS - 1
      IF(KS .GT. 2) GO TO 215

C
C     Perform interpolation:  KS = 2  (position)
C                             KS = 1  (velocity)
C
  295 J = 0
      DO 400 I = 1, 3
  305 J = J + 2
      KQQ = KQ(I)
      K = KQQ + KS
      SUM = 0.D0
  310 K = K - 1
      SUM = SUM + DT(KQQ,I)*W(K)
      KQQ = KQQ - 1
      IF(KQQ .NE. 0) GO TO 310
      IF(KS .EQ. 1) GO TO 370
  360 STATE(I) = YN(J-1) + HI*(YN(J) + HI*SUM)
      GO TO 400
  370 STATE(I+3) = YN(J) + HI*SUM
  400 CONTINUE
      IF (KS .GE. 2) GO TO 220

      CALL CHKOUT ( 'SPKE01' )
      RETURN
      END
