
C$Procedure SPKPV ( S/P Kernel, position and velocity )

      SUBROUTINE SPKPV ( HANDLE, DESCR, ET, REF, STATE, CENTER )

C$ Abstract
C
C     Return the state (position and velocity) of a target body
C     relative to some center of motion.
C
C$ Required_Reading
C
C     SPK
C
C$ Keywords
C
C     EPHEMERIS
C
C$ Declarations

      INTEGER               HANDLE
      DOUBLE PRECISION      DESCR    ( 5 )
      DOUBLE PRECISION      ET
      CHARACTER*(*)         REF
      DOUBLE PRECISION      STATE    ( 6 )
      INTEGER               CENTER

      INTEGER               MAXREC
      PARAMETER           ( MAXREC = 129 )

C$ Brief_I/O
C
C     Variable  I/O  Description
C     --------  ---  --------------------------------------------------
C     HANDLE     I   File handle.
C     DESCR      I   Segment descriptor.
C     ET         I   Target epoch.
C     REF        I   Target inertial reference frame.
C     STATE      O   Position, velocity.
C     CENTER     O   Center of state.
C     MAXREC     P   Maximum length of records returned by SPKRnn.
C
C$ Detailed_Input
C
C     HANDLE,
C     DESCR       are the file handle assigned to a SPK file, and the
C                 descriptor for a segment within the file. Together
C                 they determine the ephemeris data from which the
C                 state of the body is to be computed.
C
C     ET          is the epoch (ephemeris time) at which the state
C                 is to be computed.
C
C     REF         is the name of the inertial reference frame to
C                 which the vectors returned by the routine should
C                 be rotated. This may be any frame supported by
C                 the SPICELIB subroutine CHGIRF.
C
C$ Detailed_Output
C
C     STATE       contains the position and velocity, at epoch ET,
C                 for whatever body is covered by the specified segment.
C                 These vectors are rotated into the specified inertial
C                 reference frame, the origin of which is located at
C                 the center of motion for the body (see CENTER, below).
C                 Units are always km and km/sec.
C
C     CENTER      is the integer ID code of the center of motion for
C                 the state.
C
C$ Parameters
C
C     MAXREC      is the maximum length of a record returned by any of
C                 data type-specific routines SPKRnn, which are called
C                 by SPKPV (see Particulars).
C
C$ Files
C
C     See argument HANDLE.
C
C$ Exceptions
C
C     1) If the segment type is not supported by the current
C        version of SPKPV, the error 'SPICE(SPKTYPENOTSUPP)'
C        is signalled.
C
C     2) If the requested reference frame is not supported by the
C        current version of CHGIRF, the error 'SPICE(SPKREFNOTSUPP)'
C        is signalled.
C
C$ Particulars
C
C     SPKPV is the most basic of the SPK readers, the reader upon
C     which SPKSSB, SPKAPP, and SPKEZ are built. It should not normally
C     be called directly except in cases where some optimization is
C     required. (That is, where the calling program has prior knowledge
C     of the center-barycenter shifts to be performed, or a non-standard
C     method of determining the files and segments to be used when
C     computing states.)
C
C     This is the only reader which makes distinctions between the
C     various segment types in the SPK format. The complete list
C     of types currently supported is shown below.
C
C        Type   Description
C        ----   -----------------------
C           1   Difference Lines
C           2   Chebyshev (P)
C           3   Chebyshev (P,V)
C           4   Weighted elements
C
C     SPKPV is the only reader that needs to be changed in order to
C     add a new segment type to the SPK format.  If a new data type is
C     added, the following steps should be taken:
C
C     1) Write two new routines, SPKRnn and SPKEnn, to read and
C        evaluate, respectively, a record from a data type nn segment.
C
C     2) Insert a new case into the body of SPKPV to accomodate the new
C        type.
C
C     3) If necessary, adjust the parameter MAXREC, above, so that it
C        is large enough to encompass the maximum size of a record
C        returned by SPKRnn and passed to SPKEnn.
C
C        The maximum record lengths for each data type currently
C        supported are as follows:
C
C                  Data type       Maximum record length
C                  ---------       ---------------------
C                      1                    71
C                      2                    66
C                      3                   129
C                      4
C
C$ Examples
C
C     In the following code fragment, an entire SPK file is searched
C     for segments containing a particular epoch. For each one found,
C     the body, center, segment identifier, and range at the epoch
C     are printed out.
C
C        CALL DAFOPR ( 'TEST.SPK', HANDLE )
C        CALL DAFBFS (             HANDLE )
C
C        CALL DAFFNA ( FOUND  )
C
C        DO WHILE ( FOUND )
C           CALL DAFGS ( DESCR )
C           CALL DAFUS ( DESCR, 2, 6, DC, IC )
C
C           IF ( DC(1) .LE. ET  .AND.  ET .LE. DC(2) ) THEN
C              CALL SPKPV ( HANDLE, DESCR, ET, 'J2000', STATE, CENTER )
C              CALL DAFGN ( IDENT )
C
C              WRITE (*,*)
C              WRITE (*,*) 'Body   = ', IC(1)
C              WRITE (*,*) 'Center = ', CENTER,
C              WRITE (*,*) 'ID     = ', IDENT
C              WRITE (*,*) 'Range  = ', VNORM ( STATE )
C           END IF
C
C           CALL DAFFNA ( FOUND )
C        END DO
C
C$ Restrictions
C
C     None.
C
C$ Literature_References
C
C     NAIF Document 168.0, "S- and P- Kernel (SPK) Specification and
C     User's Guide"
C
C$ Author_and_Institution
C
C     R.E. Thurman    (JPL)
C     I.M. Underwood  (JPL)
C
C$ Version
C
C-    SPICELIB Version 1.0.1, 22-MAR-1990 (HAN)
C
C        Literature references added to the header.
C
C-    SPICELIB Version 1.0.0, 31-JAN-1990 (IMU) (RET)
C
C-&


C
C     SPICELIB functions
C
      LOGICAL               RETURN

C
C     Some local space is needed in which to return records, and
C     into which to unpack the segment descriptor.
C
      DOUBLE PRECISION      RECORD   ( MAXREC )

      DOUBLE PRECISION      DC       (      2 )
      INTEGER               IC       (      6 )
      INTEGER               IRF
      INTEGER               TYPE
      INTEGER               IRFREQ
      DOUBLE PRECISION      ROT      (    3,3 )



C
C     Standard SPICE error handling.
C
      IF ( RETURN () ) THEN
         RETURN
      ELSE
         CALL CHKIN ( 'SPKPV' )
      END IF

C
C     Unpacking the segment descriptor will tell us the center,
C     inertial reference frame, and data type for this segment.
C
      CALL DAFUS ( DESCR, 2, 6, DC, IC )

      CENTER = IC(2)
      IRF    = IC(3)
      TYPE   = IC(4)

C
C     Each data type has a pair of routines to read and evaluate
C     records for that data type. These routines are the only ones
C     that actually look inside the segments.
C
C     By the time we have more than 100 data types, we should be
C     allowed to use longer variable names.
C
      IF ( TYPE .EQ. 01 ) THEN
         CALL SPKR01 ( HANDLE, DESCR, ET, RECORD         )
         CALL SPKE01 (                ET, RECORD, STATE  )

      ELSE IF ( TYPE .EQ. 02 ) THEN
         CALL SPKR02 ( HANDLE, DESCR, ET, RECORD         )
         CALL SPKE02 (                ET, RECORD, STATE  )

      ELSE IF ( TYPE .EQ. 03 ) THEN
         CALL SPKR03 ( HANDLE, DESCR, ET, RECORD         )
         CALL SPKE03 (                ET, RECORD, STATE  )

C      ELSE IF ( TYPE .EQ. 04 ) THEN
C         CALL SPKR04 ( HANDLE, DESCR, ET, RECORD         )
C         CALL SPKE04 (                ET, RECORD, STATE  )

      ELSE
         CALL SETMSG ( 'SPK data type # not supported.' )
         CALL ERRINT ( '#', TYPE               )
         CALL SIGERR ( 'SPICE(SPKTYPENOTSUPP)' )
         CALL CHKOUT ( 'SPKPV'                 )
         RETURN
      END IF

C
C     Finally, rotate the raw state from its native frame to the
C     one requested by the user.
C
      CALL IRFNUM ( REF, IRFREQ )

      IF ( IRFREQ .EQ. 0 ) THEN
         CALL SETMSG ( 'No support for frame #.' )
         CALL ERRCH  ( '#', REF                  )
         CALL SIGERR ( 'SPICE(SPKREFNOTSUPP)'    )

      ELSE IF ( IRFREQ .NE. IRF ) THEN
         CALL IRFROT ( IRF, IRFREQ, ROT )

         CALL MXV ( ROT, STATE(1), STATE(1) )
         CALL MXV ( ROT, STATE(4), STATE(4) )
      END IF

      CALL CHKOUT ( 'SPKPV' )
      RETURN
      END
