C$Procedure      WRLINE ( Write Output Line to a Device )

      SUBROUTINE WRLINE ( DEVICE, LINE )

C$ Abstract
C
C     Write a character string to an output device.
C
C$ Required_Reading
C
C      None.
C
C$ Keywords
C
C      TEXT, FILES, ERROR
C
C$ Declarations

       CHARACTER*(*)          DEVICE
       CHARACTER*(*)          LINE

       INTEGER               FILEN
       PARAMETER           ( FILEN = 128)

C$ Brief_I/O
C
C      VARIABLE  I/O  DESCRIPTION
C      --------  ---  --------------------------------------------------
C      DEVICE     I   A string specifying the output device.
C      LINE       I   A line of text to be output.
C      FILEN      P   Maximum length of a file name.
C
C$ Detailed_Input
C
C
C      DEVICE  is the ouput device to which the line of text will
C              be written.
C
C              Possible values for DEVICE are:
C
C              1.    A device name.  This may be the name of a
C                    file, or any other name that is valid in
C                    a FORTRAN OPEN statement.  For example,
C                    on the VAX, a logical name may be used.
C
C                    Note that the device name must not
C                    be any of the reserved strings below.
C                    If the file is not open, it will be opened
C                    (if possible) as a formatted ASCII text file,
C                    and the line of text will be written to the file.
C                    If the file is already open, the line of text
C                    will be written to the file.  The effect is
C                    to append the line following the previous last
C                    line of text in the file.
C
C
C              2.    'SCREEN'    The output will go to the
C                     screen.  This is the default device.
C
C              3.    'NULL'      The data will not be output.
C
C              'SCREEN' and 'NULL' can be written in mixed
C              case.  For example, the following call will work:
C
C              CALL WRLINE ( 'screEn', LINE )
C
C
C$ Detailed_Output
C
C      No output arguments, but the character string LINE
C      is output to the device specified by DEVICE.
C
C$ Parameters
C
C      FILEN  is the maximum length of a file name.
C
C$ Exceptions
C
C      This routine is a special case as far as error handling
C      is concerned, because it is called to output error
C      messages resulting from errors detected by other routines.
C      In such a case, calling SIGERR would constitute recursion.
C      Therefore, this routine prints error messages rather
C      than signalling errors via SIGERR and setting the long
C      error message via SETMSG.
C
C      This routine detects the following error types:
C
C      1.  SPICE(TOOMANYFILESOPEN) -- The output device
C          specified by DEVICE cannot be opened because
C          the maximum number of opened devices has already
C          been reached (the maximum is a limitation of
C          this routine).
C
C      2.  SPICE(NOFREELOGICALUNIT) -- No logical unit number
C          is available to refer to the device.
C
C      3.  SPICE(FILEOPENFAILED)  -- General file open error.
C
C      4.  SPICE(FILEWRITEFAILED)  -- General file write error.
C
C$ Files
C
C      If DEVICE specifies a device other than 'SCREEN' or 'NULL',
C      that device is opened and written with a logical unit
C      determined at run time.  The write is performed with
C      list-directed format.
C
C
C      A maximum of 5 devices may be simultaneously open for output by
C      this routine.
C
C
C$ Particulars
C
C      Please read the "required reading"!
C
C      The write is performed with list-directed format.
C
C      This routine can't tell whether the name supplied
C      to indicate the output device is valid.  Be careful!
C
C$ Examples
C
C      1.  We output an error message to the file, SPUD.DAT (VAX
C          VMS file name syntax)
C
C
C
C
C                IF ( MASS .LE. 0 ) THEN
C
C          C
C          C        Write error message to SPUD.DAT:
C          C
C
C                   CALL WRLINE ( 'SPUD.DAT', 'Error, mass <= 0' )
C
C                END IF
C
C
C
C$ Restrictions
C
C      1.  File names must not exceed 128 characters.
C
C      2.  At most 5 different output devices can be open for writing
C          by this routine simultaneously.
C
C      3.  The output is written with list-directed format.
C
C$ Literature_References
C
C      None.
C
C$ Author_and_Institution
C
C      N.J. Bachman    (JPL)
C
C$ Version
C
C-    SPICELIB Version 1.0.0, 31-JAN-1990 (NJB)
C
C-&

C$ Revisions
C
C-    Beta Version 1.2.0, 27-FEB-1989 (NJB)
C
C        Call to GETLUN replaced by call to FNDLUN, which is error-free.
C        Call to IOERR replaced with in-line code to construct long
C        error message indicating file open failure. Arrangement of
C        declarations changed.  Keywords added. FILEN declaration
C        moved to "declarations" section.  Parameters section added.
C
C-    Beta Version 1.1.0, 06-OCT-1988 (NJB)
C
C        Upper bound of written substring changed to prevent use of
C        invalid substring bound.  Specifically, LASTNB ( LINE ) was
C        replaced by  MAX ( 1, LASTNB (LINE) ).  This upper bound
C        now used in the PRINT statement as well.
C
C-&


C
C     SPICELIB functions
C

      INTEGER               ISRCHC
      INTEGER               LASTNB

C
C     Local variables
C


      CHARACTER*(FILEN)     TMPNAM


      INTEGER               UNIT
      INTEGER               IOSTAT
      INTEGER               I
      INTEGER               J
      INTEGER               SHIFT

      CHARACTER*240         ERROR
      CHARACTER*11          ERRSTR

C
C     Saved variables
C

      INTEGER               MAXDEV
      PARAMETER           ( MAXDEV = 5 )

      CHARACTER*(FILEN)     DEVS    ( MAXDEV )
      INTEGER               UNITS   ( MAXDEV )
      CHARACTER*1           STATUS  ( MAXDEV )
      INTEGER               INDEX
      INTEGER               N


      SAVE                  DEVS
      SAVE                  UNITS
      SAVE                  STATUS
      SAVE                  INDEX
      SAVE                  N


C
C     Executable Code:
C


      IF ( DEVICE .EQ. ' ' ) THEN

         DEVICE = 'NULL'

      ELSE

C
C        We want to have an upper case, left justified copy of the
C        device name, in case it might be a reserved word:
C
C        Find first non-blank character of DEVICE and left shift
C        DEVICE by the number of leading blanks.
C
         I = 1

50001    IF       (   (  I .LT. LEN (DEVICE)   )       .AND.
     .
     .                (  DEVICE(I:I) .EQ. ' '  )
     .            )
     .   THEN

            I = I + 1

            GO TO 50001
         END IF

         I = I - 1

C
C        I is the number of blanks preceding the first non-blank
C        character (We know there is one).
C

         TMPNAM = ' '

         DO 50002
     .      J = 1,  MIN  (  LEN (DEVICE) - I,  FILEN   )

            TMPNAM (  J : J  )  =  DEVICE ( J + I : J + I )

50002    CONTINUE

      END IF

C
C     Convert TMPNAM to upper case:
C

      SHIFT = ICHAR ( 'A' )   -   ICHAR ( 'a' )

      DO 50003
     .    I = 1,  FILEN

         IF  (     LGE ( TMPNAM (I:I), 'a' )    .AND.
     .             LLE ( TMPNAM (I:I), 'z' )              )
     .   THEN


            TMPNAM (I:I)  =  CHAR (  ICHAR ( TMPNAM (I:I) ) + SHIFT  )


         END IF

50003 CONTINUE


C
C     TMPNAM is now left justified and is in upper case.
C


      IF   (  TMPNAM  .EQ. 'NULL'  ) THEN

         RETURN

      ELSE IF  (  TMPNAM  .EQ. 'SCREEN'  ) THEN

         PRINT *,  LINE ( : MAX ( 1, LASTNB(LINE) ) )

      ELSE


C
C        We expect that DEVICE is a device name...
C        First, see if the device has already been opened:
C

         INDEX = ISRCHC (  DEVICE,  N,  DEVS   )


         IF ( INDEX .EQ. 0 ) THEN

C
C        We will need a free logical unit. But only if we don't
C        have too many files open already.
C
            IF ( N .EQ. MAXDEV ) THEN

C
C              We can't signal the error, so just print the messages:
C

               PRINT *, 'SPICE(TOOMANYFILESOPEN)'
               PRINT *, ' '
               PRINT *, 'WRLINE: Maximum number of open devices'//
     .                  ' for writing by WRLINE has already'    //
     .                  ' been   reached'
C                              ^ these spaces are SUPPOSED to be here
               RETURN

            END IF

C
C           There is always the chance that no units are available.
C
            CALL FNDLUN ( UNIT )

            IF ( UNIT .LT. 1 ) THEN


               PRINT *, 'SPICE(NOFREELOGICALUNIT)'
               PRINT *, ' '
               PRINT *, 'WRLINE: Maximum number of logical units'     //
     .                  ' that can be allocated by SPICELIB has  '    //
     .                  ' already been reached'
               RETURN

            END IF

C
C           Okay, we have a unit. Open the file, and hope nothing goes
C           awry. (On the VAX, the qualifier CARRIAGECONTROL='LIST' may
C           be inserted into the OPEN statement.)
C
            OPEN ( FILE            = DEVICE,
     .             UNIT            = UNIT,
     .             STATUS          = 'NEW',
     .             IOSTAT          = IOSTAT  )

            IF ( IOSTAT .NE. 0 ) THEN

               PRINT *, 'SPICE(FILEOPENFAILED)'
               PRINT *, ' '

               ERROR = 'An error occurred while attempting to open'

               CALL SUFFIX ( DEVICE, 1, ERROR )
               CALL SUFFIX ( '.',    0, ERROR )
               CALL SUFFIX ( 'The value of IOSTAT returned was', 2,
     .                       ERROR  )
               CALL SUFFIX ( ':',    0, ERROR )
               CALL INTSTR ( IOSTAT, ERRSTR )
               CALL SUFFIX ( ERRSTR(:LASTNB(ERRSTR)), 1, ERROR )
               CALL SUFFIX ( '.',    0, ERROR )

               PRINT *, ERROR

               RETURN

            END IF

C
C           Whew! We're ready to write to this file. Save
C           the pertinent information. (Same as for RDTEXT.)
C
            N         = N + 1
            DEVS (N)  = DEVICE
            UNITS(N)  = UNIT
            STATUS(N) = 'W'
            INDEX     = N


         END IF



C
C        This is the easy part. Write the next line to the file.
C

         WRITE   ( UNITS(INDEX),  *,  IOSTAT=IOSTAT )
     .                    LINE    (   : MAX ( 1, LASTNB(LINE) )   )

C
C        Well, what happened? Any non-zero value for IOSTAT indicates
C        an error. Close the file immediately. Repack the UNITS, DEVS ,
C        and STATUS arrays, so that subsequent calls will not try to
C        write to the file without reopening it.
C
         IF ( IOSTAT .NE. 0 ) THEN

            CLOSE ( UNITS(INDEX) )

            DO 50004
     .         I = INDEX+1, N
               DEVS (I-1)  = DEVS (I)
               UNITS(I-1)  = UNITS(I)
               STATUS(I-1) = STATUS(I)
50004       CONTINUE

            N = N - 1


            ERROR = 'An error occurred while attempting to WRITE to'

            CALL SUFFIX ( DEVICE, 1, ERROR )
            CALL SUFFIX ( '.',    0, ERROR )
            CALL SUFFIX ( 'The value of IOSTAT returned was', 2,
     .                    ERROR  )
            CALL SUFFIX ( ':',    0, ERROR )
            CALL INTSTR ( IOSTAT, ERRSTR )
            CALL SUFFIX ( ERRSTR(:LASTNB(ERRSTR)), 1, ERROR )
            CALL SUFFIX ( '.',    0, ERROR )

            PRINT *, ERROR

            RETURN

         END IF

      END IF

      RETURN




C$Procedure  CLLINE ( Close a Device Opened by WRLINE )

      ENTRY  CLLINE ( DEVICE )

C$ Abstract
C
C      Close a device currently opened by WRLINE.
C
C$ Required_Reading
C
C      None.
C
C$ Keywords
C
C      TEXT, FILES, ERROR
C
C$ Declarations
C
C      CHARACTER*(*)        DEVICE
C
C$ Brief_I/O
C
C      VARIABLE  I/O  DESCRIPTION
C      --------  ---  --------------------------------------------------
C      DEVICE     I   Device to be closed.
C
C$ Detailed_Input
C
C      DEVICE         is the name of a device which is currently
C                     opened for reading or writing by WRLINE.
C
C                     DEVICE may refer to any device which can be
C                     opened by a FORTRAN open statement.
C
C$ Detailed_Output
C
C      None.
C
C$ Parameters
C
C      None.
C
C$ Exceptions
C
C      If DEVICE does not specify a device opened by CLLINE, this
C      routine does nothing.
C
C$ Files
C
C      This routine closes devices opened by WRLINE.  WRLINE and
C      CLLINE maintain a list of devices opened by WRLINE; any
C      device whose name is not in that list will not be closed by this
C      routine.
C
C$ Particulars
C
C      CLLINE closes one of the devices currently opened for
C      writing by WRLINE.  The information corresponding to that
C      device is deleted from the database of output devices
C      maintained by WRLINE and CLLINE.
C
C      If the specified device is not open, nothing happens.
C
C$ Examples
C
C      1.  We write two lines to the file, SPUD.DAT (VAX file name
C          syntax), and then close the file.
C
C          CALL WRLINE ( 'SPUD.DAT', ' This is line 1 ' )
C          CALL WRLINE ( 'SPUD.DAT', ' This is line 2 ' )
C
C          CALL CLLINE ( 'SPUD.DAT' )
C
C$ Restrictions
C
C      None.
C
C$ Literature_References
C
C      None.
C
C$ Author_and_Institution
C
C      N.J. Bachman    (JPL)
C
C$ Version
C
C-    SPICELIB Version 1.0.0, 31-JAN-1990 (NJB)
C
C-&

C$ Revisions
C
C-    Beta Version 1.0.1, 08-NOV-1988 (NJB)
C
C        Keywords added.
C-&


C
C     Which device?  Note that ISRCHC expects an exact match, so
C     we stuff DEVICE into TMPDEV.
C


      TMPNAM = DEVICE

      INDEX  = ISRCHC ( TMPNAM, N, DEVS  )

      IF ( INDEX .GT. 0 ) THEN

         CLOSE ( UNITS(INDEX) )

C
C        Remember all that salient information about the device?
C        Lose it.
C
         DO 50005
     .      I = INDEX+1, N
            DEVS (I-1)  = DEVS (I)
            UNITS(I-1)  = UNITS(I)
            STATUS(I-1) = STATUS(I)
50005    CONTINUE

         N = N - 1

      END IF

      RETURN


      END


