C========================================================================
C radiometer_process.f
C	contains 4 subroutines radiometer_process, temp_sensor,
C	angle_n_height, rcvr_gn_crr2
C------------------------------------------------------------------------
	Subroutine Radiometer_Process( data, calib, burst_count, type,
     .					handle )
C
C Abstract:
C
C***************************************************************************
C	Passed inputs:
C
C		data: radiometer signal from antenna, data extracted
C		     from burst header b245-256, b79= 1 in SAR status field
C
C		calib: radiometer signal from calibration source, data
C		     extracted from burst header b245-256, b79= 0 in SAR status
C		     field
C
C		burst_count:  burst counter for accessing radiom data
C
C		type:  radiometer (1) or cold sky (2) burst.
C
C		handle:  used by NAIF software, no one knows what it is...
C
C	RADIOM inputs:
C
C		Gr_byte:  74~76: receiver gain code from SAR status field
C		T_sample:  Time of radiometer data, b269~320 of burst header
C		RXU_A_STATE:  receiver unit indicator from R-2106(A)
C		ONU_A_STATE:  output network unit indicator from R-2108(A)
C		T_op:  Physical temperature of the active receiver unit
C		       from telemetry channel R-0670 (1670)for RXU A (B)
C		Cable temps:  T_cable1,T_cable2,T_cable3,T_cable4,T_cable5
C
C	VAR_LOCAL inputs:
C
C		Nav_source
C		dqm:  Delta quaternians from channels E-1028~1030
C		      from telemetry channels E-0041,E-0042,E-0043,E-0044,E-0045
C		t_p:  time of periapsis
C		dmp:  Mapping quaternion polynomials
C		Rv:  radius of Venus
C
C	Models inputs:
C
C		Ephemeris File, Quaternian polynomial coefficient file
C
C	VAR_LOCAL outputs:
C		T_sample:  Time of radiometer data, b269~320 of burst header
C		qm:  quaternions
C		lat_ra:  latitude
C		lon_ra:  longitude
C		anglk_ra:  look angle
C		height:  height
C		x_s:  s/c location
C		Gr:  gain factor used
C		T-op:
C		coef_a, coef_b, coef_c
C		T_si
C		T_cable1, T_cable2, T_cable3, T_cable4, T_cable5
C		T_emission
C		Kappa
C		signal
C		calibration
C		T_antenna_obs
C		T_Surface
C
C***************************************************************************
C
C	Iprocess: Processing mode. When Iprocess=0, it is the normal mode
C		  to derive the surface temperature. When Iprocess=1, it is
C		  the code sky calibration mode to get Sky temp.
C
C************************* Burst time **************************************
C	1. Obtain burst time tag from burst (m-1) and convert to
C	   burst cycle start time tcs[m-1]
C	   tcs=(RIM*910+MOD91*10+MOD10+UPDATE/210+1)RTI
C	   where RIM= bit 281-304 of radar burst header
C	   where MOD90=bit 305-312 of radar burst header
C	   where MOD10=bit 313-320 of radar burst header
C	   where UPDATE= bit 269-280 of radar burst header

C	2. Compute relative burst time of Radiometer mode, dtRAD[m-1] from
C	   above, using SAR status Field parameters from the (m-1)th burst.
C	   dtRAD=(4893+151(Nao+32+512-ACR))/RESCLK, where
C	   Nao=integer((512-PC)(2048-NP+256-SCR+1)-1)/151
C	   where PC=SAR PRF code, bit 10-16 in SAR status field
C	   where NP=# of SAR pulse code, bit 33-43 in SAR status field
C	   where SCR=SAR coarse resolution code, bit 17-24 in SAR status field
C	   where ACR=Alt coarse resolution code, bit 44-52 in SAR status field
C	   RESCLK=2.258523 MHz

C	3. The radiometer data contained in burst m was acquired at a time
C	   given by: tm=tcs[m-1] +dtRAD[m-1]
C****************************************************************************

C  This routine processes radiometer data.
C  The first step is to compute the sensor input noise temperature by a
C  second order polynomial as a function of signal strength
C  (=antenna signal - calibration). The second step is to obtain the antenna
C  temperature by subtracting off the losses due to the cables.
C  The third step is to obtain the surface temperature by
C  	T_Surface=(T_Antenna_obs - T_Emision)/Kappa
C	where Temission and Kappa are given by Tables
C  Kappa_table(10*Mu,R_planet) and T_Emission_table(10*Mu,R_planet)

C****** When bit 79 in SAR status field of the m-th radar burst is equal to
C****** 1, it indicates that signal is from antenna. Ant is then given by

C  signal =(((((((((( B245 *2 + B246)*2 + B247)*2 +B248)*2 + B249)*2 + B250)*2
C    1                +B251)*2 + B252)*2 + B253)*2 +B254)*2 + B255)*2 + B256

C****** where Bj= binary value of bit j in the m-th radar burst
C****** When bit 79 in SAR status field of the m+1-th radar burst is equal to
C****** 1, it indicates that signal is from calibration. Cal is then given by

Ccalibration=(((((((((( C245 *2 + C246)*2 + C247)*2 +C248)*2 + C249)*2 + C250)*2
C     1                +C251)*2 + C252)*2 + C253)*2 +C254)*2 + C255)*2 + C256

C ***** Get the signal strength by taking the difference between signal from
C ***** antenna and signal from calibration source ****************

	implicit none

C Declarations:
C	Passing Parameters: 

	real data, calib
	integer burst_count, type, handle

C	Constant Parameters: None.
C	Include files: None.

	include '../../../inc/var_local'
	include '../../../inc/radiom'

C
C Brief I/O:
C	Variable	I/O		Description
C       --------	---		------------
C	data		I		data value
C	calib		I		calibration value
C	burst_count	I		burst counter
C	type		I		radiometer or cold sky
C	handle		I		S/P kernel file handler
C
C Constant Parameters:
C	None.
C
C Common Blocks:
C	Name		I/O		Description
C       ----		---		------------
C	var_local	I/O		local variable common block
C	radiom		I		radiometer parameters
C
C Detailed I/O through common blocks:
C
C	Variable	I/O		Description
C       --------	---		------------
C	see above.
C
C Files:
C	None.
C
C Author and Institution:
C	Cynthia L. Wong (JPL).
C
C References:
C	Source is obtainted from MGN/SDPS/PSP preprocessing software on VAX.
C
C Revised Information:
C	PPLIB 05-01-1992 (CLW) - To be compatible to Alliant FX/2800.
C
C========================================================================
C Local Variables:

	BYTE		gr_byte

	REAL*4	loss_lo( 5 ), loss_hi( 5 )
	DATA loss_lo / 1.030, 1.038, 1.043, 1.029, 1.017 /
	DATA loss_hi / 1.035, 1.045, 1.048, 1.032, 1.020 /

	REAL*4	kutoff / 303.0 /

	integer cold_sky, iostat, i
	real s, r_sv

C Start:

	if(type.eq.1) cold_sky = 0
	if(type.eq.2) cold_sky = 1

	T_sample = radiom_rad_time(burst_count)
	RXU_A_STATE = radiom_RXU_STATE(burst_count)
	ONU_A_STATE = radiom_ONU_STATE(burst_count)
	T_OP = radiom_receiver_temp(burst_count)
	T_cable1 = radiom_cable_temp(1,burst_count)
	T_cable2 = radiom_cable_temp(2,burst_count)
	T_cable3 = radiom_cable_temp(3,burst_count)
	T_cable4 = radiom_cable_temp(4,burst_count)
	T_cable5 = radiom_cable_temp(5,burst_count)

C	signal = data
C	calibration = calib
C
C	S = signal - calibration

	s = data - calib

	Gr_byte = radiom_rad_gain(burst_count)
	CALL rcvr_gn_crr2( Gr_byte, RXU_A_STATE, Gr ) ! Receiver gain change

	S = S / Gr		! Compensate for the receiver gain change

D	TYPE *, 'S = ', S

C ***** Get the sensor input noise temperature by a polynomial of S

	CALL Temp_Sensor( S )	!Input S, output T_si in common block

D	TYPE *, 'T_si = ', T_si

C *****	Determine cable loss base on cable temperature ***************

	IF ( t_cable1 .LE. kutoff ) THEN
		cable1_loss = loss_lo( 1 )
	ELSE
		cable1_loss = loss_hi( 1 )
	END IF

	IF ( t_cable2 .LE. kutoff ) THEN
		cable2_loss = loss_lo( 2 )
	ELSE
		cable2_loss = loss_hi( 2 )
	END IF

	IF ( t_cable3 .LE. kutoff ) THEN
		cable3_loss = loss_lo( 3 )
	ELSE
		cable3_loss = loss_hi( 3 )
	END IF

	IF ( t_cable4 .LE. kutoff ) THEN
		cable4_loss = loss_lo( 4 )
	ELSE
		cable4_loss = loss_hi( 4 )
	END IF

	IF ( t_cable5 .LE. kutoff ) THEN
		cable5_loss = loss_lo( 5 )
	ELSE
		cable5_loss = loss_hi( 5 )
	END IF

C *****	Obtain antenna temperture based on Sensor temperature and cable loss

	T_antenna_obs = T_si*cable1_loss*cable2_loss*
     1                       cable3_loss*cable4_loss*cable5_loss
     2	- T_cable1 * (cable1_loss - 1)
     3	- T_cable2 * (cable2_loss - 1) * cable1_loss
     4	- T_cable3 * (cable3_loss - 1) * cable1_loss * cable2_loss
     5	- T_cable4 * (cable4_loss - 1) * cable1_loss * cable2_loss
     6                                 * cable3_loss
     7	- T_cable5 * (cable5_loss - 1) * cable1_loss * cable2_loss
     8                                 * cable3_loss * cable4_loss

	CALL angle_n_height( R_sv, handle ) !Get look angle, elevation of BIP

	IF ( cold_sky .EQ. 1 ) THEN
		T_cold_sky = T_antenna_obs
		T_surface = 0.0
	ELSE

C******* R_sv is the distance from spacecraft to the center of Venus *******

		R_planet = (Rv + height) / 1000.0
		I_Mu = 10.0 * R_sv / Rv * SIND( anglk_ra )

		T_Emission = T_Emission_table( I_Mu, R_planet )

    		Kappa = Kappa_table( I_Mu, R_Planet )

D		type *, 'R_planet = ', R_planet
D		type *, 'I_mu = ', I_mu
D		type *, 'T_emission = ', T_emission
D		type *, 'T_antenna_obs = ', T_antenna_obs
D		type *, 'Kappa = ', Kappa

C *****	Obtain surface temperture based on antenna temperature, emission
C ***** temperature, and transmittance constant Kappa.


		T_Surface = (T_antenna_obs - T_cold_sky -
     .                       T_Emission)/Kappa

D		type *, 'T_surface = ', T_surface

	END IF

	RETURN


C	Error Messages

 99	type *,'ERROR!  ERROR!  ERROR!  I/O status =',IOstat

	RETURN
	END


C========================================================================

	Subroutine Temp_Sensor( s )

	implicit none

	include '../../../inc/var_local'

	integer iset, i, imax, n1, n2
	real slope0, slope1, slope2, s

	real coef0(4,4), coef1(4,4), coef2(4,4), temp(4,4)
	data coef0 /242.20, 247.83, 238.99, 235.38,
     1		    257.59, 260.20, 250.66, 245.24,
     2	            271.62, 277.68, 261.07, 249.66,
     3	            282.71, 285.28, 261.17, 249.51/

	data coef1 /.7849, .7441, .8405, .8685,
     1		    .7668, .7300, .8396, .8501,
     2	            .7838, .7436, .8354, .8988,
     3	            .8107, .8069, .8939, .9579/

	data coef2 /3.108e-4, 2.733e-4, 2.736e-4, 3.145e-4,
     1		    2.256e-4, 2.239e-4, 2.321e-4, 2.340e-4,
     2	            1.791e-4, 1.831e-4, 1.845e-4, 1.910e-4,
     3	            1.786e-4, 1.802e-4, 1.746e-4, 1.813e-4/

	data temp  / 3,  5,  3,  2,
     1		    21, 20, 19, 19,
     2	            35, 36, 36, 36,
     3	            49, 50, 49, 51/


	REAL*4	k2c / 273.16 /		!  JG  08/28/90
	REAL*4	Top_k

C Start:

D	TYPE *, '####@@@*****###### S = ', S

	if(RXU_A_state.eq.1 .and. ONU_A_state.eq.1)  iset=1

	if(RXU_A_state.eq.1 .and. ONU_A_state.ne.1)  iset=2

	if(RXU_A_state.ne.1 .and. ONU_A_state.eq.1)  iset=3

	if(RXU_A_state.ne.1 .and. ONU_A_state.ne.1)  iset=4

D	TYPE *, '####@@@*****###### ISET = ', ISET


	Top_k = T_op - k2c
	DO i = 1, 4
		IF ( Top_k .LE. temp( iset, i ) ) THEN

			imax = i
			GO TO 100
		END IF
	END DO

	imax = 4

100	IF ( imax .EQ. 1 ) THEN
		n1 = imax
		n2 = imax + 1
	ELSE
		n1 = imax
		n2 = imax - 1
	END IF

D	TYPE *, '####@@@*****###### N1 = ', N1
D	TYPE *, '####@@@*****###### N2 = ', N2

	slope0 = (coef0( iset, n2 ) - coef0( iset, n1 ))
	2      / (temp( iset, n2 ) - temp( iset, n1 ))

	coef_c = coef0( iset, n1 ) + slope0 *
     .           (Top_k - temp( iset, n1 ))

	slope1 = (coef1( iset, n2 ) - coef1( iset, n1 ))
	2      / (temp( iset, n2 ) - temp( iset, n1 ))

	coef_b = coef1( iset, n1 ) + slope1 *
     .           (Top_k - temp( iset, n1 ))

	slope2 = (coef2( iset, n2 ) - coef2( iset, n1 ))
	2      / (temp( iset, n2 ) - temp( iset, n1 ))

	coef_a = coef2( iset, n1 ) + slope2 *
     .           (Top_k - temp( iset, n1 ))

D	TYPE *, '####@@@*****###### COEF_A = ', COEF_A
D	TYPE *, '####@@@*****###### COEF_B = ', COEF_B
D	TYPE *, '####@@@*****###### COEF_C = ', COEF_C

C	Reverse the order of coefficients:

	T_si = coef_c + coef_b * s + coef_a * s * s

	RETURN
	END

C======================================================================

	Subroutine angle_n_height(R_sv, handle)

	implicit none

	include '../../../inc/var_local'
	real r_sv
	integer handle

	integer i, l
	real t_in, hh, vv, r_sb, r_bv

	call sec_2_day(T_sample,day)

	call navs(nav_source,T_sample,t_p,x_s_j2000,v_s_j2000,
     .            A_s_j2000, handle)
	call j2000_to_vme(x_s_j2000,x_s_VME)
	call vme_to_vbf(DAY,x_s_VME,x_s)

	call j2000_to_vme(v_s_j2000,v_s_VME)
	call vme_to_vbf(DAY,v_s_VME,v_s)


C	T_in = T_sample/tsp_norm

	T_in = (T_sample - t_p) / tsp_norm


D	TYPE *, 'x_s =   ', x_s
D	TYPE *, 'T_in = ', t_in
Cd	TYPE *, 'dmp =  ', dmp
D	TYPE *, 'dqm =  ', dqm

	call quaternion(dmp,T_in,dqm,q,qm,tsup)

D	TYPE *, 'qm =   ', qm
	call mgn_axs(day,q,x_mgn_j2000, y_mgn_j2000, z_mgn_j2000)
	call j2000_to_VME(z_mgn_j2000,z_mgn_vme) !spacecraft vectors in VME

	call VME_to_VBF(day,z_mgn_vme,z_mgn_vbf) !spacecraft vectors in VBF

	call normal(z_mgn_vbf)

	do i=1,3
	p(i)=z_mgn_vbf(i)
	end do

	R_sv=sqrt(x_s(1)**2+x_s(2)**2+x_s(3)**2)
	anglk_ra = acosd((-p(1)*x_s(1)-p(2)*x_s(2)-p(3)*x_s(3))/R_sv)
Ccc	anglk_ra = ang_lk

	height = 0.

D	TYPE *, 'anglk_ra = ', anglk_ra

	do l = 1, 3

	R_sb = R_sv*cosd(anglk_ra) -
     c	sqrt((Rv+height)**2-R_sv**2*sind(anglk_ra)**2)

	call unit_vector(irl,x_s,v_s,p,pg,r,c,d)

	hh = R_sb * sind(anglk_ra)
	vv = R_sv - R_sb * cosd(anglk_ra)

	x_b(1) = hh * pg(1) + vv * r(1)
	x_b(2) = hh * pg(2) + vv * r(2)
	x_b(3) = hh * pg(3) + vv * r(3)

C    	write (*,*)
C       write (*,*) 'x_b=', x_b
C       write (*,*) 'hh=', hh, 'vv=',vv

	call vbf_to_sph(x_b(1),x_b(2),x_b(3),R_bv,lat_ra,lon_ra)

	call elevation(lat_ra,lon_ra,height)

	end do

	return
	end

C======================================================================

	subroutine rcvr_gn_crr2(Gr_byte,RXU_A_STATE,Gr)

	implicit none

	byte Gr_byte, RXU_A_STATE
	integer icode
	real Gr, dca, dc_rx
        real dCA_A( 0:7 ), dCA_B( 0:7 )

C	real dCA_A(8), dCA_B(8)


        data dCA_A /0.,3.97,8.2,12.17,16.22,20.19,24.42,28.39/
        data dCA_B /0.,3.94,8.08,12.02,16.07,20.01,24.15,28.09/


C	b74=0
C	b75=0
C	b76=0

C	Icode=1+4*b74+2*b75+b76

        Icode = Gr_byte

	IF ( icode .LT. 0 .OR. icode .GT. 7 ) THEN
		TYPE *, 'RCVR_GN_CRR2:'
		TYPE *, '   Icode = ', icode
		TYPE *, '   gr = ', gr
		RETURN
	END IF


        if (RXU_A_state.eq.1) then
        dCA=dCA_A(Icode)
        else
        dCA=dCA_B(Icode)
        end if

        dC_RX=dCA		!dC_RX is the receiver gain compensation
				!factor of the receiver attenuation factor.
        dC_RX=10**(dC_RX/10.)
        Gr=1./dC_RX		!Gr is the receiver gain factor.

	return
	end
